/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <stdio.h>
#include <aul.h>
#include <Ecore_X.h>
#include <ui-gadget.h>
#include <app.h>

#include "phone.h"
#include "ct-detail.h"
#include "ct-list.h"

#if !defined(CV_PACKAGE)
#  define CV_PACKAGE "contacts-viewer"
#endif

typedef struct {
	char *path;
	Evas_Object *win;
	Evas_Object *content;
	Evas_Object *popup;
	Eina_List *list;
	ph_progress_info *p_info;
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
	Evas_Object *icon;
	Evas_Object *prev_btn;
#endif
	int current_rotate;
} ct_viewer_data;

static void cv_win_del(void *data, Evas_Object *obj, void *event)
{
	elm_exit();
}

static Evas_Object* cv_create_win(const char *name)
{
	Evas_Object *eo;
	int w, h;

	eo = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (eo) {
		elm_win_title_set(eo, name);
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
#else
		elm_win_borderless_set(eo, EINA_TRUE);
#endif
		evas_object_smart_callback_add(eo, "delete,request", cv_win_del, NULL);
		ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, &h);
		evas_object_resize(eo, w, h);
		elm_win_indicator_mode_set(eo, ELM_WIN_INDICATOR_SHOW);
	}
	elm_win_conformant_set(eo, EINA_TRUE);
	return eo;
}

#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
static Evas_Object* __set_win_icon(ct_viewer_data *ad)
{
	Evas_Object *image = evas_object_image_add(evas_object_evas_get(ad->win));
	evas_object_image_file_set(image, "/usr/share/icons/default/small/org.tizen.contacts-viewer.png", NULL);
	elm_win_icon_object_set(ad->win, image);

	return image;
}

static void __win_profile_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	if(data == NULL)
		return;

	ct_viewer_data *ad = data;
	Evas_Object *navi = evas_object_data_get(ad->win, "naviframe");
	const char *profile = elm_config_profile_get();
	if(!strcmp(profile, "desktop")) {
		Elm_Object_Item *navi_item = elm_naviframe_bottom_item_get(navi);
		ad->prev_btn = elm_object_item_part_content_unset(navi_item, "prev_btn");
		elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_HIDE);
		if(!ad->icon)
			ad->icon = __set_win_icon(ad);
	}
	else {
		Elm_Object_Item *navi_item = elm_naviframe_bottom_item_get(navi);
		if (ad->prev_btn) {
			elm_object_item_part_content_set(navi_item, "prev_btn", ad->prev_btn);
			ad->prev_btn = NULL;
		}
		elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	}
}
#endif // _PHONE_CONTACTS_DESKTOP_MODE_

static void __viewer_rotate_cb(void *data, Evas_Object *obj, void *event)
{
	ct_viewer_data *cviewer_d = data;
	int changed_rotate = elm_win_rotation_get(obj);

#ifndef _PHONE_CONTACTS_DESKTOP_MODE_
	const char *config = elm_config_profile_get();
	if (!strcmp(config, "desktop"))
		return;
#endif
	p_retm_if(cviewer_d == NULL || cviewer_d->win == NULL, "Paramter is NULL");

	if (changed_rotate != cviewer_d->current_rotate) {
		ERR("rotate changed %d", changed_rotate);
		cviewer_d->current_rotate = changed_rotate;
		switch (cviewer_d->current_rotate) {
		case 0:
			ug_send_event(UG_EVENT_ROTATE_PORTRAIT);
			break;
		case 180:
			ug_send_event(UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN);
			break;
		case 270:
			ug_send_event(UG_EVENT_ROTATE_LANDSCAPE);
			break;
		case 90:
			ug_send_event(UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN);
			break;
		default:
			break;
		}
	}
}



static bool viewer_create(void *data)
{
	int rotate;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_viewer_data *cviewer_d = data;
	Evas_Object *win;

	err = contacts_connect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_connect2() Failed(%d)", err);

	/* create window */
	win = cv_create_win(CV_PACKAGE);
	p_retvm_if(NULL == win, false, "ctapp_create_win() Failed");
	cviewer_d->win = win;
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
	evas_object_smart_callback_add(cviewer_d->win, "profile,changed", __win_profile_changed_cb, cviewer_d);
#endif
	bindtextdomain(PACKAGE, LOCALEDIR);
	evas_object_show(win);

	if (elm_win_wm_rotation_supported_get(win)) {
		int rotation[4] = {0, 90, 180, 270};
		elm_win_wm_rotation_available_rotations_set(win, (const int *)(&rotation), 4);
		cviewer_d->current_rotate = elm_win_rotation_get(win);
		ERR("rotate current %d", cviewer_d->current_rotate);
		evas_object_smart_callback_add(win, "wm,rotation,changed", __viewer_rotate_cb, cviewer_d);
	}

	UG_INIT_EFL(cviewer_d->win, UG_OPT_INDICATOR_PORTRAIT_ONLY);

	return true;
}

static void viewer_terminate(void *data)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_viewer_data *cviewer_d = data;

	if (cviewer_d->p_info) {
		cviewer_d->p_info->alive = false;
		pthread_join(cviewer_d->p_info->thread, NULL);
		ecore_timer_del(cviewer_d->p_info->timer);
		free(cviewer_d->p_info);
	}

	free(cviewer_d->path);

	if (cviewer_d->win) {
		evas_object_data_del(cviewer_d->win, "naviframe");
		evas_object_del(cviewer_d->win);
		cviewer_d->win = NULL;
	}

	err = contacts_disconnect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);

	return;
}

static void viewer_pause(void *data)
{
	return;
}

static void viewer_resume(void *data)
{
	return;
}


static bool viewer_load_vcard_foreach_cb(contacts_record_h record, void *data)
{
	ct_person_list *person_info = NULL;
	ct_viewer_data *cviewer_d = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	if (NULL == cviewer_d) {
		ERR("Invalid parameter(cviewer_d is NULL)");
		return false;
	}

	if (false == cviewer_d->p_info->alive)
		return false;

	person_info = calloc(1, sizeof(ct_person_list));
	if (NULL == person_info) {
		ERR("calloc() return NULL");
		cviewer_d->p_info->result = FALSE;
		if (cviewer_d->p_info->cnt == cviewer_d->p_info->cnt_checked_total)
			cviewer_d->p_info->completed = true;
		return false;
	}

	person_info->id = cviewer_d->p_info->cnt;
	err = contacts_record_get_str(record, _contacts_contact.display_name, &(person_info->display));
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);
	person_info->img_loaded = true;

	cviewer_d->list = eina_list_append(cviewer_d->list, person_info);
	++cviewer_d->p_info->cnt;
	return true;

}

static PTHREAD_FN viewer_load_vcard_contacts_thread_fn(void* data)
{
	contacts_connect_on_thread();

	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_viewer_data *cviewer_d  = data;

	err = contacts_vcard_parse_to_contact_foreach(cviewer_d->path, viewer_load_vcard_foreach_cb, cviewer_d);
	if (CONTACTS_ERROR_NONE != err) {
		cviewer_d->p_info->result = FALSE;
		ERR("contacts_svc_vcard_foreach() Failed(%d)", err);
	}
	cviewer_d->p_info->completed = true;

	contacts_disconnect_on_thread();

	pthread_exit(NULL);
}

static inline void viewer_get_filename(char *fullpath, char *dest, int size_dest)
{
	char* index;
	index = strrchr(fullpath, '/');
	snprintf(dest, size_dest, "%s", index + 1);
}

static Eina_Bool viewer_load_vcard_timer_cb(void *data)
{
	ct_viewer_data *cviewer_d = data;

	p_retvm_if(NULL == cviewer_d , ECORE_CALLBACK_CANCEL, "parameter(ct_viewer_data) is NULL");
	p_retvm_if(NULL == cviewer_d->p_info , ECORE_CALLBACK_CANCEL, "parameter(ct_viewer_data->p_info) is NULL");

	if (!cviewer_d->p_info->completed) {
		char count[PH_TEXT_SHORT_LEN];
		char percent[PH_TEXT_SHORT_LEN];
		double value = (double)cviewer_d->p_info->cnt / (double)cviewer_d->p_info->cnt_checked_total;

		if (!cviewer_d->p_info->alive) {
			cviewer_d->p_info->result = FALSE;
			cviewer_d->p_info->completed = true;
			return ECORE_CALLBACK_RENEW;
		}

		elm_progressbar_value_set(cviewer_d->p_info->progressbar, value);
		snprintf(percent, sizeof(percent), "%d%%", (int)(100.0 * (double)cviewer_d->p_info->cnt/(double)cviewer_d->p_info->cnt_checked_total));
		snprintf(count, sizeof(count), "%d/%d", cviewer_d->p_info->cnt, cviewer_d->p_info->cnt_checked_total);
		edje_object_part_text_set(elm_layout_edje_get(cviewer_d->p_info->layout), "elm.text.subtext1", percent);
		edje_object_part_text_set(elm_layout_edje_get(cviewer_d->p_info->layout), "elm.text.subtext2", count);

		if (cviewer_d->p_info->cnt == cviewer_d->p_info->cnt_checked_total)
			cviewer_d->p_info->completed = true;

		return ECORE_CALLBACK_RENEW;
	}
	else {
		pthread_join(cviewer_d->p_info->thread, NULL);

		evas_object_del(cviewer_d->popup);
		cviewer_d->popup = NULL;

		cviewer_d->p_info->timer = NULL;

		if (!cviewer_d->p_info->result) {
			free(cviewer_d->p_info);
			cviewer_d->p_info= NULL;
			elm_exit();
			return ECORE_CALLBACK_CANCEL;
		}

		cviewer_d->content = ctui_create_vcard_list_view(cviewer_d->win, cviewer_d->list, cviewer_d->path);
		p_retvm_if(NULL == cviewer_d->content, ECORE_CALLBACK_CANCEL, "ctui_create_vcard_list_view() return NULL");

		free(cviewer_d->p_info);
		cviewer_d->p_info = NULL;

		return ECORE_CALLBACK_CANCEL;
	}
}

static int viewer_load_vcard_contact(ct_viewer_data *cviewer_d)
{
	Evas_Object *popup;

	if (cviewer_d->path && *cviewer_d->path) {
		cviewer_d->content = ctui_contacts_viewer(cviewer_d->win, NULL, 0, cviewer_d->path);
		if (NULL == cviewer_d->content) {
			ERR("ctui_contacts_viewer() return NULL");
			elm_exit();
			return -1;
		}
	}
	else {
		popup = phone_show_popup(cviewer_d->win, S_(CT_SYS_POP_ERROR), 1.5);
		evas_object_smart_callback_add(popup, "timeout", cv_win_del, NULL);
	}
	return 0;
}

static int viewer_load_vcard_contacts(ct_viewer_data *cviewer_d, int cnt_vcard)
{
	int ret;
	char title[PH_TEXT_MAX_LEN];
	char filepath[PH_TEXT_MAX_LEN];
	ph_progress_info *p_info;

	p_info = calloc(1, sizeof(ph_progress_info));
	if (NULL == p_info) {
		ERR("calloc() return NULL");
		elm_exit();
		return -1;
	}

	cviewer_d->p_info = p_info;
	cviewer_d->list = NULL;

	p_info->cnt = 0;
	p_info->cnt_checked_total = cnt_vcard;
	p_info->alive = true;
	p_info->result = TRUE;
	p_info->completed = false;

	viewer_get_filename(cviewer_d->path, filepath, sizeof(filepath));
	snprintf(title, sizeof(title), "%s<br>%s", T_(CT_GET_TEXT_BASIC, CTTEXT_LOADING_VCARD), filepath);

	ret = pthread_create(&p_info->thread, NULL, viewer_load_vcard_contacts_thread_fn, cviewer_d);
	if (0 != ret) {
		ERR("Thread creation failed(%d)", ret);
		free(p_info);
		cviewer_d->p_info = NULL;
		elm_exit();
		return -1;
	}
	p_info->timer = ecore_timer_add(0.2, viewer_load_vcard_timer_cb, cviewer_d);
	if (NULL == p_info->timer) {
		ERR("ecore_timer_add() return NULL");

		p_info->alive = false;
		pthread_join(p_info->thread, NULL);

		free(p_info);
		cviewer_d->p_info = NULL;
		elm_exit();
		return -1;
	}
	cviewer_d->popup = phone_progressbar_popup(cviewer_d->win, p_info, title);
	return 0;
}

static void viewer_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_exit();
}

static bool __viewer_is_valid_person(int person_id)
{
	contacts_error_e err;
	contacts_record_h record = NULL;
	err = contacts_db_get_record(_contacts_person._uri, person_id, &record);
	if (CONTACTS_ERROR_NONE != err) {
		contacts_record_destroy(record, true);
		return false;
	}
	contacts_record_destroy(record, true);
	return true;
}

static void viewer_service(service_h service, void *data)
{
	int ret;
	int person_id;
	char *val = NULL;
	char val_vcard[PH_TEXT_MAX_LEN];
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_viewer_data *cviewer_d = data;

	if (NULL == cviewer_d->win) {
		ERR("win is NULL");
		elm_exit();
		return;
	}

	if (cviewer_d->p_info) {
		cviewer_d->p_info->alive = false;
		pthread_join(cviewer_d->p_info->thread, NULL);
		ecore_timer_del(cviewer_d->p_info->timer);
		free(cviewer_d->p_info);
		cviewer_d->p_info = NULL;
	}

	if (cviewer_d->content) {
		evas_object_del(cviewer_d->content);
		cviewer_d->content = NULL;
	}

	ret = service_get_extra_data(service, "person_id", &val);
	p_warn_if(SERVICE_ERROR_NONE != ret, "service_get_operation is failed(%d)", ret);

	if (val && *val) {
		person_id = atoi(val);
		if (person_id <= 0) {
			ERR("service value(person_id) is wrong(%d)", person_id);
		}
		else if(false == __viewer_is_valid_person(person_id)) {
			Evas_Object *popup;
			popup = elm_popup_add(cviewer_d->win);
			evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			elm_object_text_set(popup, T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS));
			elm_popup_timeout_set(popup, 2.0);
			evas_object_smart_callback_add(popup, "timeout", viewer_popup_hide_cb, NULL);
			evas_object_show(popup);
		}
		else {
			cviewer_d->content = ctui_contacts_viewer(cviewer_d->win, NULL, person_id, NULL);
			if (NULL == cviewer_d->content) {
				Evas_Object *popup;
				ERR("ctui_contacts_viewer() return NULL");
				popup = elm_popup_add(cviewer_d->win);
				evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
				elm_object_text_set(popup, S_(PH_SYS_POP_FAILED));
				elm_popup_timeout_set(popup, 2.0);
				evas_object_smart_callback_add(popup, "timeout", viewer_popup_hide_cb, NULL);

				evas_object_show(popup);
			}
			free(val);
		}
	}
	else {
		int cnt_vcard = 0;
		ret = service_get_uri(service, &val);
		p_warn_if(SERVICE_ERROR_NONE != ret, "service_get_uri is failed(%d)", ret);

		val_vcard[0] = '\0';

		if (val && *val) {
			if (0 == strncmp(val, "file://", 7))
				snprintf(val_vcard, sizeof(val_vcard), "%s", val+7);
			else
				snprintf(val_vcard, sizeof(val_vcard), "%s", val);
		}
		else {
			ret = service_get_extra_data(service, AUL_K_MIME_CONTENT, &val);
			p_warn_if(SERVICE_ERROR_NONE != ret, "service_get_extra_data is failed(%d)", ret);
			if (val && *val)
				snprintf(val_vcard, sizeof(val_vcard), "%s", val);
		}
		free(val);

		if ('\0' == *val_vcard) {
			ERR("service value is NULL");
			elm_exit();
			return;
		}

		free(cviewer_d->path);
		cviewer_d->path = strdup(val_vcard);
		err = contacts_vcard_get_entity_count(cviewer_d->path, &cnt_vcard);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_vcard_get_entity_count() Failed(%d)", err);

		if (cnt_vcard < 1) {
			ERR("invalid vcard(%s)", cviewer_d->path);
			elm_exit();
			return;
		}
		else if (1 == cnt_vcard) {
			ret = viewer_load_vcard_contact(cviewer_d);
			if (0 != ret) {
				ERR("viewer_load_vcard_contact() Failed(%d)", ret);
				elm_exit();
				return;
			}
		}
		else {
			ret = viewer_load_vcard_contacts(cviewer_d, cnt_vcard);
			if (0 != ret) {
				ERR("viewer_load_vcard_contact() Failed(%d)", ret);
				elm_exit();
				return;
			}
		}
	}
	elm_win_activate(cviewer_d->win);
	return;
}

API int main(int argc, char *argv[])
{
	ct_viewer_data ad = {0};
	app_event_callback_s event_callback = {0,};

	//vibration engine set, guide by settings - temporary fix
	elm_init(argc, argv);
	elm_config_preferred_engine_set("opengl_x11");

	event_callback.create = viewer_create;
	event_callback.terminate = viewer_terminate;
	event_callback.pause = viewer_pause;
	event_callback.resume = viewer_resume;
	event_callback.service = viewer_service;
	event_callback.low_memory = NULL;
	event_callback.low_battery = NULL;
	event_callback.language_changed = NULL;
	event_callback.region_format_changed = NULL;

	return app_efl_main(&argc, &argv, &event_callback, &ad);
}
