/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#define _GNU_SOURCE		// for strcasestr
#include <aul.h>
#include <string.h>
#include <ui-gadget-module.h>
#include <unicode/ulocdata.h>
#include <unicode/uset.h>
#include <unicode/ustring.h>
#include <vconf.h>
#include <vconf-keys.h>

#include "phone.h"
#include "phone-common.h"
#include "ct-list.h"

static char pre_search_str[CT_LIST_MAX_SEARCH_STR];

static void list_get_number_str(int id, char *buf, int buf_len);

void ctui_list_free_contacts(Eina_List* list)
{
	ct_person_list *person_info;
	Eina_List* l;

	EINA_LIST_FOREACH(list, l, person_info) {
		if (person_info) {

			eina_list_free(person_info->addressbook_id_list);
			free(person_info->img_path);
			free(person_info->display);
			free(person_info->display_index);
			free(person_info->number);
			free(person_info->status_str);
			if (person_info->number_list)
				ctui_free_record_list(person_info->number_list);
			if (person_info->email_list)
				ctui_free_record_list(person_info->email_list);
			free(person_info);
		}
	}
	eina_list_free(list);
}

void ctui_list_free_index(Eina_List* list)
{
	ct_list_index_data *data;
	Eina_List* l;

	EINA_LIST_FOREACH(list, l, data) {
		if (data) {
			free(data);
		}
	}
	eina_list_free(list);
}

static inline int list_get_grp_person_id(Eina_List *list)
{
	ct_person_list *person_info;
	if (NULL == list) return 0;
	person_info = list->data;
	return person_info->id;
}

static contacts_filter_h list_create_filter_int(contacts_filter_h filter, const char *view_uri, int property_id, contacts_match_int_flag_e flag, int match_value)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;

	if (NULL == filter) {
		err = contacts_filter_create(view_uri, &filter);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_filter_create() Failed(%d)", err);
	}
	else {
		err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_operator() Failed(%d)", err);
	}

	err = contacts_filter_add_int(filter, property_id, flag, match_value);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_int() Failed(%d)", err);

	return filter;
}

static contacts_filter_h list_create_filter_bool(contacts_filter_h filter, const char *view_uri, int property_id, contacts_match_int_flag_e flag, bool match_value)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;

	if (NULL == filter) {
		err = contacts_filter_create(view_uri, &filter);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_filter_create() Failed(%d)", err);
	}
	else {
		err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_operator() Failed(%d)", err);
	}

	err = contacts_filter_add_bool(filter, property_id, match_value);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_int() Failed(%d)", err);

	return filter;
}


int ct_list_get_contacts(ct_list_data *clist_d, Eina_List **list_out)
{
	PH_FN_CALL;
	unsigned int count = 0;
	int person_id;
	int grp_person_id;

	unsigned int person_contact_projection[] = {
		_contacts_person_contact.person_id,
		_contacts_person_contact.status,
		_contacts_person_contact.display_name,
		_contacts_person_contact.display_name_index,
		_contacts_person_contact.addressbook_ids,
	};
	unsigned int person_grouprel_projection[] = {
		_contacts_person_grouprel.person_id,
		_contacts_person_grouprel.status,
		_contacts_person_grouprel.display_name,
		_contacts_person_grouprel.display_name_index,
		_contacts_person_grouprel.addressbook_ids,
	};
	unsigned int person_projection[] = {
		_contacts_person.id,
		_contacts_person.status,
		_contacts_person.display_name,
		_contacts_person.display_name_index,
		_contacts_person.addressbook_ids,
	};

	const char *view_uri = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list = NULL;
	contacts_query_h query = NULL;
	contacts_filter_h filter = NULL;
	contacts_filter_h filter2 = NULL;

	Eina_List *result = NULL;

	p_retvm_if(NULL == list_out, CONTACTS_ERROR_INTERNAL, "parameter(list_out) is NULL");
	*list_out = NULL;

	PH_DBG("clist_d->list_op : %d",clist_d->list_op);
	switch (clist_d->list_op) {
	case CT_GET_LIST_ALL_PERSON:
		view_uri = _contacts_person._uri;
		if (clist_d->search_str && *clist_d->search_str) {
			err = contacts_db_search_records(_contacts_person._uri, clist_d->search_str, 0, 0, &list);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_search_records() Failed(%d)", err);
		}
		else {
			PH_DBG("_contacts_person");
			err = contacts_db_get_all_records(_contacts_person._uri, 0, 0, &list);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_all_records() Failed(%d)", err);
		}
		break;
	case CT_GET_LIST_ALL_PERSON_HAD_NUMBER:
		view_uri = _contacts_person._uri;

		err = contacts_query_create(_contacts_person._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		do {
			filter = list_create_filter_bool(filter, _contacts_person._uri, _contacts_person.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_projection, sizeof(person_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);

		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_ALL_PERSON_HAD_EMAIL:
		view_uri = _contacts_person._uri;

		err = contacts_query_create(_contacts_person._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_bool(filter, _contacts_person._uri, _contacts_person.has_email, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err =  contacts_query_set_projection(query, person_projection, sizeof(person_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL:
		view_uri = _contacts_person._uri;

		err = contacts_query_create(_contacts_person._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_bool(filter, _contacts_person._uri, _contacts_person.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_OR))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_bool(filter, _contacts_person.has_email, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_projection, sizeof(person_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_contact._uri;

		err = contacts_query_create(_contacts_person_contact._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_contact._uri, _contacts_person_contact.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_contact_projection, sizeof(person_contact_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID:
		view_uri = _contacts_person_contact._uri;

		err = contacts_query_create(_contacts_person_contact._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_contact._uri, _contacts_person_contact.address_book_mode, CONTACTS_MATCH_EQUAL, CONTACTS_ADDRESS_BOOK_MODE_NONE);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_contact_projection, sizeof(person_contact_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_contact._uri;

		err = contacts_query_create(_contacts_person_contact._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_contact._uri, _contacts_person_contact.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_bool(filter, _contacts_person_contact._uri, _contacts_person_contact.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_contact_projection, sizeof(person_contact_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_contact._uri;

		err = contacts_query_create(_contacts_person_contact._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_contact._uri, _contacts_person_contact.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_bool(filter, _contacts_person_contact._uri, _contacts_person_contact.has_email, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_contact_projection, sizeof(person_contact_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		contacts_filter_destroy(filter);
		contacts_query_destroy(query);
		break;
	case CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_contact._uri;

		err = contacts_query_create(_contacts_person_contact._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_contact._uri, _contacts_person_contact.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter2 = list_create_filter_bool(NULL, _contacts_person_contact._uri, _contacts_person_contact.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter2, CONTACTS_FILTER_OPERATOR_OR))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_bool(filter2, _contacts_person_contact.has_email, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_filter(filter, filter2))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_contact_projection, sizeof(person_contact_projection)/sizeof(int)))) break;

			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);

		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		contacts_filter_destroy(filter);
		contacts_filter_destroy(filter2);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_OF_GROUP_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_EQUAL, clist_d->base_grp);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_GROUP_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_EQUAL, clist_d->base_grp);
			filter = list_create_filter_bool(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true) )) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_GROUP_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_EQUAL, clist_d->base_grp);
			filter = list_create_filter_bool(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_email, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_GROUP_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_EQUAL, clist_d->base_grp);
			filter2 = list_create_filter_bool(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter2, CONTACTS_FILTER_OPERATOR_OR))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_bool(filter2, _contacts_person_grouprel.has_email, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND))) break;

			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_filter(filter, filter2))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_filter_destroy(filter2);
		contacts_query_destroy(query);

		break;

	case CT_GET_LIST_PERSON_NOT_ASSIGNED_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_int(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_NONE, 0);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true) )) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}

		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		contacts_filter_destroy(filter);
		contacts_query_destroy(query);
		break;

	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_int(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_NONE, 0);
			filter2 = list_create_filter_bool(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter2, CONTACTS_FILTER_OPERATOR_OR))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_bool(filter2, _contacts_person_grouprel.has_email, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_filter(filter, filter2))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true) )) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_filter_destroy(filter2);
		contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_EMAIL_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_int(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_NONE, 0);
			filter = list_create_filter_bool(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_email, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true) )) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
	 	contacts_query_destroy(query);

		break;
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OF_ADDRBOOK_ID:
		view_uri = _contacts_person_grouprel._uri;

		err = contacts_query_create(_contacts_person_grouprel._uri, &query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		do {
			filter = list_create_filter_int(NULL, _contacts_person_grouprel._uri, _contacts_person_grouprel.address_book_id, CONTACTS_MATCH_EQUAL, clist_d->base_ab);
			filter = list_create_filter_int(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.group_id, CONTACTS_MATCH_NONE, 0);
			filter = list_create_filter_bool(filter, _contacts_person_grouprel._uri, _contacts_person_grouprel.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_distinct(query, true) )) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_projection(query, person_grouprel_projection, sizeof(person_grouprel_projection)/sizeof(int)))) break;
			if (clist_d->search_str && *clist_d->search_str) {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_search_records_with_query(query, clist_d->search_str, 0, 0, &list))) break;
			}
			else {
				if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;
			}
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);
		break;
	default:
		ERR("Invalid op");
		return CONTACTS_ERROR_INTERNAL;
	}
	PH_DBG("end cotnacts_db_get_list, start while");

	clist_d->cnt_list_total = 0;

	// group - add member
	Eina_List *l;
	Eina_List *list_grp_person;
	list_grp_person = clist_d->list_grp_person;

	p_retvm_if(NULL == list, CONTACTS_ERROR_INTERNAL, "list is NULL");

	err = contacts_list_get_count(list, &count);
	p_retvm_if(CONTACTS_ERROR_NONE != err, CONTACTS_ERROR_INTERNAL, "contacts_list_get_count() Failed(%d)", err);
	p_retvm_if(0 == count, CONTACTS_ERROR_NO_DATA, "list is empty");

	while (err == CONTACTS_ERROR_NONE) {
		ct_person_list *person_info;
		char *temp_addressbooks = NULL;

		if (0 == strcmp(_contacts_person_contact._uri,view_uri))
			err = phone_cts_get_int_from_list(list, _contacts_person_contact.person_id, &person_id);
		else
			err = phone_cts_get_int_from_list(list, _contacts_person.id, &person_id);

		if (person_id == clist_d->selected_person_id) {
			contacts_list_next(list);
			continue;
		}
		ct_person_list *data;
		bool check = false;
		EINA_LIST_FOREACH(list_grp_person,l,data){
			if(data) {
				grp_person_id = data->id;
				PH_DBG("add member : grp_person_id =%d,person_id =%d",grp_person_id,person_id);
				if (grp_person_id && grp_person_id == person_id) {
					contacts_list_next(list);
					check = true;
					break;
				}
			}
			else
				ERR("list_grp_person' data is null");
		}
		if(check)continue;

		person_info = calloc(1, sizeof(ct_person_list));
		if (NULL == person_info) {
			ERR("calloc() return NULL");
			break;
		}
		person_info->id = person_id;
		if (0 == strcmp(_contacts_person_contact._uri,view_uri)) {
			err = phone_cts_get_str_from_list(list, _contacts_person_contact.status, &(person_info->status_str));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person_contact.display_name, &(person_info->display));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person_contact.display_name_index, &(person_info->display_index));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person_contact.addressbook_ids, &temp_addressbooks);
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			person_info->addressbook_id_list = ctui_get_addressbook_id_list(temp_addressbooks, " ");
		}
		else {
			err = phone_cts_get_str_from_list(list, _contacts_person.status, &(person_info->status_str));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person.display_name, &(person_info->display));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_int_from_list(list, _contacts_person.link_count, &(person_info->link_count));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person.display_name_index, &(person_info->display_index));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			err = phone_cts_get_str_from_list(list, _contacts_person.addressbook_ids, &temp_addressbooks);
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			person_info->addressbook_id_list = ctui_get_addressbook_id_list(temp_addressbooks, " ");
		}

		result = eina_list_append(result, person_info);
		clist_d->cnt_list_total++;

		err = contacts_list_next(list);
	}
	*list_out = result;

	if (CONTACTS_ERROR_NO_DATA != err) {
		ERR("contacts_list_next() Failed(%d)", err);
		return err;
	}
	return 0;
}

static inline int list_check_utf8(char c)
{
	if (c < 128)
		return 1;
	else if ((c & 0xe0) == 0xc0)
		return 2;
	else if ((c & 0xf0) == 0xe0)
		return 3;
	else if ((c & 0xf8) == 0xf0)
		return 4;
	else if ((c & 0xfc) == 0xf8)
		return 5;
	else if ((c & 0xfe) == 0xfc)
		return 6;
	else
		return CONTACTS_ERROR_INTERNAL;
}

void ct_list_init_account_data(ct_list_data *clist_d)
{
	PH_FN_CALL;
	int i = 0;
	int account_id = 0;
	int addressbook_id = 0;
	bool addressbook_exist = false;
	Eina_List *l = NULL;
	ct_ab_map acc_types[CT_MAX_ADDRBOOK];
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list = NULL;
	char addressbook_name[PH_TEXT_MAX_LEN]={0,};


	addressbook_id = ctui_get_view_mode();

	err = contacts_db_get_all_records(_contacts_address_book._uri, 0, 0, &list);
	while (CONTACTS_ERROR_NONE == err) {
		err = phone_cts_get_int_from_list(list, _contacts_address_book.id, &(acc_types[i].id));
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_int_from_list() Failed(%d)", err);
			break;
		}
		err = phone_cts_get_int_from_list(list, _contacts_address_book.account_id, &(acc_types[i].account_id));
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_int_from_list() Failed(%d)", err);
			break;
		}

		acc_types[i].icon_path = ctui_get_addressbook_icon_path(acc_types[i].id);
		PH_DBG("icon_path = %s",acc_types[i].icon_path);
		if(acc_types[i].id == addressbook_id)
			addressbook_exist = true;

		ctui_get_addressbook_name(acc_types[i].id, addressbook_name,sizeof(addressbook_name));
		acc_types[i].addressbook_name = SAFE_STRDUP(addressbook_name);

		if (++i == sizeof(acc_types)/sizeof(*acc_types))
			break;

		err = contacts_list_next(list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}
	err = contacts_list_destroy(list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

	if(addressbook_exist == false)
		ctui_set_view_mode(-1);

	if (!clist_d->ug /*&& CT_GET_LIST_ALL_PERSON != clist_d->list_op*/) {
		addressbook_id = ctui_get_view_mode();
		if(addressbook_id == -1 && clist_d->list_op > CT_GET_LIST_ALL_PERSON) {
			clist_d->list_op = CT_GET_LIST_ALL_PERSON;
			clist_d->base_ab = -1;
		}
	}

	if (i == sizeof(acc_types)/sizeof(*acc_types)) {
		ERR("addressbook are too many");
		i--;
	}
	clist_d->acc_types = realloc(clist_d->acc_types, i * sizeof(ct_ab_map));
	p_retm_if(NULL == clist_d->acc_types, "realloc() return NULL");

	memcpy(clist_d->acc_types, acc_types, i * sizeof(ct_ab_map));
	clist_d->acc_count = i;
}

void ct_list_get_addrbook_icon_path(ct_list_data *clist_d, int addressbook_id, char *dest, int dest_len)
{
	int i = 0;
	bool not_found;

	p_retm_if(NULL == dest, "parameter(dest) is NULL");
	dest[0] = '\0';
	if (0 == addressbook_id) return;
	p_retm_if(NULL == clist_d->acc_types, "clist_d->acc_types is NULL");

	not_found = true;

	int count = clist_d->acc_count;
	for( i=0;i<count;i++) {
		if (addressbook_id == clist_d->acc_types[i].account_id) {
			not_found = false;
			break;
		}
	}
	p_retm_if(not_found, "invalid id_ab(%d)", addressbook_id);

	if (clist_d->acc_types[i].icon_path && *clist_d->acc_types[i].icon_path)
		snprintf(dest, dest_len, "%s", clist_d->acc_types[i].icon_path);
}

static inline void list_clean_ecore_task(ct_list_data *clist_d)
{
	if (clist_d->list_idler) {
		ecore_idler_del(clist_d->list_idler);
		clist_d->list_idler = NULL;
	}

}

static void list_searchbar_entry_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d = data;
	p_retm_if(NULL == clist_d, "parameter(clist_d) is NULL");

	ctui_list_set_unsweep(clist_d);
}


static void list_searchbar_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	ct_list_data *clist_d;
	elm_object_focus_set(data, EINA_TRUE);

	clist_d = evas_object_data_get(data, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	list_searchbar_entry_clicked_cb(clist_d, NULL, NULL);
}

static char *list_group_label_get(void *data, Evas_Object *obj, const char *part)
{
	char *group_index = data;
	if (0 == strcmp(part, "elm.text") && group_index && *group_index)
		return strdup(group_index);
	else
		return NULL;
}

static Elm_Genlist_Item_Class group_index =
{
	.item_style = "grouptitle",
	.func.text_get = list_group_label_get,
};


static void __list_set_index_letter(ct_list_data *clist_d)
{
	char *copy_index_string = NULL;
	char *p = NULL;
	char *save_p = NULL;
	char *save_p2 = NULL;
	char *p_group = NULL;
	char *copy_p_group = NULL;
	int group_count = 0;
	bool has_digit = false;
	char *index_string = NULL;
	ct_list_index_data *index_data;

	if(CT_GET_LIST_VCARD == clist_d->list_op) return;

	ctui_list_free_index(clist_d->list_index);
	clist_d->list_index = NULL;

	contacts_utils_get_index_characters(&index_string);

	PH_DBG("index_string = %s", index_string);

	if (index_string) {
		copy_index_string = SAFE_STRDUP(index_string);
		p_group = strtok_r(copy_index_string, ":", &save_p2);

		while (p_group) {
			copy_p_group = SAFE_STRDUP(p_group);
			p = strtok_r(copy_p_group, ";", &save_p);
			group_count++;

			while(p) {
				//in case number
				if(isdigit(p[0])) {
					if(has_digit == false)
						has_digit = true;
					else {
						p = strtok_r(NULL, ";", &save_p);
						continue;
					}
				}
				//in case Alphabet is sencond laguange
				if(group_count > 2) {
					if(*p > 'A' && *p < 'Z') {
						p = strtok_r(NULL, ";", &save_p);
						continue;
					}
				}

				index_data = calloc(1, sizeof(ct_list_index_data));
				snprintf(index_data->index, sizeof(index_data->index), "%s", p);

				index_data->item = elm_index_item_append(clist_d->fast_index, index_data->index, NULL, NULL);
				clist_d->list_index = eina_list_append(clist_d->list_index, index_data);

				p = strtok_r(NULL, ";", &save_p);
			}
			free(copy_p_group);
			p_group = strtok_r(NULL, ":", &save_p2);
		}
		free(copy_index_string);
	}
	return;
}


static bool __is_special_letter(char* letter, ct_list_data *clist_d)
{
	ct_list_index_data *first_data;
	ct_list_index_data *last_data;
	unsigned int count;
	if(clist_d->list_index == NULL) {
		ERR("clist_d->list_index is null");
		return false;
	}

	count = eina_list_count(clist_d->list_index);

	first_data = (ct_list_index_data*)eina_list_nth(clist_d->list_index, 1);// first primary letter
	last_data = (ct_list_index_data*)eina_list_nth(clist_d->list_index, count-2);// last primary letter

	if(*first_data->index == 'A' && *last_data->index == 'Z') {
		if(*letter < 'A' || *letter > 'Z')
			return true;
	}
	else {
		if((*letter < 'A' || *letter > 'Z') && (*letter < *first_data->index || *letter > *first_data->index))
			return true;
	}

	return false;
}

static void list_append_item(ct_list_data *clist_d, ct_person_list *person_info)
{
	char cur_index[10] = {0,};
	ct_list_index_data *data;
	Eina_List* l;
	Elm_Object_Item *item;


	// append group title if condition is accepted
	if(CT_GET_LIST_VCARD != clist_d->list_op && !(clist_d->search_str && *clist_d->search_str)) {
		if (NULL == person_info->display)
			snprintf(cur_index, sizeof(cur_index), "#");
		else {
			snprintf(cur_index, sizeof(cur_index), "%s", person_info->display_index);
			if (isdigit(person_info->display_index[0]))
				snprintf(cur_index, sizeof(cur_index), "0");
			else if(__is_special_letter(person_info->display_index, clist_d))
				snprintf(cur_index, sizeof(cur_index), "#");
		}

		if (strcmp(clist_d->group_index, cur_index)) {
			item = elm_genlist_item_append(clist_d->genlist, &group_index, strdup(cur_index), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
			elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
			snprintf(clist_d->group_index, sizeof(clist_d->group_index), "%s", cur_index);
			clist_d->group_item = item;

			//link to fastscroll
			bool matched = false;
			bool first_item = false;
			data = eina_list_nth(clist_d->list_index, 0);
			if(elm_object_item_data_get(data->item) == NULL) {
				first_item = true;
			}

			EINA_LIST_FOREACH(clist_d->list_index, l, data) {
				if(first_item == true) {
					elm_object_item_data_set(data->item, item);
					continue;
				}

				if(0 == strcmp(cur_index, data->index))
					matched = true;
				if(matched == true)
					elm_object_item_data_set(data->item, item);
			}
		}
	}

	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	bool has_phonenumber = false;

	if(!clist_d->vcard_path) {
		err = contacts_db_get_record(_contacts_person._uri, person_info->id, &record);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);
		err = contacts_record_get_bool(record, _contacts_person.has_phonenumber, &has_phonenumber);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_bool() Failed(%d)", err);
		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	}

	if (clist_d->search_str && *clist_d->search_str){
		if(has_phonenumber)
			item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER], person_info, NULL,
				ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
		else
			item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_SEARCH], person_info, NULL,
				ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
	}
	else{
		if(has_phonenumber)
			item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_NO_NUMBER], person_info, clist_d->group_item,
				ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
		else
			item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM], person_info, clist_d->group_item,
				ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
	}

	person_info->item = item;
	if (person_info->checked) clist_d->cnt_checked++;
}


#define N 20
static Eina_Bool list_append_part(void *user_data)
{
	int i = 0;
	Eina_List *cursor;
	ct_person_list *person_info;
	ct_list_data *clist_d = user_data;

	cursor = eina_list_nth_list(clist_d->list, clist_d->readed_n);
	EINA_LIST_FOREACH(cursor, cursor, person_info) {
		if (i++ == N) break;
		list_append_item(clist_d, person_info);
	}
	if (N < i) {
		clist_d->readed_n += N;
		return ECORE_CALLBACK_RENEW;
	}
	else {
		clist_d->list_idler = NULL;
			if (EINA_FALSE == elm_genlist_decorate_mode_get(clist_d->genlist)) {
				clist_d->count_item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_COUNT],
						(void *)clist_d->cnt_list_total, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
				elm_object_item_disabled_set(clist_d->count_item, EINA_TRUE);
		}
		clist_d->list_idler = NULL;
		return ECORE_CALLBACK_CANCEL;
	}
}


static bool __list_get_addressbook_name(int addressbook_id, char *buf, int buf_size)
{
	int addressbook_mode = CONTACTS_ADDRESS_BOOK_MODE_NONE;
	char *cur_ab;
	char cur_account_name[200] = {0,};
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	p_retvm_if(buf == NULL, false, "out_name in null");

	ctui_get_account_name_by_address_book_id(addressbook_id, cur_account_name, sizeof(cur_account_name));
	if (addressbook_id > 0) {
		err = contacts_db_get_record(_contacts_address_book._uri, addressbook_id, &record);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_db_get_record() Failed(%d)", err);

		err = contacts_record_get_str_p(record, _contacts_address_book.name, &cur_ab);
		if(err != CONTACTS_ERROR_NONE) {
			err = contacts_record_destroy(record, true);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
			return false;
		}

		err = contacts_record_get_int(record, _contacts_address_book.mode, &addressbook_mode);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		if (CONTACTS_ADDRESS_BOOK_MODE_READONLY == addressbook_mode)
			snprintf(buf, buf_size, "%s", cur_account_name);
		else
			snprintf(buf, buf_size, "%s %s", cur_account_name, cur_ab);

		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	}
	else{
		snprintf(buf, buf_size, "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
	}

	return true;
}


void ctui_list_handle_search(ct_list_data *clist_d)
{
	int i=0;
	int readed = 0;
	Eina_List *cursor;
	ct_person_list *person_info;
	char group_title[PH_TEXT_MAX_LEN];
	char addressbook_name[PH_TEXT_MAX_LEN];

	elm_genlist_clear(clist_d->genlist);
	elm_index_item_clear(clist_d->fast_index);
	clist_d->group_index[0] = '\0';
	clist_d->group_item = NULL;
	clist_d->cnt_checked = 0;

	if (clist_d->list_idler) {
		ecore_idler_del(clist_d->list_idler);
		clist_d->list_idler = NULL;
	}


	if(__list_get_addressbook_name(clist_d->base_ab, addressbook_name, sizeof(addressbook_name)) == true)
		snprintf(group_title, sizeof(group_title), "%s (%d)", addressbook_name, eina_list_count(clist_d->list));
	else
		snprintf(group_title, sizeof(group_title), "%s (%d)", S_(PH_SYS_BODY_CONTACTS), eina_list_count(clist_d->list));

	clist_d->search_group_item = elm_genlist_item_append(clist_d->genlist, &group_index, strdup(group_title), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
	elm_genlist_item_select_mode_set(clist_d->search_group_item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	EINA_LIST_FOREACH(clist_d->list, cursor, person_info) {
		if (i++ == N) break;
		list_append_item(clist_d, person_info);
		readed++;
	}
	if (N < i) {
		clist_d->readed_n = readed;
		clist_d->list_idler = ecore_idler_add(list_append_part, clist_d);
	}
	else {
		if (!(clist_d->search_str && *clist_d->search_str)) {
			if (EINA_FALSE == elm_genlist_decorate_mode_get(clist_d->genlist)) {
				clist_d->count_item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_COUNT],
						(void *)clist_d->cnt_list_total, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
				elm_object_item_disabled_set(clist_d->count_item, EINA_TRUE);
			}
		}
	}

}


static void list_append_all(ct_list_data *clist_d)
{
	PH_START_TIME_CHECK;
	int i = 0;
	Eina_List *cursor;
	ct_person_list *person_info;

	if (clist_d->list_idler) {
		ecore_idler_del(clist_d->list_idler);
		clist_d->list_idler = NULL;
	}

	__list_set_index_letter(clist_d);

	if(clist_d->base_ab > -1){
		contacts_error_e err = CONTACTS_ERROR_NONE;
		contacts_record_h record = NULL;
		Elm_Object_Item *item;
		int addressbook_mode;
		char title[400] = {0,};
		char cur_account_name[200] = {0,};
		char *cur_ab;

		ctui_get_account_name_by_address_book_id(clist_d->base_ab, cur_account_name, sizeof(cur_account_name));
		if (clist_d->base_ab > 0) {
			err = contacts_db_get_record(_contacts_address_book._uri, clist_d->base_ab, &record);
			p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

			err = contacts_record_get_str_p(record, _contacts_address_book.name, &cur_ab);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			err = contacts_record_get_int(record, _contacts_address_book.mode, &addressbook_mode);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

			if (CONTACTS_ADDRESS_BOOK_MODE_READONLY == addressbook_mode)
				snprintf(title, sizeof(title), "%s", cur_account_name);
			else
				snprintf(title, sizeof(title), "%s %s", cur_account_name, cur_ab);

			err = contacts_record_destroy(record, true);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
		}
		else{
			snprintf(title, sizeof(title), "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
		}

		item = elm_genlist_item_append(clist_d->genlist, &group_index, strdup(title), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}

	EINA_LIST_FOREACH(clist_d->list, cursor, person_info) {
		if (i++ == N) break;
		list_append_item(clist_d, person_info);
	}
	if (N < i) {
		clist_d->readed_n = N;
		clist_d->list_idler = ecore_idler_add(list_append_part, clist_d);
	}
	else {
		if (EINA_FALSE == elm_genlist_decorate_mode_get(clist_d->genlist)) {
			clist_d->count_item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_COUNT],
					(void *)clist_d->cnt_list_total, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
			elm_object_item_disabled_set(clist_d->count_item, EINA_TRUE);
		}
	}

	PH_END_TIME_CHECK("list_append_all");
}
#undef N

void ctui_list_refresh_genlist(ct_list_data *clist_d)
{
	PH_FN_CALL;
	elm_genlist_clear(clist_d->genlist);
	elm_index_item_clear(clist_d->fast_index);
	clist_d->group_index[0] = '\0';
	clist_d->group_item = NULL;
	clist_d->cnt_checked = 0;
	list_append_all(clist_d);
}

static void list_update_person_list_check_status(Eina_List *list_person, Eina_List *list_check)
{
	void *data;
	Eina_List *l, *l_temp;
	ct_person_list *person_info;

	p_retm_if(NULL == list_person, "parameter(list_person) is NULL");
	p_retm_if(NULL == list_check, "parameter(list_check) is NULL");

	EINA_LIST_FOREACH(list_person, l, person_info) {
		EINA_LIST_FOREACH(list_check, l_temp, data) {
			if (person_info->id == (int)data) {
				person_info->checked = true;
				break;
			}
		}
	}
}

static void list_searchbar_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	Evas_Object *en;
	Evas_Object *sb = data;
	ct_list_data *clist_d;
	char *search_str;

	clist_d = evas_object_data_get(sb, "clist_d");
	p_retm_if(NULL == sb, "evas_object_data_get() return NULL");

	en = elm_object_part_content_get(sb, "elm.swallow.content");
	p_retm_if(NULL == en, "elm_object_part_content_get() return NULL");

	if (!elm_entry_is_empty(en))
		elm_object_signal_emit(sb, "elm,state,guidetext,hide", "elm");

	if (elm_object_focus_get(sb)) {
		if (elm_entry_is_empty(en))
			elm_object_signal_emit(sb, "elm,state,eraser,hide", "elm");
		else
			elm_object_signal_emit(sb, "elm,state,eraser,show", "elm");
	}
	else{
		if (elm_entry_is_empty(en)){
			elm_object_signal_emit(sb, "elm,state,guidetext,show", "elm");
			elm_object_signal_emit(sb, "elm,state,eraser,hide", "elm");
		}
	}
	search_str = ctui_entry_markup_to_utf8(elm_entry_entry_get(en));
	p_retm_if(NULL == search_str, "ctui_entry_markup_to_utf8() return NULL");

	if(strlen(search_str)>0) {
		if(NULL !=clist_d->select_all_layout){
			elm_box_unpack(clist_d->box, clist_d->select_all_layout);
			evas_object_del(clist_d->select_all_layout);
			clist_d->select_all_layout = NULL;
		}
	}
	else {
		ctui_list_set_selectall(clist_d);
	}
	if (search_str && clist_d->search_str){
		if(pre_search_str[0] == '\0' && search_str[0] == '\0'){
			clist_d->search_str = NULL;
			ct_list_get_contacts(clist_d, &clist_d->list);
			list_update_person_list_check_status(clist_d->list, clist_d->list_check);
			elm_genlist_homogeneous_set(clist_d->genlist, EINA_TRUE);
			elm_genlist_mode_set(clist_d->genlist, ELM_LIST_SCROLL);
			ctui_list_refresh_genlist(clist_d);
		}
		else{
			if(0 == strcmp(search_str, clist_d->search_str)){
				free(search_str);
				return;
			}
		}
	}
	if('\0' == search_str[0] && NULL == clist_d->search_str) {
		if (elm_genlist_decorate_mode_get(clist_d->genlist) && clist_d->edit_sel_update)
			clist_d->edit_sel_update(clist_d);
		free(search_str);
		return;
	}

	free(clist_d->search_str);
	clist_d->search_str = search_str;

	if (strcmp(pre_search_str, search_str)) {
		ct_list_get_contacts(clist_d, &clist_d->list);
		list_update_person_list_check_status(clist_d->list, clist_d->list_check);

		if (*search_str) {
			elm_genlist_mode_set(clist_d->genlist, ELM_LIST_COMPRESS);
			ctui_list_handle_search(clist_d);
		}
		else {
			elm_genlist_homogeneous_set(clist_d->genlist, EINA_TRUE);
			elm_genlist_mode_set(clist_d->genlist, ELM_LIST_SCROLL);
			ctui_list_refresh_genlist(clist_d);
		}
	}

	snprintf(pre_search_str, sizeof(pre_search_str), "%s", search_str);

	if (elm_genlist_decorate_mode_get(clist_d->genlist) && clist_d->edit_sel_update)
		clist_d->edit_sel_update(clist_d);

}

static void list_searchbar_focused_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	elm_object_signal_emit(data, "elm,state,guidetext,hide", "elm");
	elm_object_signal_emit(data, "cancel,in", "");

	if (elm_entry_is_empty(obj))
		elm_object_signal_emit(data, "elm,state,eraser,hide", "elm");
	else
		elm_object_signal_emit(data, "elm,state,eraser,show", "elm");
}

static void list_searchbar_unfocused_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	Evas_Object *cancel_btn;

	if (elm_entry_is_empty(obj))
		elm_object_signal_emit(data, "elm,state,guidetext,show", "elm");
	else
		elm_object_signal_emit(data, "elm,state,guidetext,hide", "elm");

	elm_object_signal_emit(data, "elm,state,eraser,hide", "elm");
	cancel_btn = elm_object_part_content_get(data, "button_cancel");
	evas_object_hide(cancel_btn);
	elm_object_signal_emit(data, "cancel,out", "");
}

int ctui_list_checked_cnt_get(Eina_List *list) {
	int cnt = 0;
	Eina_List *l;
	ct_person_list *person_info;

	EINA_LIST_FOREACH(list, l, person_info) {
		if (NULL == person_info) continue;
		if (person_info->checked)
			cnt++;
	}
	return cnt;
}

void ctui_list_update_selection_info(void *data)
{
	PH_FN_CALL;
	ct_list_data *clist_d = data;
	int checked_cnt = 0;
	char buf[PH_TEXT_SHORT_LEN];

	if(CT_LIST_MODE_EDIT != clist_d->list_mode)
		return;

	if (clist_d->ug_request && NULL == clist_d->select_all_layout) {
		checked_cnt = ctui_list_checked_cnt_get(clist_d->list);
	}
	else {
		Evas_Object *check = elm_object_part_content_get(clist_d->select_all_layout, "elm.icon");
		if(check) {
			if (0 != clist_d->cnt_list_total && clist_d->cnt_checked == clist_d->cnt_list_total)
				elm_check_state_set(check, EINA_TRUE);
			else if (0 == clist_d->cnt_list_total || clist_d->cnt_checked != clist_d->cnt_list_total)
				elm_check_state_set(check, EINA_FALSE);
		}
		checked_cnt = clist_d->cnt_checked;
	}
	if (0 == clist_d->cnt_list_total) {
		phone_hide_notify(clist_d->base_layout);
		ctui_navi_btn_disabled_set(clist_d->navi, EINA_TRUE);
	}
	else {
		if (0 == checked_cnt ) {
			phone_show_notify(clist_d->base_layout, T_(CT_GET_TEXT_BASIC, CTTEXT_SELECT_CONTACTS));
			ctui_navi_btn_disabled_set(clist_d->navi, EINA_TRUE);
		}
		else {
			ctui_navi_btn_disabled_set(clist_d->navi, EINA_FALSE);
			if (clist_d->cnt_max_check)
				snprintf(buf, sizeof(buf), "%s (%d/%d)", S_(PH_SYS_POP_SELECTED), clist_d->cnt_checked, clist_d->cnt_max_check);
			else
				snprintf(buf, sizeof(buf), "%s (%d)", S_(PH_SYS_POP_SELECTED), clist_d->cnt_checked);
			phone_show_notify(clist_d->base_layout, buf);
		}
	}
}

void ctui_list_searchbar_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *sb = data;
	Evas_Object *en;
	ct_list_data *clist_d;
	const char *text;

	p_retm_if(NULL == sb, "parameter(sb) is NULL");

	clist_d = evas_object_data_get(sb, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	en = elm_object_part_content_get(sb, "elm.swallow.content");
	p_retm_if(NULL == en, "elm_object_part_content_get() return NULL");

	free(clist_d->search_str);
	clist_d->search_str = NULL;

	text = elm_entry_entry_get(en);
	if (text && *text)
		elm_entry_entry_set(en, NULL);
	elm_object_focus_set(en, EINA_FALSE);
	pre_search_str[0] = '\0';

	ct_list_get_contacts(clist_d, &clist_d->list);
	list_update_person_list_check_status(clist_d->list, clist_d->list_check);
	ctui_list_update_selection_info(clist_d);

	elm_genlist_homogeneous_set(clist_d->genlist, EINA_TRUE);
	elm_genlist_mode_set(clist_d->genlist, ELM_LIST_SCROLL);
	ctui_list_refresh_genlist(clist_d);

	// problem of set focus automatically on searchbar when resumed app
	elm_object_focus_set(clist_d->genlist, EINA_TRUE);
}

static void list_searchbar_eraser_cb(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	elm_entry_entry_set(data, "");
	pre_search_str[0] = '\0';
}

static Evas_Object* list_create_searchbar(Evas_Object* parent, ct_list_data *clist_d)
{
	Evas_Object *sb;
	Evas_Object *en;
	Evas_Object *cancel_btn;

	sb = elm_layout_add(parent);
	p_retvm_if(NULL == sb, NULL, "elm_layout_add() return NULL");
	evas_object_data_set(sb, "clist_d", clist_d);

	elm_layout_theme_set(sb, "layout", "searchbar", "cancel_button");

	en = elm_entry_add(sb);
	elm_entry_scrollable_set(en, EINA_TRUE);
	elm_entry_single_line_set(en, EINA_TRUE);
	elm_object_part_content_set(sb, "elm.swallow.content", en);
	elm_object_part_text_set(sb, "elm.guidetext", S_(CT_SYS_BODY_SEARCH));
	elm_entry_cnp_mode_set(en, ELM_CNP_MODE_PLAINTEXT);

	ctui_entry_limit_size_set(en);
	elm_entry_input_panel_layout_set(en, ELM_INPUT_PANEL_LAYOUT_NORMAL);

	cancel_btn = elm_button_add(sb);
	elm_object_part_content_set(sb, "button_cancel", cancel_btn);
	elm_object_style_set(cancel_btn, "searchbar/default");
	elm_object_text_set(cancel_btn, S_(PH_SYS_SK_CANCEL));

	evas_object_smart_callback_add(en, "focused", list_searchbar_focused_cb, sb);
	evas_object_smart_callback_add(en, "unfocused", list_searchbar_unfocused_cb, sb);
	evas_object_smart_callback_add(en, "changed", list_searchbar_changed_cb, sb);
	evas_object_smart_callback_add(en, "preedit,changed", list_searchbar_changed_cb, sb);
	evas_object_smart_callback_add(en, "clicked", list_searchbar_entry_clicked_cb, clist_d);
	elm_object_signal_callback_add(sb, "elm,bg,clicked", "elm", list_searchbar_clicked_cb, en);
	elm_object_signal_callback_add(sb, "elm,eraser,clicked", "elm", list_searchbar_eraser_cb, en);
	evas_object_smart_callback_add(cancel_btn, "clicked", ctui_list_searchbar_cancel_cb, sb);

	return sb;
}

static Evas_Object* list_create_navi_content_genlist(Evas_Object* parent)
{
	Evas_Object *genlist = elm_genlist_add(parent);

	elm_genlist_scroller_policy_set(genlist, ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);
	elm_genlist_block_count_set(genlist, 20);
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	return genlist;
}

static void list_fast_delayed_changed(void *data, Evas_Object *obj,
		void *event_info)
{
	// called on a change but delayed in case multiple changes happen in a
	// short timespan
	elm_genlist_item_bring_in(elm_object_item_data_get(event_info),
			ELM_GENLIST_ITEM_SCROLLTO_TOP);
}

static void list_fast_selected(void *data, Evas_Object *obj, void *event_info)
{
	// called on final select
	elm_genlist_item_bring_in(elm_object_item_data_get(event_info),
			ELM_GENLIST_ITEM_SCROLLTO_TOP);
}

static void list_start_sweep_effect(void *data)
{
	elm_genlist_item_decorate_mode_set(data, "slide", EINA_TRUE);
	elm_genlist_item_select_mode_set(data, ELM_OBJECT_SELECT_MODE_NONE);
}

static void list_get_number_str(int id, char *buf, int buf_len)
{
	char *number;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	err = contacts_db_get_record(_contacts_person_number._uri, id, &record);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_person_number.number, &number);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

	if (number && *number)
		snprintf(buf, buf_len, "%s", number);

	err = contacts_record_destroy(record, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
}

static void list_gl_sweep(void *data, Evas_Object *obj, void *event_info)
{
	ct_person_list *person_info;
	ct_list_data *clist_d = data;
	Elm_Object_Item *item = event_info;
	Elm_Genlist_Item_Class *itc;
	Elm_Genlist_Item_Type type;

	p_ret_if(!item);
	p_ret_if(CT_GET_LIST_VCARD == clist_d->list_op);
	p_ret_if(CT_LIST_MODE_EDIT == clist_d->list_mode);

	type = elm_genlist_item_type_get(item);
	p_ret_if(ELM_GENLIST_ITEM_GROUP == type);
	p_ret_if(&clist_d->itcs[CT_LIST_ITEM_COUNT] == elm_genlist_item_item_class_get(item));

	person_info = elm_object_item_data_get(item);
	p_retm_if(NULL == person_info, "ct_person_list is null");
	if (!person_info->number_loaded) {
		char buf[PH_TEXT_MAX_LEN];
		buf[0] = '\0';
		list_get_number_str(person_info->id, buf, sizeof(buf));
		if (*buf)
			person_info->number = strdup(buf);
		person_info->number_loaded = true;
	}

	itc = (Elm_Genlist_Item_Class *)elm_genlist_item_item_class_get(item);
	p_retm_if(NULL == itc, "item class is null");

	if ((NULL == person_info->number || '\0' == *person_info->number)
			&& strcmp(itc->decorate_item_style, clist_d->itcs[CT_LIST_ITEM].decorate_item_style)){
		if (clist_d->search_str && *clist_d->search_str)
			elm_genlist_item_item_class_update(item, &clist_d->itcs[CT_LIST_ITEM_SEARCH]);
		else
			elm_genlist_item_item_class_update(item, &clist_d->itcs[CT_LIST_ITEM]);
	}
	else if ((person_info->number && *person_info->number)
			&& strcmp(itc->decorate_item_style, clist_d->itcs[CT_LIST_ITEM_NO_NUMBER].decorate_item_style)) {
		if (clist_d->search_str && *clist_d->search_str)
			elm_genlist_item_item_class_update(item, &clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER]);
		else
			elm_genlist_item_item_class_update(item, &clist_d->itcs[CT_LIST_ITEM_NO_NUMBER]);
	}

	if(clist_d->fast_index) {
		evas_object_hide(clist_d->fast_index);
		elm_object_disabled_set(clist_d->fast_index, EINA_TRUE);
	}

	clist_d->last_sweep_item = item;
	elm_genlist_item_decorate_mode_set(event_info, "slide", EINA_TRUE);
	elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_NONE);
}

static void list_gl_unsweep(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d = data;
	if (NULL == clist_d->last_sweep_item) return;

	elm_genlist_item_decorate_mode_set(clist_d->last_sweep_item, "slide", EINA_FALSE);
	elm_genlist_item_select_mode_set(clist_d->last_sweep_item, ELM_OBJECT_SELECT_MODE_DEFAULT);
	clist_d->last_sweep_item = NULL;
	if(clist_d->fast_index) {
		evas_object_show(clist_d->fast_index);
		elm_object_disabled_set(clist_d->fast_index, EINA_FALSE);
	}
}

void ctui_list_set_unsweep(ct_list_data *clist_d)
{
	list_gl_unsweep(clist_d, NULL, NULL);
}

static Evas_Object* list_create_contact_list(Evas_Object *parent, ct_list_data *clist_d)
{
	Evas_Object *layout, *fast_index, *searchbar, *genlist;

	elm_win_conformant_set(clist_d->win, EINA_TRUE);

	layout = elm_layout_add(parent);
	if(CT_GET_LIST_VCARD == clist_d->list_op)
		elm_layout_file_set(layout, CONTACTS_EDJ, "genlist_no_searchbar");
	else
		elm_layout_file_set(layout, CONTACTS_EDJ, "genlist");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	clist_d->layout = layout;

	if(CT_GET_LIST_VCARD != clist_d->list_op){
		searchbar = list_create_searchbar(layout, clist_d);
		elm_object_part_content_set(layout, "searchbar", searchbar);
		evas_object_show(searchbar);
		clist_d->searchbar = searchbar;
	}

	clist_d->box = elm_box_add(layout);
	evas_object_size_hint_weight_set(clist_d->box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(clist_d->box);
	elm_object_part_content_set(layout, "content", clist_d->box);

	// Create genlist
	genlist = list_create_navi_content_genlist(clist_d->box);
	evas_object_show(genlist);
	elm_object_theme_set(genlist, clist_d->th);

	evas_object_smart_callback_add(genlist, "drag,start,right", list_gl_sweep, clist_d);
	evas_object_smart_callback_add(genlist, "drag,start,left", list_gl_unsweep, clist_d);
	evas_object_smart_callback_add(genlist, "drag,start,up", list_gl_unsweep, clist_d);
	evas_object_smart_callback_add(genlist, "drag,start,down", list_gl_unsweep, clist_d);

	clist_d->genlist = genlist;

	elm_box_pack_end(clist_d->box, genlist);
	evas_object_data_set(genlist, "clist_d", clist_d);

	if (CT_GET_LIST_VCARD != clist_d->list_op) {
		// Create fast scroll
		fast_index = elm_index_add(layout);
		elm_object_part_content_set(layout, "fastscroll", fast_index);
		evas_object_show(fast_index);
		clist_d->fast_index = fast_index;
		evas_object_smart_callback_add(fast_index, "delay,changed", list_fast_delayed_changed, fast_index);
		evas_object_smart_callback_add(fast_index, "selected", list_fast_selected, NULL);
		elm_index_level_go(fast_index, 0);
	}

	// Genlist item append
	pre_search_str[0] = '\0';

	evas_object_show(layout);

	return layout;
}

static void list_resize_contact_view(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *scroller;
	Evas_Object *box;
	ct_list_data *clist_d = data;
	int changed_rotate = elm_win_rotation_get(clist_d->win);

	if (clist_d->list || clist_d->search_str)
		return;

	if(clist_d->current_rotate != changed_rotate)
		clist_d->current_rotate = changed_rotate;
	else
		return;

	if (0 == changed_rotate || 180 == changed_rotate) {
		scroller = elm_object_part_content_get(clist_d->content_layout, "elm.swallow.content");
		p_retm_if(NULL == scroller, "elm_object_part_content_get() return NULL");
		box = elm_object_content_get(scroller);
		p_retm_if(NULL == box, "elm_object_content_get() return NULL");
		elm_box_unpack(box, clist_d->nocontent);
		p_retm_if(NULL == clist_d->nocontent, "elm_box_unpack() return NULL");
		evas_object_size_hint_weight_set(clist_d->nocontent, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(clist_d->nocontent, EVAS_HINT_FILL, 0.2);
		elm_box_pack_end(box, clist_d->nocontent);
		elm_object_content_set(scroller, box);
		evas_object_size_hint_min_set(box, 720, 650);
		elm_object_part_content_set(clist_d->content_layout, "elm.swallow.content", scroller);
	}
	else {
		scroller = elm_object_part_content_get(clist_d->content_layout, "elm.swallow.content");
		p_retm_if(NULL == scroller, "elm_object_part_content_get() return NULL");
		box = elm_object_content_get(scroller);
		p_retm_if(NULL == box, "elm_object_content_get() return NULL");
		elm_box_unpack(box, clist_d->nocontent);
		p_retm_if(NULL == clist_d->nocontent, "elm_box_unpack() return NULL");
		evas_object_size_hint_weight_set(clist_d->nocontent, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(clist_d->nocontent, EVAS_HINT_FILL, 0.0);
		elm_box_pack_end(box, clist_d->nocontent);
		elm_object_content_set(scroller, box);
		evas_object_size_hint_min_set(box, 720, 650);
		elm_object_part_content_set(clist_d->content_layout, "elm.swallow.content", scroller);
	}

}


static void list_fill_contact_view(Evas_Object *parent, ct_list_data *clist_d)
{
	PH_FN_CALL;
	Evas_Object *content = NULL;
	Evas_Object *base = NULL;

	if (clist_d->list || clist_d->search_str) {
		if (NULL == clist_d->genlist) {
			content = list_create_contact_list(parent, clist_d);
			list_append_all(clist_d);

			if (CT_LIST_MODE_EDIT == clist_d->list_mode)
				ctui_list_edit_mode_start(clist_d);
			elm_object_part_content_set(parent, "elm.swallow.content", content);
		}
		else {
			if (clist_d->search_str && *clist_d->search_str)
				ctui_list_handle_search(clist_d);
			else
				ctui_list_refresh_genlist(clist_d);
		}
		evas_object_smart_callback_del(clist_d->win, "wm,rotation,changed", list_resize_contact_view);
	}
	else {
		char buf[PH_TEXT_MAX_LEN] ={0,};
		Evas_Object *scroller;
		Evas_Object *box;

		list_clean_ecore_task(clist_d);

		scroller = elm_scroller_add(parent);
		p_retm_if(NULL == scroller, "elm_scroller_add() return NULL");
		elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
		elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
		evas_object_show(scroller);

		box = elm_box_add(scroller);
		p_retm_if(NULL == box, "elm_box_add() return NULL");
		evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, 0.0);
		evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
		elm_object_content_set(scroller, box);
		evas_object_show(box);

		base = elm_layout_add(box);
		p_retm_if(NULL == base, "elm_layout_add() return NULL");
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(base, EVAS_HINT_FILL, 0.2);
		elm_layout_file_set(base, NOCONTENT_LAYOUT_EDJ, "nocontent_layout");
		snprintf(buf, sizeof(buf), "%s<br>%s", T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS), T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS_HELP_TEXT));
		elm_object_part_text_set(base, "elm.text", buf);
		elm_box_pack_end(box, base);
		evas_object_size_hint_min_set(box, 720, 650);
		evas_object_show(base);
		clist_d->nocontent = base;
		clist_d->current_rotate = elm_win_rotation_get(clist_d->win);
		evas_object_smart_callback_add(clist_d->win, "wm,rotation,changed", list_resize_contact_view, clist_d);
		elm_object_part_content_set(parent, "elm.swallow.content", scroller);

		clist_d->genlist = NULL;
		clist_d->group_index[0] = '\0';
		clist_d->group_item = NULL;

		free(clist_d->search_str);
		clist_d->search_str = NULL;
		clist_d->searchbar = NULL;
	}
}

API Eina_Bool ctui_list_genlist_update(void *data)
{
	PH_FN_CALL;
	char buf[PH_TEXT_MAX_LEN+1];
	Evas_Object *content;
	ct_list_data *clist_d = data;

	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
	}

	p_retvm_if(NULL == clist_d->navi_item, ECORE_CALLBACK_CANCEL, "navi_item is NULL");

	ctui_list_free_contacts(clist_d->list);
	clist_d->list = NULL;
	ct_list_get_contacts(clist_d, &clist_d->list);

	if(CT_LIST_IN_MULTI_TAB != clist_d->tab_mode && CT_UG_REQUEST_SELECT_PERSON <= clist_d->ug_request) {
		snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);
		elm_object_item_text_set(clist_d->navi_item, buf);
	}

	content = clist_d->content_layout;
	list_fill_contact_view(content, clist_d);
	if (clist_d->list_navi_item_update_fn)
		clist_d->list_navi_item_update_fn(clist_d);

	clist_d->update_timer = NULL;
	return ECORE_CALLBACK_CANCEL;
}

void ct_list_clear_pdata(ct_list_data *clist_d)
{
	p_ret_if(NULL == clist_d);
	p_ret_if(NULL == clist_d->p_info);

	clist_d->p_info->alive = false;
	pthread_join(clist_d->p_info->thread, NULL);

	ecore_timer_del(clist_d->p_info->timer);

	if (clist_d->popup) {
		evas_object_del(clist_d->popup);
		clist_d->popup = NULL;
	}

	free(clist_d->p_info);
	clist_d->p_info = NULL;
}

static void list_addrbook_changed(const char *view_uri, void *user_data)
{
	ct_list_data *clist_d = user_data;
	ct_list_init_account_data(clist_d);
	ctui_list_lazy_update(_contacts_person._uri ,clist_d);
}

static void list_set_group_index_item(Eina_List *list_index, const char *cur_index, Elm_Object_Item *cur_item)
{
	Eina_List* l;
	ct_list_index_data *data;

	EINA_LIST_FOREACH(list_index, l, data) {
		if (0 == strcmp(cur_index, data->index)) {
			//PH_DBG("list_append_item -  %s mapping to %s", data->index, cur_index);
			elm_object_item_data_set(data->item, cur_item);
			break;
		}
	}
}

static Elm_Object_Item* list_genlist_update_group_item_get(ct_list_data *clist_d, Elm_Object_Item *item, const char *cur_index)
{
	int cmp;
	char *item_index;
	Elm_Object_Item *next_item;
	Elm_Object_Item *item_ret;

	if (NULL == item) {
		item_ret = elm_genlist_item_append(clist_d->genlist, &group_index, strdup(cur_index), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
		elm_genlist_item_select_mode_set(item_ret, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
		list_set_group_index_item(clist_d->list_index, cur_index, item_ret);
		return item_ret;
	}
	if (ELM_GENLIST_ITEM_GROUP != elm_genlist_item_type_get(item)) {
		next_item = elm_genlist_item_next_get(item);
		item_ret = list_genlist_update_group_item_get(clist_d, next_item, cur_index);
		elm_object_item_del(item);
		return item_ret;
	}

	item_index = elm_object_item_data_get(item);
	cmp = strcmp(item_index, cur_index);

	if (0 == cmp) {
		return item;
	}
	else if (0 < cmp) {
		item_ret = elm_genlist_item_insert_before(clist_d->genlist, &group_index, strdup(cur_index), NULL, item, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
		elm_genlist_item_select_mode_set(item_ret, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
		list_set_group_index_item(clist_d->list_index, cur_index, item_ret);
		return item_ret;
	}
	else {
		next_item = elm_genlist_item_next_get(item);
		item_ret = list_genlist_update_group_item_get(clist_d, next_item, cur_index);
		elm_object_item_del(item);
		return item_ret;
	}
}

static Eina_Bool list_genlist_update(void *data)
{
	PH_FN_CALL;
	int ret;
	char buf[PH_TEXT_MAX_LEN] = {0,};
	char cur_index[10] = {0,};
	char group_letter[10] = {0,};
	Eina_List *list_new;
	Eina_List *l;
	Elm_Object_Item *item = NULL;
	Elm_Object_Item *item_cur = NULL;
	Elm_Object_Item *item_second = NULL;
	Elm_Object_Item *next_item;
	Elm_Object_Item *group_item = NULL;
	ct_person_list *person_info;
	ct_list_data *clist_d = data;

	if (clist_d->list_idler) {
		ecore_idler_del(clist_d->list_idler);
		clist_d->list_idler = NULL;
	}

	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
	}

	p_retvm_if(NULL == clist_d->navi_item, ECORE_CALLBACK_CANCEL, "navi_item is NULL");

	list_new = NULL;
	ret = ct_list_get_contacts(clist_d, &list_new);
	if (0 != ret && ret != CONTACTS_ERROR_NO_DATA) {
		ERR("ct_list_get_contacts() Failed(%d)", ret);
		ctui_list_free_contacts(list_new);
		clist_d->need_update = true;
		return ECORE_CALLBACK_CANCEL;
	}

	if (!list_new && !clist_d->search_str) {
		Evas_Object *base;
		Evas_Object *scroller;
		Evas_Object *box;
		char help_text[PH_TEXT_MAX_LEN] = {0,};

		list_clean_ecore_task(clist_d);

		scroller = elm_scroller_add(clist_d->content_layout);
		p_retvm_if(NULL == scroller, ECORE_CALLBACK_CANCEL, "elm_scroller_add() return NULL");
		elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
		elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
		evas_object_show(scroller);

		box = elm_box_add(scroller);
		p_retvm_if(NULL == box, ECORE_CALLBACK_CANCEL, "elm_box_add() return NULL");
		evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, 0.0);
		evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
		elm_object_content_set(scroller, box);
		evas_object_show(box);

		base = elm_layout_add(box);
		p_retvm_if(NULL == base, ECORE_CALLBACK_CANCEL, "elm_layout_add() return NULL");
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(base, EVAS_HINT_FILL, 0.2);

		elm_layout_file_set(base, NOCONTENT_LAYOUT_EDJ, "nocontent_layout");

		snprintf(help_text, sizeof(help_text), "%s<br>%s", T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS), T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS_HELP_TEXT));
		elm_object_part_text_set(base, "elm.text", help_text);
		elm_box_pack_end(box, base);
		evas_object_size_hint_min_set(box, 720, 650);
		evas_object_show(base);
		clist_d->nocontent = base;
		clist_d->current_rotate = elm_win_rotation_get(clist_d->win);
		evas_object_smart_callback_add(clist_d->win, "wm,rotation,changed", list_resize_contact_view, clist_d);
		elm_object_part_content_set(clist_d->content_layout, "elm.swallow.content", scroller);

		clist_d->genlist = NULL;
		clist_d->group_index[0] = '\0';
		clist_d->group_item = NULL;

		free(clist_d->search_str);
		clist_d->search_str = NULL;
		clist_d->searchbar = NULL;
	}
	else {
		if (NULL == clist_d->genlist) {
			Evas_Object *content = NULL;
			content = list_create_contact_list(clist_d->content_layout, clist_d);

			if (CT_LIST_MODE_EDIT == clist_d->list_mode)
				ctui_list_edit_mode_start(clist_d);

			elm_object_part_content_set(clist_d->content_layout, "elm.swallow.content", content);
			if (!(clist_d->search_str && *clist_d->search_str))
				__list_set_index_letter(clist_d);
		}
		else {
			ctui_list_set_unsweep(clist_d);

			if (clist_d->count_item)
				elm_object_item_del(clist_d->count_item);
		}

		evas_object_smart_callback_del(clist_d->win, "wm,rotation,changed", list_resize_contact_view);

		evas_object_show(clist_d->fast_index);
		elm_object_disabled_set(clist_d->fast_index, EINA_FALSE);

		clist_d->group_index[0] = '\0';
		clist_d->group_item = NULL;

		item_cur = elm_genlist_first_item_get(clist_d->genlist);
		if(item_cur)
			item_second = elm_genlist_item_next_get(item_cur);
		else {
			if(clist_d->base_ab > -1){
				contacts_error_e err = CONTACTS_ERROR_NONE;
				contacts_record_h record = NULL;
				Elm_Object_Item *item;
				int addressbook_mode;
				char title[400] = {0,};
				char cur_account_name[200] = {0,};
				char *cur_ab;

				ctui_get_account_name_by_address_book_id(clist_d->base_ab, cur_account_name, sizeof(cur_account_name));
				if (clist_d->base_ab > 0) {
					err = contacts_db_get_record(_contacts_address_book._uri, clist_d->base_ab, &record);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

					do {
						if (CONTACTS_ERROR_NONE != (err = contacts_record_get_str_p(record, _contacts_address_book.name, &cur_ab))) break;
						if (CONTACTS_ERROR_NONE != (err = contacts_record_get_int(record, _contacts_address_book.mode, &addressbook_mode))) break;
					} while (0);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_value() Failed(%d)", err);

					err = contacts_record_destroy(record, true);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

					if (CONTACTS_ADDRESS_BOOK_MODE_READONLY == addressbook_mode)
						snprintf(title, sizeof(title), "%s", cur_account_name);
					else
						snprintf(title, sizeof(title), "%s %s", cur_account_name, cur_ab);
				}
				else{
					snprintf(title, sizeof(title), "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
				}

				item = elm_genlist_item_append(clist_d->genlist, &group_index, strdup(title), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
				elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
			}
		}

		group_letter[0] = '\0';

		if(ELM_GENLIST_ITEM_GROUP == elm_genlist_item_type_get(item_second))
			item_cur = item_second;

		if (clist_d->search_str && *clist_d->search_str)
			item_cur = elm_genlist_item_next_get(clist_d->search_group_item);

		EINA_LIST_FOREACH(list_new, l, person_info) {
			if (NULL == person_info)
				continue;

			if(!clist_d->search_str) {
				cur_index[0] = '\0';
				if (NULL == person_info->display)
					snprintf(cur_index, sizeof(cur_index), "#");
				else {
					snprintf(cur_index, sizeof(cur_index), "%s", person_info->display_index);
					if (isdigit(person_info->display_index[0]))
						snprintf(cur_index, sizeof(cur_index), "0");
					else if(__is_special_letter(person_info->display_index, clist_d))
						snprintf(cur_index, sizeof(cur_index), "#");
				}

				if ('\0' == group_letter[0] || 0 != strcmp(cur_index, group_letter)) {
					group_item = list_genlist_update_group_item_get(clist_d, item_cur, cur_index);
					if (NULL == group_item) {
						ERR("list_genlist_update_group_item_get() return NULL");
						break;
					}
					snprintf(group_letter, sizeof(group_letter), "%s", cur_index);
					item_cur = elm_genlist_item_next_get(group_item);
				}
			}

			if (NULL == item_cur) {
				if (clist_d->search_str && *clist_d->search_str){
					item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER], person_info, clist_d->search_group_item,
						ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
				}
				else{
					item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_NO_NUMBER], person_info, group_item,
						ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
				}
			}
			else if (ELM_GENLIST_ITEM_NONE == elm_genlist_item_type_get(item_cur)) {
				elm_object_item_data_set(item_cur, person_info);
				item = item_cur;
				item_cur = elm_genlist_item_next_get(item_cur);
			}
			else {
				if (clist_d->search_str && *clist_d->search_str){
					item = elm_genlist_item_insert_before(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER], person_info, clist_d->search_group_item, item_cur,
						ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
				}
				else{
					item = elm_genlist_item_insert_before(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_NO_NUMBER], person_info, group_item, item_cur,
						ELM_GENLIST_ITEM_NONE, clist_d->gl_sel, clist_d->gl_sel_data);
				}
			}
			person_info->item = item;
		}

		if (item_cur) {
			while(next_item = elm_genlist_item_next_get(item_cur)) {
				if(next_item)
					elm_object_item_del(next_item);
				else
					break;
			}
			elm_object_item_del(item_cur);
		}


		if(clist_d->search_str && *clist_d->search_str) {
			if(clist_d->search_group_item) {
				char group_title[PH_TEXT_MAX_LEN];
				snprintf(group_title, sizeof(group_title), "%s (%d)", S_(PH_SYS_BODY_CONTACTS), eina_list_count(list_new));
				elm_object_item_data_set(clist_d->search_group_item, strdup(group_title));
				elm_genlist_item_update(clist_d->search_group_item);
			}
		}

		if(CT_LIST_IN_MULTI_TAB != clist_d->tab_mode && CT_UG_REQUEST_SELECT_PERSON <= clist_d->ug_request) {
			snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);
			elm_object_item_text_set(clist_d->navi_item, buf);
		}
	}

	ctui_list_free_contacts(clist_d->list);
	clist_d->list = list_new;
	list_update_person_list_check_status(clist_d->list, clist_d->list_check);

	if (clist_d->genlist) {
		elm_genlist_realized_items_update(clist_d->genlist);
		if (!(clist_d->search_str && *clist_d->search_str)) {
			if (EINA_FALSE == elm_genlist_decorate_mode_get(clist_d->genlist)) {
				clist_d->count_item = elm_genlist_item_append(clist_d->genlist, &clist_d->itcs[CT_LIST_ITEM_COUNT],
						(void *)clist_d->cnt_list_total, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
				elm_object_item_disabled_set(clist_d->count_item, EINA_TRUE);
			}
		}
	}
	if (clist_d->list_navi_item_update_fn)
		clist_d->list_navi_item_update_fn(clist_d);

	if (clist_d->base_grp)
	   ctui_list_navi_set(CT_LIST_MODE_DEFAULT, clist_d);

	if(clist_d->genlist)
		elm_object_focus_set(clist_d->genlist, EINA_TRUE);

	PH_FN_END;
	return ECORE_CALLBACK_CANCEL;
}

static Eina_Bool list_genlist_update_queue_timer_cb(void *data)
{
	ct_list_data *clist_d = data;
	p_retvm_if(NULL == clist_d, ECORE_CALLBACK_CANCEL, "parameter(clist_d) is NULL");

	if (clist_d->need_update) {
		clist_d->need_update = false;
		list_genlist_update(clist_d);
		return ECORE_CALLBACK_RENEW;
	}
	clist_d->update_queue_timer = NULL;
	return ECORE_CALLBACK_CANCEL;
}

void ctui_list_lazy_update(const char *view_uri, void *data)
{
	int ui_updated;
	ct_list_data *clist_d = data;

	if (clist_d->updating)
		return;
	PH_DBG("ctui_list_lazy_update");
	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
	}

	PH_DBG("is_background = %d", clist_d->is_background);

	if (clist_d->is_background) {
		clist_d->need_update = false;
		clist_d->update_timer = ecore_timer_add(3.0, list_genlist_update, clist_d);
		return;
	}

	ui_updated = (int)evas_object_data_del(clist_d->navi, "ct_list_update_ui");
	if (0 == ui_updated && clist_d->update_queue_timer) {
		clist_d->need_update = true;
	}
	else {
		list_genlist_update(clist_d);
		clist_d->update_queue_timer = ecore_timer_add(1.0, list_genlist_update_queue_timer_cb, clist_d);
		p_warn_if(NULL == clist_d->update_queue_timer, "ecore_timer_add() return NULL");
	}
}

static void list_view_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	int i;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_list_data *clist_d = data;

	ct_list_clear_pdata(clist_d);
	list_clean_ecore_task(clist_d);

	// TODO: Fixme BS when destroy a contact ug
	if(clist_d->genlist) {
		evas_object_del(clist_d->genlist);
		clist_d->genlist = NULL;
	}

	evas_object_smart_callback_del(clist_d->win, "wm,rotation,changed", list_resize_contact_view);

	if (CT_GET_LIST_PERSON_OF_GROUP_ID == clist_d->list_op) {
		err = contacts_db_remove_changed_cb(_contacts_contact._uri, ctui_list_lazy_update, clist_d);
		p_warn_if(CONTACTS_ERROR_NONE, "contacts_db_remove_changed_cb() Failed(%d)", err);
	}

	// if (clist_d->gl_sel_data != clist_d) - 1. Input > Relationship, 2. Details > Link
	if (CT_GET_LIST_VCARD != clist_d->list_op && clist_d->gl_sel_data == clist_d) {
		err = contacts_db_remove_changed_cb(_contacts_person._uri, ctui_list_lazy_update, clist_d);
		p_warn_if(CONTACTS_ERROR_NONE, "contacts_db_remove_changed_cb() Failed(%d)", err);

		err = contacts_db_remove_changed_cb(_contacts_address_book._uri, list_addrbook_changed, clist_d);
		p_warn_if(CONTACTS_ERROR_NONE, "contacts_db_remove_changed_cb() Failed(%d)", err);

	}

	if(clist_d->list) {
		ctui_list_free_contacts(clist_d->list);
		clist_d->list = NULL;
	}
	if(clist_d->list_index) {
		ctui_list_free_index(clist_d->list_index);
		clist_d->list_index = NULL;
	}

	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
	}

	if (clist_d->update_queue_timer) {
		ecore_timer_del(clist_d->update_queue_timer);
		clist_d->update_queue_timer = NULL;
	}

	if (clist_d->popup) {
		evas_object_del(clist_d->popup);
		clist_d->popup = NULL;
	}

	if(clist_d->nocontent) {
		evas_object_del(clist_d->nocontent);
		clist_d->nocontent = NULL;
	}

	if (clist_d->th) {
		elm_theme_extension_del(clist_d->th, CONTACTS_EDJ);
		elm_theme_free(clist_d->th);
		clist_d->th = NULL;
	}

	if (clist_d->child_ug) {
		ug_destroy(clist_d->child_ug);
		clist_d->child_ug = NULL;
	}

	if(clist_d->list_check) {
		eina_list_free(clist_d->list_check);
		clist_d->list_check = NULL;
	}

	evas_object_data_del(clist_d->navi, "gadd_list_d");
	evas_object_data_del(clist_d->navi, "favor_list_d");

	if (clist_d->acc_types) {
		int count = clist_d->acc_count;
		for (i=0;i<count;i++)
			free(clist_d->acc_types[i].icon_path);
		free(clist_d->acc_types);
		clist_d->acc_types = NULL;
	}

	if(clist_d->search_str) {
		free(clist_d->search_str);
		clist_d->search_str = NULL;
	}
	if(clist_d->sub_title) {
		free(clist_d->sub_title);
		clist_d->sub_title = NULL;
	}
	if(clist_d->title) {
		free(clist_d->title);
		clist_d->title = NULL;
	}

	if(!clist_d->ug) {
		PH_DBG("no ug");
		free(clist_d);
	}
	else {
		PH_DBG("view destroy called earlier than ug destroy");
		clist_d->ug = NULL;
	}
}

char* ctui_list_get_highlight_text(ct_list_data *clist_d, const char *src)
{
	int len_search;
	int len_display;
	char *pos;
	char display[PH_TEXT_MAX_LEN] = {0, };

	if (NULL == clist_d->search_str || '\0' == clist_d->search_str[0])
		return strdup(src);

	len_display = strlen(src);
	len_search = strlen(clist_d->search_str);
	pos = strcasestr(src, clist_d->search_str);

	if (0 < pos) {
		char first[PH_TEXT_MAX_LEN] = {0};
		char middle[PH_TEXT_MAX_LEN] = {0};

		snprintf(first, (int)(pos - src)+1, "%s", src);
		snprintf(middle, len_search+1, "%s", (char *)pos);

		snprintf(display, sizeof(display), "%s<match>%s</>%s", first, middle, (char *)(pos + len_search));
		return strdup(display);
	}
	return strdup(src);
}

char* ct_list_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	ct_person_list *person_info = data;
	int count = eina_list_count(person_info->addressbook_id_list);
	p_retvm_if(NULL == person_info, NULL, "The parameter(data) is NULL");

	if ((person_info->status_str && 0 == strcmp(part, "elm.text.1")) ||
		NULL == person_info->status_str && 0 == strcmp(part, "elm.text")  ) {
		if (person_info->display && *person_info->display) {
			ct_list_data *clist_d = evas_object_data_get(obj, "clist_d");
			p_retvm_if(NULL == clist_d, strdup(person_info->display), "evas_object_data_get() return NULL");
			return ctui_list_get_highlight_text(clist_d, person_info->display);
		}
		else
			return strdup(T_(CT_GET_TEXT_ERR, CTTEXT_NONAME));
	}
	else if ( person_info->status_str && 0 == strcmp(part, "elm.text.2") ) {
		return strdup(person_info->status_str);
	}

	return NULL;
}

char* ct_list_count_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[PH_TEXT_SHORT_LEN];
	int count = (int)data;
	if(count == 1)
		return strdup(T_(CT_GET_TEXT_BASIC,CTTEXT_1_CONTACT));
	else
	{
		snprintf(buf, sizeof(buf), T_(CT_GET_TEXT_BASIC, CTTEXT_COUNT_CONTACTS), count);
		return strdup(buf);
	}
}


Evas_Object* ct_list_create_contact_layout(ct_list_data *clist_d)
{
	PH_FN_CALL;
	int ret;
	Elm_Theme *th;
	Evas_Object *base;
	Evas_Object *layout;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	th = elm_theme_new();
	elm_theme_ref_set(th, NULL);
	elm_theme_extension_add(th, LIST_THEME_EDJ);
	clist_d->th = th;

	base = elm_layout_add(clist_d->navi);
	p_retvm_if(NULL == base, NULL, "elm_layout_add() return NULL");
	elm_layout_file_set(base, SELECTION_INFO_EDJ, "selectioninfo_layout");

	layout = phone_create_base_layout(base, false);
	elm_object_part_content_set(base, "contents", layout);
	list_fill_contact_view(layout, clist_d);
	clist_d->content_layout = layout;

	// if (clist_d->gl_sel_data != clist_d) - 1. Input > Relationship, 2. Details > Link
	if (CT_GET_LIST_VCARD != clist_d->list_op && clist_d->gl_sel_data == clist_d) {
		err = contacts_db_add_changed_cb(_contacts_person._uri, ctui_list_lazy_update, clist_d);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);

		err = contacts_db_add_changed_cb(_contacts_address_book._uri, list_addrbook_changed, clist_d);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);

		if (CT_GET_LIST_PERSON_OF_GROUP_ID == clist_d->list_op) {
			err = contacts_db_add_changed_cb(_contacts_contact._uri, ctui_list_lazy_update, clist_d);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);
		}
	}

	evas_object_event_callback_add(base, EVAS_CALLBACK_DEL, list_view_delete_cb, clist_d);

	PH_DBG("##### [TSP_TEST] lauching contact application #####");
	PH_FN_END;
	return base;
}

