/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include "phone.h"
#include "ct-detail-history.h"
#include "ct-detail-utils.h"
#include "ct-detail-view.h"

typedef struct
{
	int plog_id;
	int plog_type;
	int plog_time;
	int plog_duration;
	int plog_msgid;
	int group_index;
	char *plog_number;
	char *plog_shortmsg;
	Eina_Bool checked;
}ct_detail_history_info;

enum HISTORY_ITC_TYPE {
	HISTORY_ITC_NUM_UTIL_BTN = 0,
	HISTORY_ITC_HISTORY,
	HISTORY_ITC_DATE,
	HISTORY_ITC_SELECT_ALL,
	HISTORY_ITC_MAX,
};

static Elm_Genlist_Item_Class history_itcs[] = {
	{.item_style="detail.3icon"},
	{.item_style="history_list"},
	{.item_style="grouptitle"},
	{.item_style="1icon"},
};

static void detail_history_check_del(ct_history_data *chistory_d);
static void detail_history_set_navi(ct_history_data *chistory_d, int mode);
static Eina_Bool detail_history_refresh_view(void *data);

void ct_detail_history_changed_cb(const char *view_uri, void *data)
{
	ct_history_data *chistory_d = data;

	if (chistory_d->updating) return;
	if (chistory_d->idler) {
		ecore_idler_del(chistory_d->idler);
		chistory_d->idler = NULL;
	}
	chistory_d->idler = ecore_idler_add(detail_history_refresh_view, chistory_d);
}

static void detail_history_delete_records_cb(int err, void *data)
{
	ct_history_data *chistory_d = data;
	ph_progress_info *p_info = NULL;

	p_retm_if(NULL == chistory_d, "Invalid parameter(chistory_d is NULL)");
	p_info = chistory_d->p_info;
	p_retm_if(NULL == p_info, "Invalid parameter(p_info is NULL)");

	if (p_info->update_ids) {
		free(p_info->update_ids);
		p_info->update_ids = NULL;
	}

	if (CONTACTS_ERROR_NONE != err || p_info->alive == false) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
		free(p_info);
		chistory_d->p_info = NULL;
		chistory_d->updating = false;
		elm_genlist_decorate_mode_set(chistory_d->history_genlist, EINA_FALSE);
		chistory_d->view_mode = DETAIL_VIEW_MODE_HISTORY;
		ct_detail_history_changed_cb(_contacts_phone_log._uri, chistory_d);
		return;
	}
	else if (p_info->cnt == p_info->cnt_checked_total) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
		free(p_info);
		chistory_d->p_info = NULL;
		chistory_d->updating = false;
		elm_genlist_decorate_mode_set(chistory_d->history_genlist, EINA_FALSE);
		chistory_d->view_mode = DETAIL_VIEW_MODE_HISTORY;
		ct_detail_history_changed_cb(_contacts_phone_log._uri, chistory_d);
		return;
	}
	else {
		detail_history_check_del(chistory_d);
	}
}

static void detail_history_check_del(ct_history_data *chistory_d)
{
	const int DETAIL_BULK_LENGTH = 50;
	char percent_total[PH_TEXT_MAX_LEN+1] = {0, };
	char percent[PH_TEXT_MAX_LEN+1] = {0, };
	int *ids = NULL;
	double value = 0.0;
	int count = 0;
	int base_count = 0;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ph_progress_info *p_info = chistory_d->p_info;
	Eina_List *l;
	ct_detail_history_info *l_info;

	if (p_info->cnt_checked_total - p_info->cnt < DETAIL_BULK_LENGTH)
		ids = calloc(1, sizeof(int) * (p_info->cnt_checked_total - p_info->cnt));
	else
		ids = calloc(1, sizeof(int) * DETAIL_BULK_LENGTH);
	p_retm_if(NULL == ids, "calloc() return NULL");

	EINA_LIST_FOREACH(chistory_d->list, l, l_info) {
		if (!l_info) continue;
		if (TRUE == l_info->checked) {
			if (base_count < p_info->cnt) {
				base_count++;
				continue;
			}

			ids[count++] = l_info->plog_id;
			if (DETAIL_BULK_LENGTH == count || (count + p_info->cnt) == p_info->cnt_checked_total) {
				p_info->cnt += count;
				break;
			}
		}
	}
	value = (double)p_info->cnt / (double)p_info->cnt_checked_total;
	elm_progressbar_value_set(p_info->progressbar, value);
	snprintf(percent, sizeof(percent), "%d%%", (int)(100.0 * (double)p_info->cnt/(double)p_info->cnt_checked_total));
	snprintf(percent_total, sizeof(percent_total), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", percent_total);

	p_info->update_ids = ids;
	err = contacts_db_delete_records_async(_contacts_phone_log._uri, ids, count, detail_history_delete_records_cb, chistory_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_delete_records_async() Failed(%d)", err);
}

static void detail_history_del_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = data;
	ph_progress_info *p_info;

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc return NULL");

	chistory_d->updating = true;

	chistory_d->p_info = p_info;
	p_info->cnt_checked_total = chistory_d->cnt_checked;
	p_info->alive = true;
	p_info->result = TRUE;
	p_info->popup = phone_progressbar_popup(chistory_d->win, p_info, T_(PH_GET_TEXT_BASIC, PHTEXT_DELETING_LOGS));
	detail_history_check_del(chistory_d);
}

static void detail_history_all_items_checked_set(Eina_List *list,
		Eina_Bool checked)
{
	Eina_List *l;
	ct_detail_history_info *info;

	EINA_LIST_FOREACH(list, l, info) {
		if (NULL == info) {
			ERR("elm_object_item_data_get() return NULL");
			continue;
		}
		info->checked = checked;
	}
}

static void detail_history_back_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = (ct_history_data *)data;
	elm_naviframe_item_pop(chistory_d->navi);
}

static void detail_history_back_btn(ct_history_data *chistory_d)
{
	Evas_Object *back_btn;
	back_btn = elm_button_add(chistory_d->navi);
	p_retm_if(NULL == back_btn, "elm_button_add() return NULL");
	elm_object_style_set(back_btn, "naviframe/back_btn/default");
	elm_object_item_part_content_set(chistory_d->navi_item, "prev_btn", back_btn);
	evas_object_smart_callback_add(back_btn, "clicked", detail_history_back_btn_cb, chistory_d);
}

static void detail_history_edit_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = data;

	chistory_d->view_mode = DETAIL_VIEW_MODE_CHECK;

	if (chistory_d->popup)
		evas_object_del(chistory_d->popup);

	elm_genlist_decorate_mode_set(chistory_d->history_genlist, EINA_TRUE);

	chistory_d->select_all_item = elm_genlist_item_prepend(chistory_d->history_genlist, &history_itcs[HISTORY_ITC_SELECT_ALL],
		chistory_d, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	elm_genlist_item_select_mode_set(chistory_d->select_all_item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	phone_show_notify(chistory_d->layout, T_(PH_GET_TEXT_BASE_OP, PHTEXT_SELECT_LOGS));
	detail_history_set_navi(chistory_d, DETAIL_VIEW_MODE_CHECK);
}

static void detail_history_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = data;
	evas_object_del(chistory_d->popup);
	chistory_d->popup = NULL;
	elm_object_tree_focus_allow_set(chistory_d->navi, EINA_TRUE);
}

static void detail_history_more_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	int rotate;
	ct_history_data *chistory_d = data;
	int x,y,w,h;

	if (chistory_d->popup)
		detail_history_popup_hide_cb(chistory_d, NULL, NULL);

	chistory_d->popup = elm_ctxpopup_add(chistory_d->navi);
	p_retm_if(NULL == chistory_d->popup, "elm_ctxpopup_add() return NULL");

	rotate = (int)evas_object_data_get(chistory_d->navi, "phone_rotate");
	if (0 == rotate || 180 == rotate)
		elm_ctxpopup_direction_priority_set(chistory_d->popup, ELM_CTXPOPUP_DIRECTION_DOWN, ELM_CTXPOPUP_DIRECTION_UP,
				ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_RIGHT);

	evas_object_smart_callback_add(chistory_d->popup, "dismissed", detail_history_popup_hide_cb, chistory_d);

	evas_object_geometry_get(chistory_d->ctrl_button, &x, &y, &w, &h);
	elm_ctxpopup_item_append(chistory_d->popup, S_(PH_SYS_SK_DELETE), NULL,
	detail_history_edit_btn_clicked_cb, chistory_d);
	evas_object_move(chistory_d->popup, w/2, y+h/2);
	evas_object_show(chistory_d->popup);
}

static void detail_history_cancel_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = data;
	phone_hide_notify(chistory_d->layout);

	if (chistory_d->select_all_item) {
		elm_object_item_del(chistory_d->select_all_item);
		chistory_d->select_all_item = NULL;
	}
	detail_history_all_items_checked_set(chistory_d->list, EINA_FALSE);
	elm_genlist_decorate_mode_set(chistory_d->history_genlist, EINA_FALSE);
	detail_history_set_navi(chistory_d, DETAIL_VIEW_MODE_HISTORY);
}

static void detail_history_set_navi(ct_history_data *chistory_d, int mode)
{
	Evas_Object *cancel_button = NULL;

	chistory_d->cnt_checked = 0;
	chistory_d->view_mode = mode;
	p_retm_if(NULL == chistory_d->navi_item, "navi item is NULL");

	phone_navi_item_clear(chistory_d->navi_item);

	if (DETAIL_VIEW_MODE_HISTORY == mode) {
		detail_history_back_btn(chistory_d);
		chistory_d->ctrl_button = ctui_create_more_btn(chistory_d->navi, detail_history_more_btn_clicked_cb, chistory_d);
		elm_object_item_part_content_set(chistory_d->navi_item, "toolbar_more_btn", chistory_d->ctrl_button);
		if (chistory_d->list_count <= 0)
			elm_object_disabled_set(chistory_d->ctrl_button, EINA_TRUE);
	}
	else if (DETAIL_VIEW_MODE_CHECK == mode) {
		chistory_d->ctrl_button = phone_create_toolbar_btn(chistory_d->navi, S_(PH_SYS_SK_DELETE), detail_history_del_btn_clicked_cb, chistory_d);
		elm_object_item_part_content_set(chistory_d->navi_item, "toolbar_button1", chistory_d->ctrl_button);
		elm_object_disabled_set(chistory_d->ctrl_button, EINA_TRUE);

		cancel_button = phone_create_toolbar_btn(chistory_d->navi, S_(PH_SYS_SK_CANCEL), detail_history_cancel_btn_cb, chistory_d);
		elm_object_item_part_content_set(chistory_d->navi_item, "toolbar_button2", cancel_button);
	}
}

static void detail_history_check_update_selection_info(ct_history_data *chistory_d)
{
	if (chistory_d->cnt_checked == 0)
		phone_show_notify(chistory_d->layout, T_(PH_GET_TEXT_BASE_OP, PHTEXT_SELECT_LOGS));
	else {
		char info_text[PH_TEXT_SHORT_LEN]={0};
		snprintf(info_text, sizeof(info_text), "%s (%d)", S_(PH_SYS_POP_SELECTED),
				chistory_d->cnt_checked);
		phone_show_notify(chistory_d->layout, info_text);
	}
}

static void detail_history_check_update_button_status(ct_history_data *chistory_d)
{
	Eina_Bool disabled;
	p_retm_if(NULL == chistory_d, "parameter is NULL");
	p_retm_if(NULL == chistory_d->ctrl_button, "ctrl_item is NULL");

	if (0 == chistory_d->list_count)
		disabled = EINA_TRUE;
	else if (chistory_d->history_genlist && elm_genlist_decorate_mode_get(chistory_d->history_genlist)) {
		if (0 < chistory_d->cnt_checked)
			disabled = EINA_FALSE;
		else
			disabled = EINA_TRUE;
	}
	else
		disabled = EINA_FALSE;

	elm_object_disabled_set(chistory_d->ctrl_button, disabled);
}

static void detail_history_check_select_all(void *data, Eina_Bool checked)
{
	Eina_List *l;
	Elm_Object_Item *item;
	ct_detail_history_info *l_info;
	ct_history_data *chistory_d = data;

	if (checked)
		chistory_d->cnt_checked = chistory_d->list_count;
	else
		chistory_d->cnt_checked = 0;

	EINA_LIST_FOREACH(chistory_d->list, l, l_info) {
		if (!l_info) continue;
		l_info->checked = checked;
	}

	item = elm_genlist_item_next_get(chistory_d->select_all_item);
	while (item) {
		elm_genlist_item_update(item);
		item = elm_genlist_item_next_get(item);
	}

	detail_history_check_update_selection_info(chistory_d);
	detail_history_check_update_button_status(chistory_d);
}

static void detail_history_append_separate_date_index(ct_history_data *chistory_d, void *data, int index)
{
	ct_detail_history_info *l_info = data;
	Elm_Object_Item *item;
	int type = phone_detail_log_get_date_type(l_info->plog_time, l_info->plog_duration);

	if (0 == index || chistory_d->pre_date_type != type) {
		chistory_d->pre_date_type = type;
		l_info->group_index = type;
		item = elm_genlist_item_append(chistory_d->history_genlist, &history_itcs[HISTORY_ITC_DATE], data, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}
}
static void detail_history_gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = event_info;
	ct_detail_history_info *l_info = data;
	ct_history_data *chistory_d = evas_object_data_get(obj, "chistory_d");

	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (elm_genlist_decorate_mode_get(chistory_d->history_genlist)) {
		l_info->checked = !l_info->checked;
		if (l_info->checked)
			chistory_d->cnt_checked++;
		else
			chistory_d->cnt_checked--;

		elm_genlist_item_fields_update(event_info, "elm.edit.icon.1", ELM_GENLIST_ITEM_FIELD_CONTENT);

		if (chistory_d->select_all_layout) {
			Evas_Object *check;
			check = elm_object_part_content_get(chistory_d->select_all_layout, "elm.icon");
			if (chistory_d->cnt_checked == chistory_d->list_count)
				elm_check_state_set(check, EINA_TRUE);
			else
				elm_check_state_set(check, EINA_FALSE);
		}
		detail_history_check_update_selection_info(chistory_d);
		detail_history_check_update_button_status(chistory_d);
	}
}

static void detail_history_fill_genlist(ct_history_data *chistory_d)
{
	Evas_Object *genlist;
	Evas_Object *layout;
	Eina_List *l;
	int count;
	ct_detail_history_info *l_info;

	layout = elm_object_part_content_get(chistory_d->layout, "contents");

	if (NULL == chistory_d->history_genlist) {
		genlist = elm_genlist_add(layout);
		elm_object_theme_set(genlist, chistory_d->th);
		chistory_d->history_genlist = genlist;
		evas_object_data_set(genlist, "chistory_d", chistory_d);
		elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	}
	else {
		genlist = chistory_d->history_genlist;
		elm_genlist_clear(genlist);
	}

	count = 0;
	EINA_LIST_FOREACH(chistory_d->list, l, l_info) {
		detail_history_append_separate_date_index(chistory_d, l_info, count);
		elm_genlist_item_append(genlist, &history_itcs[HISTORY_ITC_HISTORY], l_info,  NULL,
				ELM_GENLIST_ITEM_NONE, detail_history_gl_sel, l_info);
		count++;
	}
}

static void detail_history_check_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_history_data *chistory_d = data;

	if (elm_check_state_get(obj))
		chistory_d->cnt_checked++;
	else
		chistory_d->cnt_checked--;

	if (chistory_d->select_all_layout) {
		Evas_Object *check;
		check = elm_object_part_content_get(chistory_d->select_all_layout, "elm.icon");
		if (chistory_d->cnt_checked == chistory_d->list_count)
			elm_check_state_set(check, EINA_TRUE);
		else
			elm_check_state_set(check, EINA_FALSE);
	}
	detail_history_check_update_selection_info(chistory_d);
	detail_history_check_update_button_status(chistory_d);
}

static char* detail_history_gl_sep_date_label_get(void *data, Evas_Object *obj, const char *part)
{
	ct_detail_history_info * history_info = data;
	const char *when;
	int index = history_info->group_index;

	if (0 == strcmp(part, "elm.text")) {
		char *date_temp;
		ct_history_data *chistory_d;

		if (0 == index)
			when = S_(PH_SYS_BODY_TODAY);
		else if (1 == index)
			when = S_(PH_SYS_BODY_YESTERDAY);
		else
			when = S_(PH_SYS_BODY_PREVIOUS);

		chistory_d = evas_object_data_get(obj, "chistory_d");
		date_temp = phone_detail_log_get_date(chistory_d->hformatter, history_info->plog_time);
		if (date_temp) {
			char buf[PATH_MAX]={0};
			snprintf(buf, sizeof(buf), " %s (%s)", when, date_temp);
			free(date_temp);
			return strdup(buf);
		}
		else
			return strdup(when);
	}
	else
		return NULL;
}

static Evas_Object *detail_history_gl_select_all_icon_get(void *data, Evas_Object *obj,
		const char *part)
{
	Evas_Object *check;
	ct_history_data *chistory_d = data;

	p_retvm_if(NULL == chistory_d, NULL, "evas_object_data_get() return NULL");
	if (0 == strcmp(part, "elm.icon")) {
		chistory_d->select_all_layout = phone_create_select_all(chistory_d->history_genlist, S_(PH_SYS_BODY_SELECT_ALL), detail_history_check_select_all, chistory_d);
		check = elm_object_part_content_get(chistory_d->select_all_layout, "elm.icon");
		if (chistory_d->cnt_checked == chistory_d->list_count)
			elm_check_state_set(check, EINA_TRUE);
		return chistory_d->select_all_layout;
	}
	return NULL;
}

static void detail_history_gl_select_all_del(void *data, Evas_Object *obj)
{
	ct_history_data *chistory_d = data;
	chistory_d->select_all_layout = NULL;
}

static Evas_Object *detail_history_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	ct_detail_history_info * l_detail_info = data;
	char buf[PATH_MAX]={0};
	Evas_Object *ic;

	if (0 == strcmp(part, "elm.icon.2")) {
		char *made_data = phone_detail_log_get_icon_type(l_detail_info->plog_type);
		snprintf(buf, sizeof(buf), "%s", made_data);
		free(made_data);
	}
	else if (0 == strcmp(part, "elm.icon.1")) {
		char *made_data = phone_detail_log_get_icon_path_with_action(l_detail_info->plog_type);
		snprintf(buf, sizeof(buf), "%s", made_data);
		free(made_data);
	}
	else if (0 == strcmp(part, "elm.edit.icon.1")) {
		ct_history_data *chistory_d = evas_object_data_get(obj, "chistory_d");
		Evas_Object *check;
		check = elm_check_add(obj);
		p_retvm_if(NULL == check, NULL, "elm_check_add() return NULL");
		elm_check_state_pointer_set(check, &l_detail_info->checked);
		evas_object_propagate_events_set(check, EINA_FALSE);
		evas_object_smart_callback_add(check, "changed", detail_history_check_changed_cb, chistory_d);
		return check;
	}
	else
		return NULL;
	ic = elm_icon_add(obj);
	p_retvm_if(NULL == ic, NULL, "elm_icon_add() return NULL");
	elm_icon_file_set(ic, buf, NULL);
	evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
	return ic;
}

static char* detail_history_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	ct_detail_history_info * history_info = data;
	ct_history_data *chistory_d;
	char *time;
	char *duration;
	char buf[PATH_MAX] = {0};
	chistory_d = evas_object_data_get(obj, "chistory_d");

	if (0 == strcmp(part, "elm.text.1")) {
		snprintf(buf, sizeof(buf), "%s", history_info->plog_number);
		return strdup(buf);
	}
	else if (0 == strcmp(part, "elm.text.2")) {
		if (TRUE == phone_detail_log_is_call_ing_type(history_info->plog_type)) {
			duration = phone_detail_log_get_duration_from_number(history_info->plog_duration);
			snprintf(buf, sizeof(buf), "%s", duration);
			free(duration);
			return strdup(buf);
		}
		else if (TRUE == phone_detail_log_is_sms_type(history_info->plog_type)
				|| TRUE == phone_detail_log_is_mms_type(history_info->plog_type)) {
			if (history_info->plog_shortmsg && *history_info->plog_shortmsg) {
				snprintf(buf, sizeof(buf), "%s", history_info->plog_shortmsg);
				return strdup(buf);
			}
		}
		else if (TRUE == phone_detail_log_is_email_type(history_info->plog_type)) {
			if (history_info->plog_shortmsg && *history_info->plog_shortmsg) {
				snprintf(buf, sizeof(buf), "%s", history_info->plog_shortmsg);
				return strdup(buf);
			}
		}
	}
	else if (0 == strcmp(part, "elm.text.3")) {
		time = phone_detail_log_get_time(chistory_d->hformatter, history_info->plog_time);
		snprintf(buf, sizeof(buf), "%s", time);
		free(time);
		return strdup(buf);
	}
	return NULL;
}

void ct_detail_history_create_itcs()
{
	history_itcs[HISTORY_ITC_SELECT_ALL].func.content_get = detail_history_gl_select_all_icon_get;
	history_itcs[HISTORY_ITC_SELECT_ALL].func.del = detail_history_gl_select_all_del;
	history_itcs[HISTORY_ITC_SELECT_ALL].decorate_all_item_style = NULL;

	history_itcs[HISTORY_ITC_NUM_UTIL_BTN].func.content_get = ct_detail_gl_number_util_icon_get;

	history_itcs[HISTORY_ITC_DATE].func.text_get = detail_history_gl_sep_date_label_get;

	history_itcs[HISTORY_ITC_HISTORY].func.text_get = detail_history_gl_label_get;
	history_itcs[HISTORY_ITC_HISTORY].func.content_get = detail_history_gl_icon_get;
	history_itcs[HISTORY_ITC_HISTORY].decorate_all_item_style = "edit_default";
}

void ct_detail_history_load_data(ct_history_data *chistory_d)
{
	Eina_List *list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list_phonelog = NULL;
	contacts_record_h record_phonelog = NULL;
	int count = 0;

	err = phone_cts_get_list(_contacts_person_phone_log._uri, _contacts_person_phone_log.person_id, chistory_d->person_id, &list_phonelog);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_list() Failed(%d)", err);

	while (CONTACTS_ERROR_NONE == err) {
		char *msg = NULL;
		char *number;
		ct_detail_history_info *history_info;

		err = contacts_list_get_current_record_p(list_phonelog, &record_phonelog);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_get_current_record_p() Failed(%d)", err);
			break;
		}
		history_info = calloc(1, sizeof(ct_detail_history_info));

		err = contacts_record_get_str_p(record_phonelog, _contacts_person_phone_log.address, &number);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(address) Failed(%d)", err);
		history_info->plog_number = SAFE_STRDUP(number);
		err = contacts_record_get_int(record_phonelog, _contacts_person_phone_log.log_id, &(history_info->plog_id));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(id) Failed(%d)", err);
		err = contacts_record_get_int(record_phonelog, _contacts_person_phone_log.log_type, &(history_info->plog_type));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(log_type) Failed(%d)", err);
		err = contacts_record_get_int(record_phonelog, _contacts_person_phone_log.log_time, &(history_info->plog_time));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(log_time) Failed(%d)", err);
		err = contacts_record_get_int(record_phonelog, _contacts_person_phone_log.extra_data1, &(history_info->plog_duration));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(duration) Failed(%d)", err);
		err = contacts_record_get_str_p(record_phonelog, _contacts_person_phone_log.extra_data2, &msg);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(shortmsg) Failed(%d)", err);
		if (msg) {
			int i;
			char temp[PH_TEXT_MAX_LEN];
			for (i=0;i<strlen(msg);i++) {
				if (msg[i] =='\n')
					temp[i] = ' ';
				else
					temp[i] = msg[i];
			}
			temp[i] = '\0';
			history_info->plog_shortmsg = strdup(temp);
		}
		list = eina_list_append(list, (void*)history_info);
		count++;
		err = contacts_list_next(list_phonelog);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}

	chistory_d->list = list;
	chistory_d->list_count = count;
}

static void detail_history_view_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout;
	Evas_Object *scroller;
	Evas_Object *box;
	int rotate;
	ct_history_data *chistory_d = data;

	if (0 < chistory_d->list_count)
		return;

	rotate = (int)evas_object_data_get(chistory_d->navi, "phone_rotate");

	if (90 == rotate || 270 == rotate) {
		layout = elm_object_part_content_get(chistory_d->layout, "contents");
		p_retm_if(NULL == layout, "elm_object_part_content_get() return NULL");
		scroller = elm_object_part_content_get(layout, "elm.swallow.content");
		p_retm_if(NULL == scroller, "elm_object_part_content_get() return NULL");
		box = elm_object_content_get(scroller);
		p_retm_if(NULL == box, "elm_object_content_get() return NULL");
		elm_box_unpack(box, chistory_d->noc);
		p_retm_if(NULL == chistory_d->noc, "elm_box_unpack() return NULL");
		evas_object_size_hint_weight_set(chistory_d->noc, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(chistory_d->noc, EVAS_HINT_FILL, 0.0);
		elm_box_pack_end(box, chistory_d->noc);
		elm_object_content_set(scroller, box);
		evas_object_size_hint_min_set(box, 720, 650);
		elm_object_part_content_set(layout, "elm.swallow.content", scroller);
	}
	else if (0 == rotate || 180 == rotate) {
		layout = elm_object_part_content_get(chistory_d->layout, "contents");
		p_retm_if(NULL == layout, "elm_object_part_content_get() return NULL");
		scroller = elm_object_part_content_get(layout, "elm.swallow.content");
		p_retm_if(NULL == scroller, "elm_object_part_content_get() return NULL");
		box = elm_object_content_get(scroller);
		p_retm_if(NULL == box, "elm_object_content_get() return NULL");
		elm_box_unpack(box, chistory_d->noc);
		p_retm_if(NULL == chistory_d->noc, "elm_box_unpack() return NULL");
		evas_object_size_hint_weight_set(chistory_d->noc, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(chistory_d->noc, EVAS_HINT_FILL, 0.2);
		elm_box_pack_end(box, chistory_d->noc);
		elm_object_content_set(scroller, box);
		evas_object_size_hint_min_set(box, 720, 650);
		elm_object_part_content_set(layout, "elm.swallow.content", scroller);
	}
}

static Eina_Bool detail_history_refresh_view(void *data)
{
	ct_history_data *chistory_d = data;
	Evas_Object *layout;

	phone_hide_notify(chistory_d->layout);
	if (chistory_d->select_all_item && DETAIL_VIEW_MODE_CHECK != chistory_d->view_mode) {
		elm_object_item_del(chistory_d->select_all_item);
		chistory_d->select_all_item = NULL;
	}
	ct_detail_history_load_data(chistory_d);

	layout = elm_object_part_content_get(chistory_d->layout, "contents");

	if (0 < chistory_d->list_count) {
		detail_history_fill_genlist(chistory_d);
		elm_object_part_content_set(layout, "elm.swallow.content", chistory_d->history_genlist);
	}
	else {
		if (!chistory_d->scroller) {
			Evas_Object *scroller;
			Evas_Object *nocontent;
			Evas_Object *box;
			char buf[PH_TEXT_MAX_LEN] = {0,};

			scroller = elm_scroller_add(layout);
			p_retvm_if(NULL == scroller, ECORE_CALLBACK_CANCEL, "elm_scroller_add() return NULL");
			elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
			elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);

			box = elm_box_add(scroller);
			p_retvm_if(NULL == box, ECORE_CALLBACK_CANCEL, "elm_box_add() return NULL");
			evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, 0.0);
			evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
			elm_object_content_set(scroller, box);
			evas_object_show(box);

			nocontent = elm_layout_add(box);
			p_retvm_if(NULL == nocontent, ECORE_CALLBACK_CANCEL, "elm_layout_add() return NULL");
			evas_object_size_hint_weight_set(nocontent, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			evas_object_size_hint_align_set(nocontent, EVAS_HINT_FILL, 0.2);
			elm_layout_file_set(nocontent, NOCONTENT_LAYOUT_EDJ, "nocontent_layout");
			snprintf(buf, sizeof(buf), "%s<br>%s<br>", T_(CT_GET_TEXT_BASIC, CTTEXT_NO_HISTORIES), T_(CT_GET_TEXT_ERR, CTTEXT_NO_HISTORIES_HELP_TEXT));
			elm_box_pack_end(box, nocontent);
			evas_object_size_hint_min_set(box, 720, 650);
			evas_object_show(nocontent);
			elm_object_part_text_set(nocontent, "elm.text", buf);
			chistory_d->scroller = scroller;
			chistory_d->noc = nocontent;
		}
		evas_object_show(chistory_d->scroller);
		elm_object_part_content_set(layout, "elm.swallow.content", chistory_d->scroller);
	}

	detail_history_set_navi(chistory_d, DETAIL_VIEW_MODE_HISTORY);
	chistory_d->idler = NULL;
	return ECORE_CALLBACK_CANCEL;
}

static void detail_history_free_list(ct_history_data *chistory_d)
{
	ct_detail_history_info *l_info;

	EINA_LIST_FREE(chistory_d->list, l_info) {
		if (NULL == l_info) continue;
		free(l_info->plog_number);
		free(l_info->plog_shortmsg);
		free(l_info);
	}
	chistory_d->list = NULL;
}

static void detail_history_view_delete_cb(void *data, Evas *e, Evas_Object *obj,
		void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_history_data *chistory_d = data;

	if (0 < chistory_d->list_count)
		detail_history_free_list(chistory_d);

	if (chistory_d->history_genlist)
		evas_object_del(chistory_d->history_genlist);
	if (chistory_d->noc)
		evas_object_del(chistory_d->noc);
	if (chistory_d->scroller)
		evas_object_del(chistory_d->scroller);
	if (chistory_d->popup)
		evas_object_del(chistory_d->popup);
	if (chistory_d->th) {
		elm_theme_extension_del(chistory_d->th, DETAIL_THEME_EDJ);
		elm_theme_free(chistory_d->th);
	}

	err = contacts_db_remove_changed_cb(_contacts_phone_log._uri, ct_detail_history_changed_cb, chistory_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_remove_changed_cb() Failed(%d)", err);
	evas_object_event_callback_del(chistory_d->layout, EVAS_CALLBACK_RESIZE, detail_history_view_resize);

	free(chistory_d);

}

void ct_detail_history_fill_layout(ct_history_data *chistory_d)
{
	Evas_Object *base;
	Evas_Object *layout;

	base = elm_layout_add(chistory_d->navi);
	p_retm_if(NULL == base, "elm_layout_add() return NULL");

	elm_layout_file_set(base, SELECTION_INFO_EDJ, "selectioninfo_layout");
	layout = phone_create_base_layout(base, false);
	elm_object_part_content_set(base, "contents", layout);

	chistory_d->layout = base;

	evas_object_event_callback_add(chistory_d->layout, EVAS_CALLBACK_RESIZE, detail_history_view_resize, chistory_d);

	if (0 < chistory_d->list_count) {
		detail_history_fill_genlist(chistory_d);
		elm_object_part_content_set(layout, "elm.swallow.content", chistory_d->history_genlist);
	}
	else {
		if (!chistory_d->scroller) {
			Evas_Object *scroller;
			Evas_Object *nocontent;
			Evas_Object *box;
			char buf[PH_TEXT_MAX_LEN] = {0,};

			scroller = elm_scroller_add(layout);
			p_retm_if(NULL == scroller, "elm_scroller_add() return NULL");
			elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
			elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);

			box = elm_box_add(scroller);
			p_retm_if(NULL == box, "elm_box_add() return NULL");
			evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, 0.0);
			evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
			elm_object_content_set(scroller, box);
			evas_object_show(box);

			nocontent = elm_layout_add(box);
			p_retm_if(NULL == nocontent, "elm_layout_add() return NULL");
			evas_object_size_hint_weight_set(nocontent, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			evas_object_size_hint_align_set(nocontent, EVAS_HINT_FILL, 0.2);
			elm_layout_file_set(nocontent, NOCONTENT_LAYOUT_EDJ, "nocontent_layout");
			snprintf(buf, sizeof(buf), "%s<br>%s<br>", T_(CT_GET_TEXT_BASIC, CTTEXT_NO_HISTORIES), T_(CT_GET_TEXT_ERR, CTTEXT_NO_HISTORIES_HELP_TEXT));
			elm_box_pack_end(box, nocontent);
			evas_object_size_hint_min_set(box, 720, 650);
			evas_object_show(nocontent);
			elm_object_part_text_set(nocontent, "elm.text", buf);
			chistory_d->noc = nocontent;
			chistory_d->scroller = scroller;
		}
		evas_object_show(chistory_d->scroller);
		elm_object_part_content_set(layout, "elm.swallow.content", chistory_d->scroller);
	}

	chistory_d->navi_item = elm_naviframe_item_push(chistory_d->navi, "History", NULL, NULL, chistory_d->layout, NULL);
	elm_naviframe_item_title_visible_set(chistory_d->navi_item, EINA_TRUE);
	evas_object_event_callback_add(chistory_d->layout, EVAS_CALLBACK_DEL, detail_history_view_delete_cb, chistory_d);
	detail_history_set_navi(chistory_d, DETAIL_VIEW_MODE_HISTORY);
}

