/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include "phone.h"
#include "ct-image-viewer.h"

typedef struct {
	char *image_path;
	int image_id;
	int page_id;
}ct_image_info;

typedef struct {
	Evas_Object *navi;
	Evas_Object *box;
	Evas_Object *layout;
	Evas_Object *scroller;

	Evas_Object *btn_set;

	ct_person_changed_cb person_changed_cb;
	void *person_changed_cb_data;

	int width;
	int height;
	contacts_record_h person;

	Eina_List *list_img;
	Eina_List *list_contact;
	int list_img_count;
}ct_image_viewer_data;


static void __ct_image_rotate_cb(void *data, Evas *e, Evas_Object *obj, void *ei);

static void image_viewer_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	ct_image_info *cimage_info;
	ct_image_viewer_data *cimage_viewer_d = data;
	p_retm_if(NULL == cimage_viewer_d, "parameter(cimage_viewer_d) is NULL");

	evas_object_event_callback_del(cimage_viewer_d->layout, EVAS_CALLBACK_RESIZE, __ct_image_rotate_cb);

	EINA_LIST_FREE(cimage_viewer_d->list_img, cimage_info) {
		free(cimage_info->image_path);
		free(cimage_info);
	}
	free(cimage_viewer_d);
}

static Eina_List* image_viewer_get_list_image_info(Eina_List *list_contact, ct_image_viewer_data *cimage_viewer_d)
{
	Eina_List *l;
	Eina_List *list_img_info = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_defaut_image = NULL;
	char *img = NULL;
	ct_image_info *cimage_info = NULL;
	int default_image_id = 0;
	int person_id = 0;

	p_retvm_if(NULL == list_contact, NULL, "parameter(list_contact) is NULL");

	if(cimage_viewer_d->person) {
		err = contacts_record_get_int(cimage_viewer_d->person, _contacts_person.id, &person_id);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_record_get_int() Failed(%d)", err);
		PH_DBG("person id %d", person_id);

		err = contacts_person_get_default_property(CONTACTS_PERSON_PROPERTY_IMAGE, person_id, &default_image_id);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_person_get_default_property() Failed(%d)", err);
		PH_DBG("default_image_id %d", default_image_id);

		err = contacts_db_get_record(_contacts_image._uri, default_image_id, &record_defaut_image);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_db_get_record() Failed(%d)", err);

		err = contacts_record_get_str_p(record_defaut_image, _contacts_image.path, &img);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		PH_DBG("default_image_img %s", img);

		cimage_info = calloc(1, sizeof(ct_image_info));
		cimage_info->image_id = default_image_id;
		cimage_info->image_path = strdup(img);
		list_img_info = eina_list_append(list_img_info, cimage_info);

		err = contacts_record_destroy(record_defaut_image, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	}

	EINA_LIST_FOREACH(list_contact, l, record_contact) {
		contacts_record_h record_image = NULL;
		int image_id;
		if (NULL == record_contact)
			continue;

		err = contacts_record_get_str_p(record_contact, _contacts_contact.image_thumbnail_path, &img);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		if (NULL == img || '\0' == *img)
			continue;

		err = contacts_record_get_child_record_at_p(record_contact, _contacts_contact.image, 0, &record_image);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);

		err = contacts_record_get_int(record_image, _contacts_image.id, &image_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		if(image_id != default_image_id) {
			cimage_info = calloc(1, sizeof(ct_image_info));
			cimage_info->image_id = image_id;
			cimage_info->image_path = strdup(img);
			list_img_info = eina_list_append(list_img_info, cimage_info);
		}
	}

	return list_img_info;
}

static void image_viewer_fill_layout(Evas_Object *box, Eina_List *list_img, int width, int height)
{
	Eina_List *l;
	Evas_Object *img;
	ct_image_info *cimage_info;
	int page = 0;

	p_retm_if(NULL == list_img, "Invalid parameter(list_img is NULL)");

	EINA_LIST_FOREACH(list_img, l, cimage_info) {
		img = elm_image_add(box);
		p_retm_if(NULL == img, "elm_image_add() return NULL");

		elm_image_file_set(img, cimage_info->image_path, NULL);
		evas_object_size_hint_min_set(img, width, height);
		evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_HORIZONTAL, 1, 1);

		elm_box_pack_end(box, img);

		cimage_info->page_id = page++;

		evas_object_show(img);
	}
}

static void image_viewer_render_post_cb(void *data, Evas *e, void *even_info)
{
	int width;
	int height;

	Evas_Object *box;
	Evas_Object *scroller;
	ct_image_viewer_data* cimage_viewer_d = data;

	evas_event_callback_del(e, EVAS_CALLBACK_RENDER_FLUSH_POST, image_viewer_render_post_cb);
	evas_object_geometry_get(cimage_viewer_d->layout, NULL, NULL, &width, &height);

	cimage_viewer_d->width = width;
	cimage_viewer_d->height = height;

	scroller = elm_scroller_add(cimage_viewer_d->layout);
	p_retm_if(NULL == scroller, "elm_scroller_add() return NULL");
	cimage_viewer_d->scroller = scroller;

	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_FALSE);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);

	elm_object_part_content_set(cimage_viewer_d->layout, "elm.swallow.content", scroller);
	elm_scroller_page_size_set(scroller, width, height);
	elm_object_scroll_lock_y_set(scroller, EINA_TRUE);
	evas_object_show(scroller);

	box = elm_box_add(scroller);
	p_retm_if(NULL == box, "elm_box_add() return NULL");
	cimage_viewer_d->box = box;
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_horizontal_set(box, EINA_TRUE);
	elm_object_content_set(scroller, box);
	evas_object_show(box);

	image_viewer_fill_layout(cimage_viewer_d->box, cimage_viewer_d->list_img, cimage_viewer_d->width, cimage_viewer_d->height);
}

static void image_viewer_set_cb(void *data, Evas_Object *obj, void *event_info)
{
	int h_page = 0;
	int v_page = 0;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	int person_id = 0;
	Eina_List *l;
	ct_image_info *cimage_info;
	ct_image_viewer_data *cimage_viewer_d = data;
	p_retm_if(NULL == cimage_viewer_d, "parameter(cimage_viewer_d) is NULL");

	elm_scroller_current_page_get(cimage_viewer_d->scroller, &h_page, &v_page);

	EINA_LIST_FOREACH(cimage_viewer_d->list_img, l, cimage_info) {
		if(cimage_info) {
			if(cimage_info->page_id == h_page)
				break;
		}
		else
			ERR("cimage_info is null");
	}

	if(cimage_info) {
		err = contacts_record_get_int(cimage_viewer_d->person, _contacts_person.id, &person_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_person_set_default_property(CONTACTS_PERSON_PROPERTY_IMAGE, person_id, cimage_info->image_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_person_set_default_property() Failed(%d)", err);
	}
	else
		ERR("cimage_info is null");

	elm_naviframe_item_pop(cimage_viewer_d->navi);
}

static void __ct_image_refresh(ct_image_viewer_data *cimage_viewer_d)
{
	Evas_Object *box;
	Evas_Object *scroller;
	int h_page = 0;
	int v_page = 0;

	elm_scroller_current_page_get(cimage_viewer_d->scroller, &h_page, &v_page);

	evas_object_del(cimage_viewer_d->scroller);

	evas_object_geometry_get(cimage_viewer_d->layout, NULL, NULL, &cimage_viewer_d->width, &cimage_viewer_d->height);

	scroller = elm_scroller_add(cimage_viewer_d->layout);
	p_retm_if(NULL == scroller, "elm_scroller_add() return NULL");
	cimage_viewer_d->scroller = scroller;

	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_FALSE);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);

	elm_object_part_content_set(cimage_viewer_d->layout, "elm.swallow.content", scroller);
	elm_scroller_page_size_set(scroller, cimage_viewer_d->width, cimage_viewer_d->height);
	elm_object_scroll_lock_y_set(scroller, EINA_TRUE);
	evas_object_show(scroller);

	box = elm_box_add(scroller);
	p_retm_if(NULL == box, "elm_box_add() return NULL");
	cimage_viewer_d->box = box;
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_horizontal_set(box, EINA_TRUE);
	elm_object_content_set(scroller, box);
	evas_object_show(box);

	image_viewer_fill_layout(cimage_viewer_d->box, cimage_viewer_d->list_img, cimage_viewer_d->width, cimage_viewer_d->height);

	elm_scroller_page_bring_in(cimage_viewer_d->scroller, h_page, v_page);
}


static void __ct_image_rotate_cb(void *data, Evas *e, Evas_Object *obj, void *ei)
{
	__ct_image_refresh(data);
}

void ct_image_viewer(Evas_Object *navi, contacts_record_h person, Eina_List *list_contact,
		ct_person_changed_cb person_changed_cb, void *person_changed_cb_data)
{
	char navi_title[PH_TEXT_MAX_LEN+1];

	Evas_Object *bg;
	Evas_Object *btn;
	Evas_Object *layout;
	Elm_Object_Item *navi_item;

	Eina_List *list_img_info;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	ct_image_info *cimage_info;
	ct_image_viewer_data *cimage_viewer_d;

	cimage_viewer_d = calloc(1, sizeof(ct_image_viewer_data));
	p_retm_if(NULL == cimage_viewer_d, "calloc() return NULL");

	cimage_viewer_d->navi = navi;
	cimage_viewer_d->person = person;
	cimage_viewer_d->person_changed_cb = person_changed_cb;
	cimage_viewer_d->person_changed_cb_data = person_changed_cb_data;

	layout = phone_create_base_layout(navi, EINA_FALSE);
	if (NULL == layout) {
		ERR("phone_create_base_layout() return NULL");
		free(cimage_viewer_d);
		return;
	}
	evas_object_show(layout);
	cimage_viewer_d->layout = layout;

	bg = phone_create_bg(layout);
	elm_bg_color_set(bg, 0, 0, 0);
	evas_event_callback_add(evas_object_evas_get(layout), EVAS_CALLBACK_RENDER_FLUSH_POST, image_viewer_render_post_cb, cimage_viewer_d);
	evas_object_event_callback_add(layout, EVAS_CALLBACK_DEL, image_viewer_delete_cb, cimage_viewer_d);
	evas_object_event_callback_add(layout, EVAS_CALLBACK_RESIZE, __ct_image_rotate_cb, cimage_viewer_d);

	cimage_viewer_d->list_img = image_viewer_get_list_image_info(list_contact, cimage_viewer_d);

	cimage_viewer_d->list_img_count = eina_list_count(cimage_viewer_d->list_img);
	if (1 == cimage_viewer_d->list_img_count) {
		char *display_name;
		if (person) {
			err = contacts_record_get_str_p(person, _contacts_person.display_name, &display_name);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			if (NULL != display_name)
				snprintf(navi_title, sizeof(navi_title), "%s", display_name);
			else
				snprintf(navi_title, sizeof(navi_title), "%s", T_(CT_GET_TEXT_ERR, CTTEXT_NONAME));
		}
		else {
			record_contact = (contacts_record_h)list_contact->data;
			err = contacts_record_get_str_p(record_contact, _contacts_contact.display_name, &display_name);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			if (NULL != display_name)
				snprintf(navi_title, sizeof(navi_title), "%s", display_name);
			else
				snprintf(navi_title, sizeof(navi_title), "%s", T_(CT_GET_TEXT_ERR, CTTEXT_NONAME));
		}
	}
	else
		snprintf(navi_title, sizeof(navi_title), "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_SET_DEFAULT));

	btn = elm_button_add(navi);
	elm_object_style_set(btn, "naviframe/back_btn/default");
	evas_object_smart_callback_add(btn, "clicked", phone_view_back_btn_cb, navi);
	navi_item = elm_naviframe_item_push(navi, navi_title, btn, NULL, layout, NULL);

	if(cimage_viewer_d->list_img_count > 1) {
		cimage_viewer_d->btn_set = phone_create_toolbar_btn(navi, S_(CT_SYS_SK_SET), image_viewer_set_cb, cimage_viewer_d);
		elm_object_item_part_content_set(navi_item, "toolbar_button1", cimage_viewer_d->btn_set);
	}
}
