/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#define _GNU_SOURCE
#include <string.h>
#include <vconf.h>
#include <vconf-keys.h>
#include <bundle.h>
#include <aul.h>
#include <tapi_common.h>
#include <ITapiSim.h>
#include <TapiUtility.h>
#include <feedback.h>
#include <ui-gadget-module.h>

#include "phone.h"
#include "phone-dialer.h"
#include "phone-speed.h"

#include "ph-dialer-utils.h"
#include "ph-dialer-view.h"
#include "ph-dialer-search.h"

#define PH_DIALER_KEY_MAX_LEN 256

ph_speeddial_info* ph_dialer_util_get_speeddial_by_number(void *data, int number)
{
	ph_speeddial_info * find_sd = NULL;
	ph_dialer_data *dial_d = data;
	Eina_List *l;

	p_retvm_if(NULL == dial_d->speeddial_list, NULL, "There is no speeddial list");

	EINA_LIST_FOREACH(dial_d->speeddial_list, l, find_sd) {
		if (!find_sd) continue;
		if (number == find_sd->dial_number && find_sd->ph_number)
		{
			if (number == 1)
			{
				free(find_sd->ph_number);
				find_sd->ph_number = phone_get_voice_mail_number();
			}
			break;
		}
	}
	return find_sd;
}

void ph_dialer_util_entry_clear(ph_dialer_data *dial_d)
{
	ph_dialer_util_del_all_number(dial_d);
	ph_dialer_entry_set_number(dial_d, "", 0);
	if (dial_d->has_focus) {
		elm_object_focus_set(dial_d->entry, EINA_FALSE);
		dial_d->has_focus = false;
	}
	ph_dialer_search_request(dial_d, FALSE);
}

void ph_dialer_util_launch_call(ph_dialer_data *dial_d)
{
	char num[PH_TEXT_MAX_LEN] = {0};
	int text_len;
	bundle* kb;

	p_retm_if(dial_d == NULL, "Invalid argument: ph_dialer_data *is NULL");

	text_len = ph_dialer_util_get_pure_numstr_with_plus(dial_d, num);
	if (0 < text_len) {
		if (PH_DAILER_EMERGENCY == dial_d->run_mode) {
			kb = bundle_create();
			p_retm_if(NULL == kb, "bundle_create() Failed");
			bundle_add(kb, "launch-type", "EMERGENCY");
			bundle_add(kb, "number", num);
			aul_launch_app(VOICECALL_PKG, kb);
			bundle_free(kb);
		}
		else
			phone_launch_voice_call(-1, num);
	}

	ph_dialer_util_entry_clear(dial_d);
}

int ph_dialer_util_get_cursor_pos(ph_dialer_data *dial_d)
{
	InputNumber *node;
	int pos = 0;
	p_retvm_if(NULL == dial_d->number_cursor, 0, "cursor of number is null");

	node = &dial_d->number_head;
	while (dial_d->number_cursor != node) {
		node = node->next;
		if (!node) break;
		pos++;
	}
	return pos;
}

int ph_dialer_util_is_dash(char ch)
{
	if (ch == '-' || ch == '(' || ch == ')' || ch == ' ')
		return TRUE;
	return FALSE;
}

static int dialer_util_is_dash_and_plus(char ch)
{
	if (ch == '-' || ch == '+' || ch == '(' || ch == ')' || ch == ' ')
		return TRUE;
	return FALSE;
}

int ph_dialer_util_get_pure_numstr(ph_dialer_data *dial_d, char *dest)
{
	InputNumber *node;
	int cnt = 0;
	p_retvm_if(NULL == dest, 0, "dest is null");

	node = dial_d->number_head.next;
	while (1) {
		if (!node) break;
		if (dialer_util_is_dash_and_plus(node->num) == FALSE)
			dest[cnt++] = node->num;
		node = node->next;
	}
	dest[cnt] = '\0';
	return cnt;
}

InputNumber *ph_dialer_util_get_node_by_index(ph_dialer_data *dial_d, int index)
{
	InputNumber *node;
	int i = 0;

	node = &dial_d->number_head;
	for (; i<index; i++) {
		if (node->next)
			node = node->next;
	}
	return node;
}

int ph_dialer_util_get_pure_numstr_with_plus(ph_dialer_data *dial_d, char *dest)
{
	InputNumber *node;
	int cnt = 0;
	p_retvm_if(NULL == dest, 0, "dest is null");

	node = dial_d->number_head.next;
	while (node) {
		if (ph_dialer_util_is_dash(node->num) == FALSE)
			dest[cnt++] = node->num;
		node = node->next;
	}
	dest[cnt] = '\0';
	return cnt;
}

int ph_dialer_util_get_numstr_from_node(ph_dialer_data *dial_d, char *dest)
{
	InputNumber *node;
	int cnt = 0;
	p_retvm_if(NULL == dest, 0, "dest is null");

	node = dial_d->number_head.next;
	while (node) {
		dest[cnt++] = node->num;
		node = node->next;
	}
	dest[cnt] = '\0';
	return cnt;
}

int ph_dialer_util_del_all_number(ph_dialer_data *dial_d)
{
	int len = 0;
	InputNumber *node = NULL;
	InputNumber *tmp_node = NULL;
	p_retvm_if(NULL == dial_d, 0, "dialer data is null");

	node = dial_d->number_head.next;
	while (node) {
		tmp_node = node;

		if (node->next)
			node->next->prev = node->prev;
		if (node->prev)
			node->prev->next = node->next;

		node = node->next;
		len++;

		free(tmp_node);
	}
	dial_d->number_cursor = &dial_d->number_head;
	dial_d->entry_font_size = MAX_DIAL_NUMBER_FONT_SIZE;

	return len;
}

static int dialer_util_is_num_length_over(int len)
{
	if (len >= PH_DIALER_KEY_MAX_LEN)
		return TRUE;
	return FALSE;
}

int ph_dialer_util_append_node_from_numstr(ph_dialer_data *dial_d, char *string)
{
	int i = 0;
	int j = 0;
	int len = 0;
	InputNumber *node;
	InputNumber *tmp_node = NULL;
	char* tmp_string = NULL;

	ph_dialer_util_del_all_number(dial_d);

	p_retvm_if(NULL == string, 0, "string data is null");

	node = &dial_d->number_head;

	len = strlen(string);
	if (dialer_util_is_num_length_over(len-1) == TRUE) {
		phone_show_popup(dial_d->navi, T_(PH_GET_TEXT_BASIC, PHTEXT_EXCEED_NUM_MAX_LENGTH), 2.0);
		len = PH_DIALER_KEY_MAX_LEN;
		string[len] = '\0';
	}

	tmp_string = calloc(1, len+1);
	memset(tmp_string, 0x00, len+1);

	for(i = 0; i<len; i++) {
		if((string[i] >= '0' && string[i] <= '9')
			||string[i]=='*'
			||string[i]=='#'
			||string[i]=='+')
		{
			tmp_string[j++] = string[i];
		}
	}

	len = strlen(tmp_string);
	for (i = 0; i<len; i++) {
		tmp_node = calloc(1, sizeof(InputNumber));
		tmp_node->next = NULL;
		tmp_node->prev = node;
		tmp_node->num = tmp_string[i];

		node->next = tmp_node;
		node = node->next;
	}
	dial_d->number_cursor = tmp_node;
	free(tmp_string);

	return len;
}

int ph_dialer_util_append_node_from_numstr_with_strip(ph_dialer_data *dial_d, char *string)
{
	int i = 0, cnt = 0, len = 0;
	InputNumber *node;
	InputNumber *tmp_node = NULL;
	p_retvm_if(NULL == string, 0, "Number string is null");
	ph_dialer_util_del_all_number(dial_d);

	node = &dial_d->number_head;
	len = strlen(string);
	if (dialer_util_is_num_length_over(len-1) == TRUE) {
		phone_show_popup(dial_d->navi, T_(PH_GET_TEXT_BASIC, PHTEXT_EXCEED_NUM_MAX_LENGTH), 2.0);
		len = PH_DIALER_KEY_MAX_LEN;
		string[len] = '\0';
	}

	for (; i<len; i++) {
		if (ph_dialer_util_is_dash(string[i]) == TRUE)
			continue;

		tmp_node = calloc(1, sizeof(InputNumber));
		tmp_node->next = NULL;
		tmp_node->prev = node;
		tmp_node->num = string[i];
		node->next = tmp_node;

		node = node->next;
		cnt++;
	}
	dial_d->number_cursor = tmp_node;
	return cnt;
}

char ph_dialer_util_get_value_word(char in)
{
	switch (in) {
	case '2':
		return 'a';
	case '3':
		return 'd';
	case '4':
		return 'g';
	case '5':
		return 'j';
	case '6':
		return 'm';
	case '7':
		return 'p';
	case '8':
		return 't';
	case '9':
		return 'w';
	default:
		return ' ';
	}
}

char ph_dialer_util_get_next_value_abcmode(char in)
{
	switch (in) {
	case 'a':
		return 'b';
	case 'b':
		return 'c';
	case 'c':
		return 'a';
	case 'd':
		return 'e';
	case 'e':
		return 'f';
	case 'f':
		return 'd';
	case 'g':
		return 'h';
	case 'h':
		return 'i';
	case 'i':
		return 'g';
	case 'j':
		return 'k';
	case 'k':
		return 'l';
	case 'l':
		return 'j';
	case 'm':
		return 'n';
	case 'n':
		return 'o';
	case 'o':
		return 'm';
	case 'p':
		return 'q';
	case 'q':
		return 'r';
	case 'r':
		return 's';
	case 's':
		return 'p';
	case 't':
		return 'u';
	case 'u':
		return 'v';
	case 'v':
		return 't';
	case 'w':
		return 'x';
	case 'x':
		return 'y';
	case 'y':
		return 'z';
	case 'z':
		return 'w';
	default:
		return ' ';
	}
}

static InputNumber* dialer_util_insert_number_after(InputNumber *node, char num)
{
	InputNumber *tmp;
	p_retvm_if(NULL == node, NULL, "number node is null");

	tmp = calloc(1, sizeof(InputNumber));
	tmp->next = node->next;
	tmp->prev = node;
	tmp->num = num;

	if (node->next)
		node->next->prev = tmp;
	node->next = tmp;

	return tmp;
}

void ph_dialer_util_insert_number_at_cursor(ph_dialer_data *dial_d, char num)
{
	InputNumber *cursor;
	InputNumber *tmp;
	char buf[PH_TEXT_MAX_LEN] = {0};
	int len;

	if (dial_d->number_cursor)
		cursor = dial_d->number_cursor;
	else
		cursor = &dial_d->number_head;

	/* check length */
	len = ph_dialer_util_get_numstr_from_node(dial_d, buf);
	if (dialer_util_is_num_length_over(len) == TRUE) {
		phone_show_popup(dial_d->navi, T_(PH_GET_TEXT_BASIC, PHTEXT_EXCEED_NUM_MAX_LENGTH), 2.0);
		return;
	}

	tmp = dialer_util_insert_number_after(cursor, num);
	dial_d->number_cursor = tmp;
}

char ph_dialer_util_get_number_at_cursor(ph_dialer_data *dial_d)
{
	if (dial_d->number_cursor == NULL)
		return '\0';

	return dial_d->number_cursor->num;
}

int ph_dialer_util_remove_node_at_cursor(ph_dialer_data *dial_d)
{
	InputNumber *cursor = NULL;
	InputNumber *node_prev = NULL;
	p_retvm_if(NULL == dial_d->number_cursor, 0, "current cursor is null");
	cursor = dial_d->number_cursor;

	if (cursor == &dial_d->number_head)
		return 0;

	if (cursor)
		node_prev = cursor->prev;

	if (node_prev)
		node_prev->next = cursor->next;

	if (cursor->next)
		cursor->next->prev = node_prev;

	free(cursor);
	dial_d->number_cursor = node_prev;
	return ph_dialer_util_get_cursor_pos(dial_d);
}

void ph_dialer_util_strip_from_number_node(ph_dialer_data *dial_d)
{
	InputNumber *node, *tmp_node;
	node = dial_d->number_head.next;

	while (node) {
		if (ph_dialer_util_is_dash(node->num) == TRUE) {
			if (dial_d->number_cursor == node)
				dial_d->number_cursor = node->prev;
			tmp_node = node;

			if (node->next)
				node->next->prev = node->prev;
			if (node->prev)
				node->prev->next = node->next;

			node = node->next;
			free(tmp_node);
		}
		else
			node = node->next;
	}
}

void ph_dialer_util_add_dashes(ph_dialer_data *dial_d, DashMode kMode)
{
	InputNumber *cursor, *node;
	int len = 0;
	int remain = 0;
	char numstr[PH_TEXT_MAX_LEN] = {0};

	cursor = dial_d->number_cursor;
	node = dial_d->number_head.next;

	len = ph_dialer_util_get_numstr_from_node(dial_d, numstr);
	p_retm_if(len < 2, "number is too short");
	p_retm_if(numstr[0] == '*' || numstr[0] == '#', "the character is reserved string");

	remain = len;
	/* 123 ~ 12345678901(123-4567-8901) */
	if (len > (2-kMode) && len < (12+kMode) && node) {      /* 02, 010, 011, ... 0xx */
		if (node->num == '0') {         /* 02, +cc 2 */
			remain--;
			node = node->next;
			remain--;

			if (node->num == '2') {          /* 02-, +cc 2- */
				node = dialer_util_insert_number_after(node, '-');
				if (node) {
					if (kMode) {
						if (dial_d->number_cursor == node->prev) {
							dial_d->number_cursor = node;
						}
					}
					if (node->next)
						node = node->next;
				}
			}
			else {            /* 0xx, +cc xx */
				if (node->next) {
					node = node->next;
					remain--;
				}

				if (len > 3-kMode) {            /* 0xx-, +cc xx- */
					node = dialer_util_insert_number_after(node, '-');
					if (node) {
						if (kMode) {
							if (dial_d->number_cursor == node->prev) {
								dial_d->number_cursor = node;
							}
						}
						if (node->next)
							node = node->next;
					}
				}
			}

			if (remain == 8) {            /* xxx-1234-1234, +cc xx-1234-1234 */
				node = node->next;
				p_ret_if(node == NULL);
				node = node->next;
				p_ret_if(node == NULL);
				node = node->next;
				dialer_util_insert_number_after(node, '-');
			}
			else if (remain > (int)(3-kMode) && remain < 8) {         /* xxx-123-1234, +cc xx-123-1234 */
				node = node->next;
				p_ret_if(node == NULL);
				node = node->next;
				node = dialer_util_insert_number_after(node, '-');
				if (node && kMode) {
					if (dial_d->number_cursor == node->prev) {
						dial_d->number_cursor = node;
					}
				}
			}
			else if (remain > 8) {            /* not match rule */
				ph_dialer_util_strip_from_number_node(dial_d);
			}
		}

		else {                 /* 02, +cc 2 */   /* 1234-1234 */
			if (len > 4-kMode && len < 9+kMode) {
				node = node->next;
				p_ret_if(node == NULL);
				node = node->next;
				p_ret_if(node == NULL);
				node = node->next;
				node = dialer_util_insert_number_after(node, '-');
				if (node && kMode) {
					if (dial_d->number_cursor == node->prev) {
						dial_d->number_cursor = node;
					}
				}
			}
		}
	}
}

void ph_dialer_util_highlight_substring(char *dest, int dest_len, char *string,
		char *substr)
{
	char middle[PH_TAG_TEXT_MAX_LEN] = {0};
	char first[PH_TAG_TEXT_MAX_LEN] = {0};

	p_retm_if(NULL == string, "string is null");

	if (substr && *substr) {
		int len = strlen(substr);
		snprintf(first, sizeof(first), "%s", string);
		char *last = strcasestr(first, substr);
		if (last) {
			snprintf(middle, strlen(substr)+1, "%s", last);
			*(last) = '\0';
			snprintf(dest, dest_len, "%s<match>%s</>%s",first, middle, last+len);
		}
		else
			snprintf(dest, dest_len, "%s", string);
	}
	else
		snprintf(dest, dest_len, "%s", string);
}

void ph_dialer_util_highlight_substring_with_pos(char *dest, int dest_len, char *string,
		int position, int length)
{
	char middle[PH_TEXT_MAX_LEN] = {0};
	char first[PH_TEXT_MAX_LEN] = {0};

	p_retm_if(NULL == string, "string is null");

	if (0 < length) {
		snprintf(first, sizeof(first), "%s", string);
		snprintf(middle, length+1, "%s", string + position);
		first[position] = '\0';
		snprintf(dest, dest_len, "%s<match>%s</>%s",first, middle, string + position + length);
	}
	else
		snprintf(dest, dest_len, "%s", string);
}

int ph_dialer_util_show_matched_one_number(ph_dialer_data *dial_d, char *name, char *number)
{
	PH_FN_CALL;
	p_retvm_if(dial_d==NULL, 0, "dialer data is null");

	if (dial_d->slide) {
		edje_object_signal_emit(_EDJ(dial_d->layout), "elm,state,slide,stop", "elm");
		dial_d->slide = false;
	}

	edje_object_signal_emit(_EDJ(dial_d->layout), "viewcontrol/search_result", "viewcontrol");
	edje_object_part_text_set(_EDJ(dial_d->layout), "search_result/item/number/textblock", number);

	if (name && *name) {
		edje_object_part_text_set(_EDJ(dial_d->layout), "search_result/item/name/text", name);
		edje_object_signal_emit(_EDJ(dial_d->layout), "viewobject/search_result/search_number", "viewobject/search_result");
	}
	else
		edje_object_signal_emit(_EDJ(dial_d->layout), "viewobject/search_result/noname", "viewobject/search_result");


	ph_dialer_update_cnt_btn(dial_d, 1);
	dial_d->is_show_items = true;
	return 0;
}


static void __ph_dialer_speed_dial_view_ug_destroy_cb(ui_gadget_h ug, void *priv)
{
	if (ug)
		ug_destroy(ug);
}

static void __ph_dialer_speed_dial_view_ug_result_cb(ui_gadget_h ug, service_h service, void *priv)
{
	ug_destroy(ug);
}

static void __ph_dialer_speed_dial_view_ug_layout_cb(ui_gadget_h ug, enum ug_mode mode, void *priv)
{
 	Evas_Object *base = ug_get_layout(ug);
	p_retm_if(base == NULL, "base == NULL");

	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(base);
}


void ph_dialer_util_speeddial_show_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_dialer_data *dial_d = data;

	//phone_create_speeddial_view(dial_d->ug, dial_d->win, dial_d->navi);

	struct ug_cbs cbs = { 0, };
	ui_gadget_h ug;

	cbs.priv = (void *)dial_d;
	cbs.layout_cb = __ph_dialer_speed_dial_view_ug_layout_cb;
	cbs.result_cb = __ph_dialer_speed_dial_view_ug_result_cb;
	cbs.destroy_cb = __ph_dialer_speed_dial_view_ug_destroy_cb;

	dial_d->child_ug = ug_create((ui_gadget_h)dial_d->ug, "speeddial-efl", UG_MODE_FULLVIEW, NULL, &cbs);

	evas_object_del(dial_d->popup);
	dial_d->popup = NULL;
}

void ph_dialer_util_show_speeddial_msg(ph_dialer_data *dial_d, const char *msg)
{
	char dest[PH_TEXT_MAX_LEN] = {0};
	snprintf(dest, sizeof(dest), "%s", msg);
	phone_create_confirm_popup(&dial_d->popup, dial_d->win, msg, ph_dialer_util_speeddial_show_cb, dial_d);
}

void ph_dialer_util_setup_run_mode(ph_dialer_data *dial_d)
{
	int ret = 0;
	int value = 0;

	/* Check call state */
	ret = vconf_get_int(VCONFKEY_CALL_STATE, &value);
	if (ret < 0)
		ERR("vconf_get_int is failed(%d)", ret);
	else {
		if (value == VCONFKEY_CALL_OFF
				|| value == VCONFKEY_CALL_VOICE_CONNECTING
				|| value == VCONFKEY_CALL_VIDEO_CONNECTING) {
			if (dial_d->run_mode == PH_DAILER_EMERGENCY)
			{
				edje_object_signal_emit(_EDJ(dial_d->layout), "softkey/emergency", "softkey");
			}
			else {
				if (dial_d->run_mode == PH_DAILER_NORMAL)
					return;
				dial_d->run_mode = PH_DAILER_NORMAL;
				edje_object_signal_emit(_EDJ(dial_d->layout), "softkey/normal", "softkey");
			}
		}
		else {   // VCONFKEY_CALL_VOICE_ACTIVE, VCONFKEY_CALL_VIDEO_ACTIVE
			if (dial_d->run_mode == PH_DAILER_DURING_CALL)
				return;
			dial_d->run_mode = PH_DAILER_DURING_CALL;
			edje_object_signal_emit(_EDJ(dial_d->layout), "softkey/dc", "softkey");
		}
		PH_DBG("run_mode:%d", dial_d->run_mode);
	}
}

void ph_dialer_util_init_sim_value(ph_dialer_data *dial_d)
{
	/* Network MCC */
	int value = 0;
	int ret;
	TelSimImsiInfo_t sim_imsi_info;
	TapiHandle *handle;
	ret = vconf_get_int(VCONFKEY_TELEPHONY_PLMN, &value);
	if (ret < 0)
		dial_d->cur_mcc = -1;
	else
		dial_d->cur_mcc = value / 1000;

	/* SIM MCC/MNC */
	dial_d->cur_sim_mcc = -1;
	handle = tel_init(NULL);
	if (NULL == handle) {
		ERR("tel_init() Failed");
	}
	else {
		ret = tel_get_sim_imsi(handle, &sim_imsi_info);
		if (0 == ret)
			dial_d->cur_sim_mcc = atoi(sim_imsi_info.szMcc);
		else
			ERR("tel_get_sim_imsi() Failed(%d)", ret);
		tel_deinit(handle);
	}
	dial_d->cur_cc_index = -1;

	ret = vconf_get_int(VCONFKEY_TELEPHONY_SVC_ROAM, &value);
	if (ret < 0)
		dial_d->roaming = 0;
	else
		dial_d->roaming = value;
}

void ph_dialer_util_svi_play(int vibration_key, int sound_key)
{
	int effect_status = FALSE;
	vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &effect_status);
	if (effect_status)
		feedback_play_type(FEEDBACK_TYPE_SOUND, sound_key);

	vconf_get_bool(VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL, &effect_status);
	if (effect_status)
		feedback_play_type(FEEDBACK_TYPE_VIBRATION, vibration_key);
}
