/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <ui-gadget-module.h>

#include "phone.h"
#include "ct-favor.h"
#include "ph-favorite-view.h"
#include "ph-favorite-edit-view.h"
#include "ct-list.h"

static void favor_edit_del(ph_favorite_data *favor_d);

static void favor_edit_del_cb(int err, void *data)
{
	ph_favorite_data *favor_d = data;
	ph_progress_info *p_info = NULL;

	p_retm_if(NULL == favor_d, "Invalid parameter(favor_d is NULL)");
	p_info = favor_d->p_info;
	p_retm_if(NULL == p_info, "Invalid parameter(p_info is NULL)");

	if (p_info->update_list) {
		contacts_list_destroy(p_info->update_list, true);
		p_info->update_list = NULL;
	}

	if (CONTACTS_ERROR_NONE != err || p_info->alive == false) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
		free(p_info);
		favor_d->p_info = NULL;
		favor_d->updating = false;
		ph_favor_data_lazy_update(_contacts_person._uri, favor_d);
		return;
	}
	else if (p_info->cnt == p_info->cnt_checked_total) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
		free(p_info);
		favor_d->p_info = NULL;
		favor_d->updating = false;
		ph_favor_edit_mode_end(favor_d, true);
		ph_favor_data_lazy_update(_contacts_person._uri, favor_d);
		return;
	}
	else {
		favor_edit_del(favor_d);
	}
}

static void favor_edit_del(ph_favorite_data *favor_d)
{
	const int FAVOR_BULK_LENGTH = 50;
	char percent_total[PH_TEXT_MAX_LEN+1] = {0, };
	char percent[PH_TEXT_MAX_LEN+1] = {0, };
	int count = 0;
	int base_count = 0;
	double value = 0.0;

	Eina_List *l = NULL;
	ph_favor_info *f_info = NULL;
	ph_progress_info *p_info = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	contacts_list_h list = NULL;

	p_retm_if(NULL == favor_d, "parameter(favor_d) is NULL");
	p_info = favor_d->p_info;
	p_retm_if(NULL == p_info, "parameter(p_info) is NULL");

	err = contacts_list_create(&list);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_list_create() Failed(%d)", err);

	EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
		if (NULL == f_info) continue;

		if (TRUE == f_info->checked) {
			if (base_count < p_info->cnt) {
				base_count++;
				continue;
			}

			if (CONTACTS_ERROR_NONE != contacts_db_get_record(_contacts_person._uri, f_info->person_id, &record)) {
				ERR("contacts_db_get_record() Failed()");
				p_info->completed = true;
				p_info->result = FALSE;
				break;
			}
			err = contacts_record_set_bool(record, _contacts_person.is_favorite, false);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_ercord_set_bool() Failed(%d)", err);

			contacts_list_add(list, record);
			count++;
			if (FAVOR_BULK_LENGTH == count || (count + p_info->cnt) == p_info->cnt_checked_total) {
				p_info->cnt += count;
				break;
			}
		}
	}
	value = (double)p_info->cnt / (double)p_info->cnt_checked_total;
	elm_progressbar_value_set(p_info->progressbar, value);
	snprintf(percent, sizeof(percent), "%d%%", (int)(100.0 * (double)p_info->cnt/(double)p_info->cnt_checked_total));
	snprintf(percent_total, sizeof(percent_total), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", percent_total);

	p_info->update_list = list;
	contacts_db_update_records_async(list, favor_edit_del_cb, favor_d);
}

void ph_favor_edit_del_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	ph_progress_info *p_info;

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc return NULL");

	favor_d->p_info = p_info;
	favor_d->updating = true;
	p_info->cnt_checked_total = favor_d->checked_cnt;
	p_info->alive = true;
	p_info->popup = phone_progressbar_popup(favor_d->win, p_info, T_(PH_GET_TEXT_BASIC, PHTEXT_DELETING_FAVORITES));
	favor_edit_del(favor_d);
}

void ph_favor_edit_check_for_number(Elm_Object_Item *item, ph_favorite_data *favor_d)
{
	ph_favor_info *f_info;
	Evas_Object *list_nums;
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	f_info = elm_object_item_data_get(item);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

	if (NULL == f_info->number_list) {
		ERR("f_info->number_list is NULL");
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_NUMBER), 1.5);

		f_info->checked = EINA_FALSE;
		favor_d->checked_cnt--;

		elm_gengrid_item_update(item);
		return;
	}

	if (1 == eina_list_count(f_info->number_list)) {
		record = (contacts_record_h)f_info->number_list->data;
		f_info->id_ret_type = 1;
		err = contacts_record_get_int(record, _contacts_number.id, &f_info->id_ret);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		ph_favor_edit_set_selection_info(favor_d);
	}
	else {
		list_nums = ph_favor_create_value_list_popup(favor_d, f_info);
		p_retm_if(NULL == list_nums, "ph_favor_create_value_list_popup() return NULL");
		evas_object_data_set(list_nums, "item", item);
	}
}

void ph_favor_edit_check_for_email(Elm_Object_Item *item, ph_favorite_data *favor_d)
{
	Evas_Object *list_emails;
	ph_favor_info *f_info;
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	f_info = elm_object_item_data_get(item);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

	if (NULL == f_info->email_list) {
		ERR("f_info->number_list is NULL");
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);

		f_info->checked = EINA_FALSE;
		favor_d->checked_cnt--;

		elm_gengrid_item_update(item);
		return;
	}

	if (1 == eina_list_count(f_info->email_list)) {
		record = (contacts_record_h)f_info->email_list->data;
		f_info->id_ret_type = 2;
		err = contacts_record_get_int(record, _contacts_email.id, &f_info->id_ret);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		ph_favor_edit_set_selection_info(favor_d);
	}
	else {
		list_emails = ph_favor_create_value_list_popup(favor_d, f_info);
		p_retm_if(NULL == list_emails, "ph_favor_create_value_list_popup() return NULL");
		evas_object_data_set(list_emails, "item", item);
	}
}

void ph_favor_edit_check_for_number_or_email(Elm_Object_Item *item, ph_favorite_data *favor_d)
{
	int number_list_count = 0;
	int email_list_count = 0;
	Eina_List *list_contact;
	Evas_Object *list;
	ph_favor_info *f_info;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	f_info = elm_object_item_data_get(item);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

	list_contact = ctui_person_get_contact_list(f_info->person_id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if (NULL == f_info->email_list)
		f_info->email_list = ctui_person_get_email_list(list_contact);
	if (NULL == f_info->number_list)
		f_info->number_list = ctui_person_get_number_list(list_contact);

	if (f_info->number_list)
		number_list_count = eina_list_count(f_info->number_list);
	if (f_info->email_list)
		email_list_count = eina_list_count(f_info->email_list);

	if (0 == number_list_count && 0 == email_list_count) {
		ctui_free_record_list(list_contact);
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);
		f_info->checked = EINA_FALSE;
		favor_d->checked_cnt--;
		elm_genlist_item_update(item);
		return;
	}

	if (1 == number_list_count && 0 == email_list_count) {
		record = (contacts_record_h)f_info->number_list->data;
		err = contacts_record_get_int(record, _contacts_number.id, &(f_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		f_info->id_ret_type = 1;
		ph_favor_edit_set_selection_info(favor_d);
	}
	else if (0 == number_list_count && 1 == email_list_count) {
		record = (contacts_record_h)f_info->email_list->data;
		err = contacts_record_get_int(record, _contacts_email.id, &(f_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		f_info->id_ret_type = 2;
		ph_favor_edit_set_selection_info(favor_d);
	}
	else {
		list = ph_favor_create_value_list_popup(favor_d, f_info);
		if (list)
			evas_object_data_set(list, "item", item);
	}
	ctui_free_record_list(list_contact);
}


void ph_favor_edit_check_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = evas_object_data_get(obj, "favor_d");
	Elm_Object_Item *item;
	Eina_Bool checked = elm_check_state_get(obj);

	ph_favor_info *f_info = data;
	p_retm_if(NULL == f_info, "favor info is NULL");
	item = f_info->item;

	if (favor_d->longpressed) {
		checked = !checked;
		elm_check_state_set(obj, checked);
		favor_d->longpressed = EINA_FALSE;
		return;
	}

	if (checked) {
		if (favor_d->ug_cnt_max) {
			if (favor_d->ug_cnt_max  <= favor_d->checked_cnt) {
				phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_EXCEED_LIMIT), 2.0);
				checked = !checked;
				elm_check_state_set(obj, checked);
				return;
			}
		}
		favor_d->checked_cnt++;
	}
	else {
		favor_d->checked_cnt--;
		ph_favor_edit_set_selection_info(favor_d);
		return;
	}

	if (PH_UG_REQUEST_CHECK_FOR_NUMBER == favor_d->ug_request)
		ph_favor_edit_check_for_number(item, favor_d);
	else if (PH_UG_REQUEST_CHECK_FOR_EMAIL== favor_d->ug_request)
		ph_favor_edit_check_for_email(item, favor_d);
	else if (PH_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == favor_d->ug_request)
		ph_favor_edit_check_for_number_or_email(item, favor_d);
	else
		ph_favor_edit_set_selection_info(favor_d);
}

void ph_favor_edit_set_selection_info(ph_favorite_data * favor_d)
{
	p_retm_if(NULL == favor_d, "Selection info is null");

	if (0 == favor_d->checked_cnt)
		phone_show_notify(favor_d->layout, T_(CT_GET_TEXT_BASIC, CTTEXT_SELECT_CONTACTS));
	else {
		char info_text[PH_TEXT_SHORT_LEN]={0};

		if (favor_d->ug_cnt_max)
			snprintf(info_text, sizeof(info_text), "%s (%d/%d)", S_(PH_SYS_POP_SELECTED),
					favor_d->checked_cnt, favor_d->ug_cnt_max);
		else
			snprintf(info_text, sizeof(info_text), "%s (%d)", S_(PH_SYS_POP_SELECTED),
					favor_d->checked_cnt);
		phone_show_notify(favor_d->layout, info_text);
	}

	ph_favor_update_button_status(favor_d);
}

static void favor_gengrid_realized(void *data, Evas_Object *obj, void *event_info)
{
	ph_favor_info *f_info;
	Elm_Object_Item *it = event_info;
	ph_favorite_data *favor_d = data;
	f_info = elm_object_item_data_get(it);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

	if (favor_d->list_mode == PH_FAVOR_MODE_EDIT)
		elm_object_item_signal_emit(f_info->item, "favorites_theme/edit_no_effect", "favorites_theme");
}

API void phone_favor_edit_mode_start(ph_favorite_data *favor_d)
{
	PH_FN_CALL;
	Eina_List *list;
	Eina_List *cursor;
	Elm_Object_Item *it;
	ph_favor_info *f_info;
	p_retm_if(NULL == favor_d, "Favorite data is null");
	favor_d->list_mode = PH_FAVOR_MODE_EDIT;
	phone_favor_set_navi(favor_d->layout, PH_FAVOR_MODE_EDIT);
	if(favor_d->favor_list)
		phone_show_notify(favor_d->layout, T_(CT_GET_TEXT_BASIC, CTTEXT_SELECT_CONTACTS));
	elm_gengrid_reorder_mode_set(favor_d->favor_gengrid, EINA_TRUE);
	elm_gengrid_clear(favor_d->favor_gengrid);
	ph_favor_fill_favor_gengrid(favor_d);

	evas_object_smart_callback_add(favor_d->favor_gengrid, "realized", favor_gengrid_realized, favor_d);
	list = elm_gengrid_realized_items_get(favor_d->favor_gengrid);
	EINA_LIST_FOREACH(list, cursor, it) {
		f_info = elm_object_item_data_get(it);
		p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");
		elm_object_item_signal_emit(f_info->item, "favorites_theme/edit", "favorites_theme");
	}
}

static void favor_all_item_check_false(Eina_List *list)
{
	Eina_List *l;
	ph_favor_info *f_info;
	p_retm_if(NULL == list, "List is null");

	EINA_LIST_FOREACH(list, l, f_info) {
		if (NULL == f_info)
			continue;

		if (EINA_TRUE == f_info->checked)
			f_info->checked = EINA_FALSE;
	}
}

void ph_favor_edit_mode_end(ph_favorite_data *favor_d, bool redraw)
{
	Evas_Object *bg;
	Evas_Object *layout;

	favor_d->list_mode = PH_FAVOR_MODE_DEFAULT;

	layout = elm_object_part_content_get(favor_d->layout, "contents");
	bg = elm_object_part_content_unset(layout, "elm.swallow.bg");
	if (bg)
		evas_object_del(bg);

	phone_hide_notify(favor_d->layout);

	if (favor_d->checked_cnt)
		favor_d->checked_cnt = 0;

	if (favor_d->favor_gengrid) {
		elm_gengrid_clear(favor_d->favor_gengrid);
		if (true == favor_d->favor_move)
			ph_favor_load_favor_list(favor_d);

		ph_favor_fill_favor_gengrid(favor_d);
		if (0 == ctui_get_mfc_option()) {
			ph_favor_load_mf_list(favor_d);
			ph_favor_fill_mf_gengrid(favor_d);
		}
	}
	favor_all_item_check_false(favor_d->favor_list);
	elm_gengrid_reorder_mode_set(favor_d->favor_gengrid, EINA_FALSE);
	if (redraw)
		phone_favor_set_navi(favor_d->layout, PH_FAVOR_MODE_DEFAULT);
}

void ph_favor_edit_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	ph_favor_edit_mode_end(favor_d, true);
}
