/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <errno.h>
#include <fcntl.h>
#include <vconf.h>
#include <vconf-keys.h>
#include <unistd.h>
#include <pthread.h>
#include <media_content.h>
#include "phone.h"
#include "phone-common.h"

#include "ct-list.h"
#include "ct-setting-view.h"

#define CT_SETTING_PHONE_PATH "/opt/usr/media/Backup/Contacts"
#define CT_SETTING_MEMORY_CARD_PATH "/opt/storage/sdcard/Backup/Contacts"


#define CT_SETTING_TIMER_DURATION 0.2

enum SETTING_ITC_TYPE{
	SETTING_ITC_TEXT = 0,
	SETTING_ITC_IMPORT = SETTING_ITC_TEXT,
	SETTING_ITC_EXPORT = SETTING_ITC_TEXT,
	SETTING_ITC_IMPORT_EXPORT = SETTING_ITC_TEXT,
	SETTING_ITC_ORDER = 1,
	SETTING_ITC_RADIO = 2,
	SETTING_ITC_FIRSTLAST = SETTING_ITC_RADIO,
	SETTING_ITC_LASTFIRST = SETTING_ITC_RADIO,
	SETTING_ITC_MFC = 3,
	SETTING_ITC_MFC_RIDIO = 4,
	SETTING_ITC_SHOW = SETTING_ITC_MFC_RIDIO,
	SETTING_ITC_HIDE = SETTING_ITC_MFC_RIDIO,
	SETTING_ITC_SDN = 5,
	SETTING_ITC_MAX,
};

enum SETTING_STORAGE_TYPE{
	SETTING_STORAGE_PHONE = 0,
	SETTING_STORAGE_MEMORY_CARD = 1,
};

static Elm_Genlist_Item_Class setting_itcs[] = {
	{.item_style="dialogue/1text"},
	{.item_style="dialogue/2text.3/expandable"},
	{.item_style="dialogue/1text.1icon/expandable2"},
	{.item_style="dialogue/2text.3/expandable"},
	{.item_style="dialogue/1text.1icon/expandable2"},
	{.item_style="dialogue/2text.2"},
};

typedef enum {
	CT_SETTING_IMPORT,
	CT_SETTING_EXPORT,
} ct_setting_op;


static void __setting_insert_records(ct_setting_data *csetting_d);

static void setting_set_mfc_option(int mfc_option)
{
	int fd;
	char buf[PH_TEXT_MAX_LEN+1] = {0};

	fd = open(SETTING_MFC_PATH, O_WRONLY|O_CREAT|O_TRUNC, 0644);
	p_retm_if(fd < 0, "Open(%s) Failed", SETTING_MFC_PATH);
	snprintf(buf, sizeof(buf), "%d", mfc_option);
	if (-1 == write(fd, buf, strlen(buf)))
		ERR("write()... Failed(%s)", strerror(errno));
	close(fd);
}

static void setting_order(contacts_name_display_order_e order_type)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;

	/* set ORDER_OF_DISPLAY */
	err = contacts_setting_set_name_display_order(order_type);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_setting_set_name_display_order() Failed(%d)", err);
}


static void setting_order_ui(Evas_Object *genlist)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e order;
	ct_setting_data *csetting_d;
	Elm_Object_Item *item_order;

	p_retm_if(NULL == genlist, "parameter(genlist) is NULL");

	csetting_d = evas_object_data_get(genlist, "csetting_d");
	p_retm_if(NULL == csetting_d, "evas_object_data_get() return NULL");

	err = contacts_setting_get_name_display_order(&order);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_name_display_order_e() Failed(%d)", err);
	elm_radio_value_set(csetting_d->radio_order, (int)order);

	item_order = evas_object_data_get(genlist, "order");
	p_retm_if(NULL == item_order, "evas_object_data_get() return NULL");
	elm_genlist_item_update(item_order);
}

static void setting_handle_order(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = event_info;
	elm_genlist_item_selected_set(it, EINA_FALSE);

	setting_order((contacts_name_display_order_e)data);
	setting_order_ui(obj);
}

static void setting_handle_mfc(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d;
	Elm_Object_Item *it = event_info;
	Evas_Object *genlist = obj;
	Elm_Object_Item *item;
	elm_genlist_item_selected_set(it, EINA_FALSE);

	csetting_d = evas_object_data_get(genlist, "csetting_d");
	p_retm_if(NULL == csetting_d, "evas_object_data_get() return NULL");

	setting_set_mfc_option((int)data);
	elm_radio_value_set(csetting_d->radio_mfc, (int)data);

	item = evas_object_data_get(genlist, "mfc");
	p_retm_if(NULL == item, "evas_object_data_get() return NULL");

	elm_genlist_item_update(item);
}

static PTHREAD_FN setting_import_sim_contacts(void *data)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_connect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_connect2() Failed(%d)", err);

	ct_setting_data *csetting_d  = data;
	ph_progress_info *p_info = csetting_d->p_info;
	p_info->result = contacts_sim_import_all_contacts();
	PH_DBG("p_info->result = %d",p_info->result);
	p_info->completed = true;

	contacts_disconnect2();

	pthread_exit(NULL);
}

static Eina_Bool setting_import_sim_timer_cb(void *data)
{
	const char *message;
	ct_setting_data *csetting_d = data;
	ph_progress_info *p_info;

	p_retvm_if(NULL == csetting_d , ECORE_CALLBACK_CANCEL, "parameter(ct_setting_data) is NULL");
	p_info = csetting_d->p_info;

	if (!p_info->completed)
		return ECORE_CALLBACK_RENEW;

	pthread_join(p_info->thread, NULL);

	if (CONTACTS_ERROR_NONE == p_info->result)
		message = T_(CT_GET_TEXT_BASIC, CTTEXT_IMPORTED);
	else if (CONTACTS_ERROR_NO_DATA == p_info->result)
		message = T_(CT_GET_TEXT_ERR, CTTEXT_IMPORT_SIM_NO_DATA);
	else {
		switch (p_info->result) {
		case CONTACTS_ERROR_DB:
			message = T_(CT_GET_TEXT_ERR, CTTEXT_ERR_DB_LOCKED);
			break;
		case CONTACTS_ERROR_FILE_NO_SPACE:
			message = S_(CT_SYS_POP_NOT_ENOUGH_MEMORY);
			break;
		default:
			message = S_(PH_SYS_POP_FAILED);
			break;
		}
	}

	evas_object_del(csetting_d->popup);
	csetting_d->popup = NULL;

	free(p_info);
	csetting_d->p_info = NULL;

	phone_show_tickernoti(message);
	return ECORE_CALLBACK_CANCEL;
}

static void setting_import_sim(ct_setting_data *csetting_d)
{
	int ret;
	ph_progress_info *p_info;

	p_retm_if(csetting_d->p_info, "thread is running");

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc() return NULL");
	csetting_d->p_info = p_info;
	p_info->alive = true;
	p_info->result = TRUE;
	p_info->without_cancel_btn = true;
	ret = pthread_create(&p_info->thread, NULL, setting_import_sim_contacts, csetting_d );
	if (0 != ret) {
		ERR("Thread creation failed(%d)", ret);
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}

	p_info->timer = ecore_timer_add(CT_SETTING_TIMER_DURATION, setting_import_sim_timer_cb, csetting_d );
	if (NULL == p_info->timer) {
		ERR("ecore_timer_add() return NULL");
		p_info->alive = false;
		pthread_join(p_info->thread, NULL);
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}
	csetting_d->popup = phone_progressbar_popup(csetting_d->navi, p_info, T_(CT_GET_TEXT_BASIC, CTTEXT_IMPORTING_SIM_CONTACTS));
}

static inline void setting_progress_fail(ph_progress_info *p_info)
{
	p_info->completed = true;
	p_info->result = FALSE;
	pthread_exit(NULL);
}

static Eina_Bool setting_export_timer_cb(void *data)
{
	ct_setting_data *csetting_d  = data;
	ph_progress_info *p_info;

	p_retvm_if(NULL == csetting_d, ECORE_CALLBACK_CANCEL, "parameter(ct_setting_data) is NULL");
	p_info = csetting_d->p_info;

	if (!p_info->completed) {
		char count[PH_TEXT_SHORT_LEN];
		char percent[PH_TEXT_SHORT_LEN];
		double value = (double)p_info->cnt/(double)p_info->cnt_checked_total;

		if (!p_info->alive) {
			p_info->result = FALSE;
			p_info->completed = true;
			return ECORE_CALLBACK_RENEW;
		}

		elm_progressbar_value_set(p_info->progressbar, value);
		snprintf(percent, sizeof(percent), "%d%%", (int)(100.0*value));
		snprintf(count, sizeof(count), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
		edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
		edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", count);

		if (p_info->cnt == p_info->cnt_checked_total)
			p_info->completed = true;

		return ECORE_CALLBACK_RENEW;
	}
	else {
		const char *message;

		pthread_join(p_info->thread, NULL);

		if (p_info->result)
			message = T_(CT_GET_TEXT_BASIC, CTTEXT_EXPORTED);
		else
			message = S_(PH_SYS_POP_FAILED);

		free(p_info);
		csetting_d->p_info = NULL;

		if (csetting_d->list_export_ct) {
			free(csetting_d->list_export_ct);
			csetting_d->list_export_ct = NULL;
		}

		evas_object_del(csetting_d->popup);
		csetting_d->popup = NULL;
		phone_show_tickernoti(message);
		elm_naviframe_item_pop_to(csetting_d->navi_item_main);
		return ECORE_CALLBACK_CANCEL;
	}
}

static Eina_Bool setting_import_timer_cb(void *data)
{
	ct_setting_data *csetting_d  = data;
	ph_progress_info *p_info;

	p_retvm_if(NULL == csetting_d, ECORE_CALLBACK_CANCEL, "parameter(ct_setting_data) is NULL");
	p_info = csetting_d->p_info;

	if(p_info->next_job) {
		pthread_join(p_info->thread, NULL);
		__setting_insert_records(csetting_d);
		p_info->next_job = false;
	}

	if (!p_info->alive && !p_info->completed) {
		p_info->result = FALSE;
		p_info->completed = true;
	}

	if (!p_info->completed) {
		return ECORE_CALLBACK_RENEW;
	}
	else {
		const char *message;

		pthread_join(p_info->thread, NULL);

		if (p_info->result)
			message = T_(CT_GET_TEXT_BASIC, CTTEXT_IMPORTED);
		else
			message = S_(PH_SYS_POP_FAILED);

		evas_object_del(csetting_d->popup);
		csetting_d->popup = NULL;
		phone_show_tickernoti(message);

		free(p_info);
		csetting_d->p_info = NULL;

		elm_naviframe_item_pop_to(csetting_d->navi_item_main);
		return ECORE_CALLBACK_CANCEL;
	}
}


static void __setting_update_media_db(const char *path)
{
	p_retm_if(NULL == path, "path is NULL");

	int error = media_content_connect();
	p_retm_if(error != MEDIA_CONTENT_ERROR_NONE, "media_content_connect(%s) is failed(%d)", path, error);

	error = media_content_scan_file(path);
	p_retm_if(error != MEDIA_CONTENT_ERROR_NONE, "media_content_scan_file(%s) is failed(%d)", path, error);

	error = media_content_disconnect();
	p_retm_if(error != MEDIA_CONTENT_ERROR_NONE, "media_content_disconnect(%s) is failed(%d)", path, error);
}

static PTHREAD_FN setting_export_contacts(void *data)
{
	int i;
	int ret;
	int fd;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	ct_setting_data *csetting_d  = data;
	ph_progress_info *p_info = csetting_d->p_info;

	fd = open(csetting_d->path, O_WRONLY|O_CREAT|O_TRUNC, 0644);
	if (fd < 0) {
		ERR("open(%s) Failed", csetting_d->path);
		setting_progress_fail(p_info);
	}
	else
		__setting_update_media_db(csetting_d->path);

	err = contacts_connect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_connect2() Failed(%d)", err);

	for (i=0;i<csetting_d->list_export_ct_cnt;i++) {
		char* vcard;

		if (!p_info->alive) {
			p_info->result = FALSE;
			p_info->completed = true;
			break;
		}
		PH_DBG("csetting_d->list_export_ct[%d] = %d",i,csetting_d->list_export_ct[i]);
		err = contacts_db_get_record(_contacts_person._uri, csetting_d->list_export_ct[i], &record);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_db_get_record() Failed(%d)", err);
			p_info->result = FALSE;
			continue;
		}

		err = contacts_vcard_make_from_person(record, &vcard);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_vcard_make_from_person() Failed(%d)", err);
			p_info->result = FALSE;
			err = contacts_record_destroy(record, true);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
			continue;
		}
		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

		if (-1 == write(fd, vcard, strlen(vcard)))
			ERR("write()... Failed(%s)", strerror(errno));
		else
			__setting_update_media_db(csetting_d->path);

		p_info->cnt++;
		free(vcard);
	}

	ret = fsync(fd);
	if (0 != ret)
		ERR("fsync() failed(%d)", ret);

	close(fd);
	if (!p_info->result) {
		ret = unlink(csetting_d->path);
		if (0 != ret)
			ERR("unlink(%s) failed(%d)", csetting_d->path, ret);
	}

	contacts_disconnect2();

	pthread_exit(NULL);
}

static void setting_get_filename(char *fullpath, char *dest, int size_dest)
{
	char* index;
	index = strrchr(fullpath, '/');
	snprintf(dest, size_dest, "%s", index + 1);
}

static inline char* setting_export_get_filename(char *path)
{
	int ret;
	time_t timer;
	struct tm *stime;
	char buf[PH_TEXT_MAX_LEN+1] = {0};
	char mkcmd[PH_TEXT_MAX_LEN+1] = {0};

	timer = time(NULL);
	stime = localtime(&timer);
	p_retvm_if(NULL == stime, NULL, "localtime() Failed(%d)", errno);

	sprintf(mkcmd, "mkdir -p %s", path);
	ret = system(mkcmd);

	snprintf(buf, sizeof(buf), "%s/contacts_%02d%02d%02d%02d%02d%02d.vcf", path,
			stime->tm_year-100, stime->tm_mon+1, stime->tm_mday,
			stime->tm_hour, stime->tm_min, stime->tm_sec);

	return strdup(buf);
}

static void setting_ug_destroy_cb(ui_gadget_h ug, void *priv)
{
	ct_setting_data *csetting_d = priv;
	p_retm_if(NULL == ug, "parameter(ug) is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == priv, "parameter(ct_setting_data) is NULL");
	csetting_d->child_ug = NULL;
}

static bool setting_import_vcard_foreach_cb(contacts_record_h record, void *data)
{
	int ret_id = 0;
	contacts_error_e err;
	ph_progress_info *p_info;
	ct_setting_data *csetting_d = data;
	contacts_record_h record_clone = NULL;

	p_retvm_if(NULL == csetting_d, -1, "parameter(csetting_d) is NULL");

	p_info = csetting_d->p_info;
	p_retvm_if(NULL == p_info, -1, "csetting_d->p_info is NULL");

	PH_DBG("setting_import_vcard_foreach_cb total_cnt %d", csetting_d->import_total_cnt);

	if (!p_info->alive) {
		p_info->result = FALSE;
		p_info->completed = true;
		return false;
	}

	err = contacts_record_clone(record, &record_clone);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_record_clone() Failed(%d)", err);
		return false;
	}
	csetting_d->list_import_records = eina_list_append(csetting_d->list_import_records, record_clone);
	csetting_d->import_total_cnt++;

	return true;
}



static void __setting_insert_records_cb(int err, int *ids, unsigned int count, void *data)
{
	ct_setting_data *csetting_d = data;
	contacts_error_e _err = CONTACTS_ERROR_NONE;

	if(CONTACTS_ERROR_NONE != err) {
		if(csetting_d->p_info) {
			csetting_d->p_info->completed = true;
			csetting_d->p_info->alive = false;
			csetting_d->p_info->result = false;
		}
		ctui_create_contacts_error_popup(csetting_d->navi, err);
		return;
	}

	_err = contacts_list_destroy(csetting_d->import_list, true);
	p_retm_if(CONTACTS_ERROR_NONE != _err, "contacts_list_destroy() Failed(%d)", _err);

	PH_DBG("__setting_insert_records_cb total count %d, count %d", csetting_d->import_total_cnt, csetting_d->import_cnt);

	if(csetting_d->p_info) {
		if(!csetting_d->p_info->alive) {
			csetting_d->p_info->result = FALSE;
			csetting_d->p_info->completed = true;
			return;
		}
	}
	else
		return;

	if(csetting_d->import_cnt == csetting_d->import_total_cnt) {
		if(csetting_d->p_info) {
			csetting_d->p_info->completed = true;
			csetting_d->p_info->alive = false;
			csetting_d->p_info->result = true;
		}
		return;
	}

	__setting_insert_records(csetting_d);
}

static void __setting_insert_records(ct_setting_data *csetting_d)
{
	const int LIST_BULK_LENGTH = 50;
	Eina_List *l;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	err = contacts_list_create(&csetting_d->import_list);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_list_create() Failed(%d)", err);

	EINA_LIST_FOREACH(csetting_d->list_import_records, l, record_temp) {
		err = contacts_list_add(csetting_d->import_list, record_temp);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_list_add() Failed(%d) count %d", err, csetting_d->import_cnt);
		csetting_d->list_import_records = eina_list_remove_list(csetting_d->list_import_records, l);

		csetting_d->import_cnt++;

		if((csetting_d->import_cnt % LIST_BULK_LENGTH) == 0)
			break;

	}
	err = contacts_db_insert_records_async(csetting_d->import_list, __setting_insert_records_cb, csetting_d);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_insert_records_async() Failed(%d)", err);
}

static PTHREAD_FN setting_import_vcard_contacts(void *data)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char *path = NULL;
	char *last = NULL;
	ct_setting_data *csetting_d  = data;
	ph_progress_info *p_info = csetting_d->p_info;

	err = contacts_connect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_connect2() Failed(%d)", err);

	path = strtok_r(csetting_d->path, "?", &last);
	while (path) {
		err = contacts_vcard_parse_to_contact_foreach(path, setting_import_vcard_foreach_cb, csetting_d);
		if (CONTACTS_ERROR_NONE != err) {
			p_info->result = FALSE;
			ERR("contacts_svc_vcard_foreach() Failed(%d)", p_info->result);
			break;
		}
		path = strtok_r(NULL, "?", &last);
	}

	p_info->next_job = true;

	contacts_disconnect2();

	pthread_exit(NULL);
}


static void setting_import_vcard_myfile_ug_result(ui_gadget_h ug, service_h service, void *data)
{
	int ret;
	char *val = NULL;
	ct_setting_data *csetting_d = data;
	ph_progress_info *p_info;

	p_retm_if(NULL == ug, "parameter(ug) is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == data, "parameter(ct_setting_data) is NULL");
	csetting_d->child_ug = NULL;

	p_retm_if(NULL == service, "parameter(result) is NULL");

	service_get_extra_data(service, "result", &val);
	p_retm_if(NULL == val, "service_get_extra_data() return NULL");

	FREEandSTRDUP(csetting_d->path, val);
	p_retm_if(NULL == csetting_d->path, "strdup() return NULL");

	free(val);

	p_retm_if(csetting_d->p_info, "thread is running");

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc() return NULL");
	csetting_d->p_info = p_info;
	p_info->alive = true;
	p_info->result = TRUE;

	ret = pthread_create(&p_info->thread, NULL, setting_import_vcard_contacts, csetting_d);
	if (0 != ret) {
		ERR("Thread creation failed(%d)", ret);
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}

	p_info->timer = ecore_timer_add(CT_SETTING_TIMER_DURATION, setting_import_timer_cb, csetting_d );
	if (NULL == p_info->timer) {
		ERR("ecore_timer_add() return NULL");
		p_info->alive = false;
		pthread_join(p_info->thread, NULL);
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}
	csetting_d->popup = phone_progressbar_popup(csetting_d->navi, p_info, T_(CT_GET_TEXT_BASIC, CTTEXT_IMPORTING_SD_CONTACTS));
}

static void setting_import_vcard_myfile_ug(ct_setting_data *csetting_d)
{
	ui_gadget_h ug;
	struct ug_cbs cbs = {0};
	service_h service;

	service_create(&service);
	service_add_extra_data(service, "path", CT_SETTING_PHONE_PATH);
	service_add_extra_data(service, "select_type", "IMPORT");
	service_add_extra_data(service, "file_type", "vcf");

	cbs.result_cb = setting_import_vcard_myfile_ug_result;
	cbs.destroy_cb = setting_ug_destroy_cb;
	cbs.layout_cb = ug_common_layout_cb;
	cbs.priv = csetting_d;

	ug = ug_create(csetting_d->ug, MYFILES_UG, UG_MODE_FULLVIEW, service, &cbs);
	p_warn_if(NULL == ug, "ug_create() Failed");
	csetting_d->child_ug = ug;

	service_destroy(service);
}

static PTHREAD_FN setting_export_sim_contacts(void *data)
{
	PH_FN_CALL;
	int i;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_setting_data *csetting_d  = data;

	csetting_d->p_info->result = CONTACTS_ERROR_NONE;
	for (i=0;i<csetting_d->list_export_ct_cnt;i++) {
		err = contacts_sim_export_person(csetting_d->list_export_ct[i]);
		if (CONTACTS_ERROR_NONE == err) {
			csetting_d->p_info->cnt++;
		}
		else {
			csetting_d->p_info->result = err;
			ERR("contacts_sim_export_person() Failed(%d)", err);
		}
	}
	pthread_exit(NULL);
}

static Eina_Bool setting_export_sim_timer_cb(void *data)
{
	PH_FN_CALL;
	const char *message;
	ct_setting_data *csetting_d = data;
	ph_progress_info *p_info;
	p_info = csetting_d->p_info;

	double value = (double)p_info->cnt/(double)p_info->cnt_checked_total;
	p_retvm_if(NULL == csetting_d , ECORE_CALLBACK_CANCEL, "parameter(ct_setting_data) is NULL");
	p_info = csetting_d->p_info;

	if (!p_info->completed)
	{
		char count[PH_TEXT_SHORT_LEN];
		char percent[PH_TEXT_SHORT_LEN];
		if (!p_info->alive)
		{
			p_info->result = FALSE;
			p_info->completed = true;
			return ECORE_CALLBACK_RENEW;
		}
		elm_progressbar_value_set(p_info->progressbar, value);
		snprintf(percent, sizeof(percent), "%d%%", (int)(100.0*value));
		snprintf(count, sizeof(count), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
		edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
		edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", count);

		if(p_info->cnt == p_info->cnt_checked_total)
			p_info->completed = true;

		if(p_info->result == CONTACTS_ERROR_INTERNAL)
			p_info->completed = true;

		return ECORE_CALLBACK_RENEW;
	}
	pthread_join(p_info->thread, NULL);

	if (CONTACTS_ERROR_NONE == p_info->result)
		message = T_(CT_GET_TEXT_BASIC, CTTEXT_EXPORTED);
	else
		message = S_(PH_SYS_POP_FAILED);


	evas_object_del(csetting_d->popup);
	csetting_d->popup = NULL;

	free(p_info);
	csetting_d->p_info = NULL;

	phone_show_tickernoti(message);
	return ECORE_CALLBACK_CANCEL;
}
static void setting_export_sim(void *data, int *list, int cnt)
{
	PH_FN_CALL;
	int ret;
	ct_setting_data *csetting_d = data;
	ph_progress_info *p_info;
	char title[PH_TEXT_MAX_LEN] = {0,};

	if (NULL == csetting_d) {
		ERR("parameter(csetting_d) is NULL");
		return;
	}

	if (cnt <= 0) {
		ERR("evas_object_data_get() return NULL");
		return;
	}
	if(list == NULL){
		ERR("list is NULL");
		return;
	}
	p_retm_if(csetting_d->p_info, "thread is running");
	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc() return NULL");

	csetting_d->p_info = p_info;
	p_info->cnt_checked_total = csetting_d->list_export_ct_cnt = cnt;
	int *export_list = calloc(cnt,sizeof(int));
	memcpy(export_list,list,sizeof(int)*cnt);
	csetting_d->list_export_ct = export_list;

	p_info->alive = true;
	p_info->result = TRUE;
	p_info->without_cancel_btn = true;

	ret = pthread_create(&p_info->thread, NULL, setting_export_sim_contacts, csetting_d );
	if (0 != ret) {
		ERR("Thread creation failed(%d)", ret);
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}

	p_info->timer = ecore_timer_add(CT_SETTING_TIMER_DURATION, setting_export_sim_timer_cb, csetting_d );
	if (NULL == p_info->timer) {
		ERR("ecore_timer_add() return NULL");
		p_info->alive = false;
		free(p_info);
		csetting_d->p_info = NULL;
		return;
	}
	snprintf(title, sizeof(title), T_(CT_GET_TEXT_BASIC, CTTEXT_EXPORT_TO_SIM_CARD));
	csetting_d->popup = phone_progressbar_popup(csetting_d->navi, p_info, title);

	return;

}
static void setting_import_export_sim_sel(void *data, Evas_Object *obj, void *event_info)
{
	int cnt_max = 0;
	ct_setting_data *csetting_d = data;
	contacts_error_e err;
	elm_genlist_item_selected_set(event_info, EINA_FALSE);

	if (CT_SETTING_IMPORT == csetting_d->mode) {
		setting_import_sim(csetting_d);
	}
	else { // CT_SETTING_EXPORT
		err = contacts_sim_get_empty_record_count(&cnt_max);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_sim_get_empty_record_count() Failed(%d)", err);

		if (0 == cnt_max) {
			phone_show_popup(csetting_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_SIM_MEMORY_FULL), 2.0);
			PH_DBG("cnt_max is zero");
			return;
		}
		ctui_create_list_select_view(csetting_d->navi, setting_export_sim, csetting_d, CT_GET_LIST_ALL_PERSON, 0, 0, cnt_max);
	}
}

static void setting_export_vcard_contacts(void *data, int *list, int cnt)
{
	int ret;
	char title[PH_TEXT_MAX_LEN]= {0};;
	char filepath[PH_TEXT_MAX_LEN+1]= {0};;
	ph_progress_info *p_info;
	ct_setting_data *csetting_d = data;

	if (NULL == csetting_d) {
		ERR("parameter(csetting_d) is NULL");
		return;
	}

	if (cnt <= 0) {
		ERR("evas_object_data_get() return NULL");
		return;
	}
	if(list == NULL){
		ERR("list is NULL");
		return;
	}
	int *export_list = calloc(cnt,sizeof(int));
	memcpy(export_list,list,sizeof(int)*cnt);
	csetting_d->list_export_ct = export_list;
	csetting_d->list_export_ct_cnt = cnt;

	p_retm_if(csetting_d->p_info, "thread is running");

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc() return NULL");
	csetting_d->p_info = p_info;

	p_info->cnt_checked_total = csetting_d->list_export_ct_cnt;
	if (p_info->cnt_checked_total <= 0) {
		char err[PH_TEXT_SHORT_LEN];
		ERR("cnt_checked_total is %d", p_info->cnt_checked_total);
		free(p_info);
		free(csetting_d->list_export_ct);
		csetting_d->p_info = NULL;
		csetting_d->list_export_ct = NULL;
		snprintf(err, sizeof(err), "Fail (%s)", T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS));
		phone_show_tickernoti(err);
		return;
	}
	free(csetting_d->path);
	if (SETTING_STORAGE_MEMORY_CARD == csetting_d->storage_type) {
		csetting_d->path = setting_export_get_filename(CT_SETTING_MEMORY_CARD_PATH);
	}
	else{
		csetting_d->path = setting_export_get_filename(CT_SETTING_PHONE_PATH);
	}

	p_info->alive = true;
	p_info->result = TRUE;
	setting_get_filename(csetting_d->path, filepath, sizeof(filepath));
	snprintf(title, sizeof(title), T_(CT_GET_TEXT_BASIC, CTTEXT_EXPORTING_CONTACTS_TO), filepath);
	ret = pthread_create(&p_info->thread, NULL, setting_export_contacts, csetting_d );
	if (0 != ret) {
		ERR("Thread creation failed(%d)", ret);
		free(p_info);
		free(csetting_d->list_export_ct);
		csetting_d->p_info = NULL;
		csetting_d->list_export_ct = NULL;
		return;
	}

	p_info->timer = ecore_timer_add(CT_SETTING_TIMER_DURATION, setting_export_timer_cb, csetting_d );
	if (NULL == p_info->timer) {
		ERR("ecore_timer_add() return NULL");
		p_info->alive = false;
		pthread_join(p_info->thread, NULL);
		free(p_info);
		free(csetting_d->list_export_ct);
		csetting_d->p_info = NULL;
		csetting_d->list_export_ct = NULL;
		return;
	}

	csetting_d->popup = phone_progressbar_popup(csetting_d->navi, p_info, title);
}

static void setting_import_export_phone_sel(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d = data;
	elm_genlist_item_selected_set(event_info, EINA_FALSE);
	csetting_d->storage_type = SETTING_STORAGE_PHONE;
	if (CT_SETTING_IMPORT == csetting_d->mode)
		setting_import_vcard_myfile_ug(csetting_d);
	else {// CT_SETTING_EXPORT
		ctui_create_list_select_view(csetting_d->navi, setting_export_vcard_contacts, csetting_d, CT_GET_LIST_ALL_PERSON, 0, 0, 0);
	}
}

static void setting_import_export_memory_card_sel(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d = data;
	elm_genlist_item_selected_set(event_info, EINA_FALSE);
	csetting_d->storage_type = SETTING_STORAGE_MEMORY_CARD;
	if (CT_SETTING_IMPORT == csetting_d->mode)
		setting_import_vcard_myfile_ug(csetting_d);
	else {// CT_SETTING_EXPORT
		ctui_create_list_select_view(csetting_d->navi, setting_export_vcard_contacts, csetting_d, CT_GET_LIST_ALL_PERSON, 0, 0, 0);
	}
}

static void setting_import_export(ct_setting_data *csetting_d)
{
	int sim_state;
	int mmc_mode = VCONFKEY_SYSMAN_MMC_REMOVED;
	char buf[PH_TEXT_MAX_LEN+1]= {0};;
	Evas_Object *genlist;
	Evas_Object *l_btn;
	Elm_Object_Item *item;

	genlist = elm_genlist_add(csetting_d->navi);
	elm_object_style_set(genlist,"dialogue");
	vconf_get_int(VCONFKEY_TELEPHONY_SIM_PB_INIT, &sim_state);

	ctui_genlist_append_separator_20px_with_line(genlist);

	PH_DBG("import/export sim_state is %d",sim_state);
	item = elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_IMPORT_EXPORT], S_(CT_SYS_BODY_SIM), NULL, ELM_GENLIST_ITEM_NONE, setting_import_export_sim_sel, csetting_d);
	if(sim_state <= 0)
		elm_object_item_disabled_set(item, EINA_TRUE);

	if (CT_SETTING_EXPORT == csetting_d->mode) {
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_IMPORT_EXPORT], T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE), NULL, ELM_GENLIST_ITEM_NONE, setting_import_export_phone_sel, csetting_d);

		item = elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_IMPORT_EXPORT], S_(CT_SYS_BODY_MEMORYCARD), NULL, ELM_GENLIST_ITEM_NONE, setting_import_export_memory_card_sel, csetting_d);
		vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &mmc_mode);
	   if (mmc_mode == VCONFKEY_SYSMAN_MMC_REMOVED || mmc_mode == VCONFKEY_SYSMAN_MMC_INSERTED_NOT_MOUNTED)
			elm_object_item_disabled_set(item, EINA_TRUE);
		snprintf(buf, sizeof(buf), "%s", T_(CT_GET_TEXT_BASE_OP, CTTEXT_EXPORT_CONTACTS));
	}
	else if (CT_SETTING_IMPORT == csetting_d->mode) {
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_IMPORT_EXPORT], S_(CT_SYS_BODY_MEMORYCARD), NULL, ELM_GENLIST_ITEM_NONE, setting_import_export_phone_sel, csetting_d);
		snprintf(buf, sizeof(buf), "%s", T_(CT_GET_TEXT_BASE_OP, CTTEXT_IMPORT_CONTACTS));
	}

	l_btn = elm_button_add(csetting_d->navi);
	evas_object_smart_callback_add(l_btn, "clicked", phone_view_back_btn_cb, csetting_d->navi);
	elm_object_style_set(l_btn, "naviframe/back_btn/default");

	elm_naviframe_item_push(csetting_d->navi, buf, l_btn, NULL, genlist, NULL);
}

static void setting_export_contacts_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d = data;
	elm_genlist_item_selected_set(event_info, EINA_FALSE);

	csetting_d->mode = CT_SETTING_EXPORT;
	setting_import_export(csetting_d);
}

static void setting_import_contacts_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d = data;
	elm_genlist_item_selected_set(event_info, EINA_FALSE);

	csetting_d->mode = CT_SETTING_IMPORT;
	setting_import_export(csetting_d);
}

static void setting_order_item_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = event_info;
	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (EINA_TRUE == elm_genlist_item_expanded_get(item))
		elm_genlist_item_expanded_set(item, EINA_FALSE);
	else
		elm_genlist_item_expanded_set(item, EINA_TRUE);
}

static void setting_mfc_item_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = event_info;
	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (EINA_TRUE == elm_genlist_item_expanded_get(item))
		elm_genlist_item_expanded_set(item, EINA_FALSE);
	else
		elm_genlist_item_expanded_set(item, EINA_TRUE);
}

static void setting_gl_exp(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e order;
	int mfc_option;
	Evas_Object *genlist = obj;
	Elm_Object_Item *it = event_info;
	ct_setting_data *csetting_d = data;

	if (evas_object_data_get(obj, "order") == it) {
		csetting_d->radio_order = elm_radio_add(genlist);
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_FIRSTLAST],
				(void *)CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST, it, ELM_GENLIST_ITEM_NONE,
				setting_handle_order, (void *)CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST);
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_LASTFIRST],
				(void *)CONTACTS_NAME_DISPLAY_ORDER_LASTFIRST, it, ELM_GENLIST_ITEM_NONE,
				setting_handle_order, (void *)CONTACTS_NAME_DISPLAY_ORDER_LASTFIRST);

		err = contacts_setting_get_name_display_order(&order);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_name_display_order_e() Failed(%d)", err);
		elm_radio_value_set(csetting_d->radio_order, (int)order);
	}
	else if (evas_object_data_get(obj, "mfc") == it) {
		csetting_d->radio_mfc = elm_radio_add(genlist);
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_SHOW],
				(void *)0, it, ELM_GENLIST_ITEM_NONE,
				setting_handle_mfc, (void *)0);
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_HIDE],
				(void *)1, it, ELM_GENLIST_ITEM_NONE,
				setting_handle_mfc, (void *)1);
		mfc_option = ctui_get_mfc_option();
		elm_radio_value_set(csetting_d->radio_mfc, mfc_option);
	}

}

static void setting_gl_con(void *data, Evas_Object *obj, void *event_info)
{
	ct_setting_data *csetting_d = data;
	Elm_Object_Item *item = event_info;
	elm_genlist_item_subitems_clear(item);

	p_retm_if(NULL == csetting_d, "parameter(csetting_d) is NULL");

	if (evas_object_data_get(obj, "order") == item) {
		if(csetting_d->radio_order) {
			evas_object_del(csetting_d->radio_order);
			csetting_d->radio_order = NULL;
		}
	}
	else if (evas_object_data_get(obj, "mfc") == item) {
		if(csetting_d->radio_mfc) {
			evas_object_del(csetting_d->radio_mfc);
			csetting_d->radio_mfc = NULL;
		}
	}
}

static char *setting_gl_text_label_get(void *data, Evas_Object *obj, const char *part)
{
	if (0 == strcmp(part, "elm.text"))
		return strdup(data);
	return NULL;
}

static char *setting_gl_order_label_get(void *data, Evas_Object *obj, const char *part)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e order;

	if (0 == strcmp(part, "elm.text.1"))
		return strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_ORDER_DISPLAY_NAME));
	else if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_setting_get_name_display_order(&order);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_name_display_order_e() Failed(%d)", err);

		if (CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST == order)
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_FIRSTLAST));
		else
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_LASTFIRST));
	}
	return NULL;
}

static char *setting_gl_mf_label_get(void *data, Evas_Object *obj, const char *part)
{
	if (0 == strcmp(part, "elm.text.1"))
		return strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_MOST_FREQUENT_CONTACTS));
	else if (0 == strcmp(part, "elm.text.2")) {
		if (0 == ctui_get_mfc_option())
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_SHOW));
		else if (1 == ctui_get_mfc_option())
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_HIDE));
	}
	return NULL;
}

static char *setting_gl_mf_radio_label_get(void *data, Evas_Object *obj, const char *part)
{
	PH_FN_CALL;
	int type = (int)data;
	if (0 == strcmp(part, "elm.text")) {
		if (0 == type)
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_SHOW));
		else if (1 == type)
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_HIDE));
	}
	return NULL;
}

static char *setting_gl_radio_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type = (int)data;
	if (0 == strcmp(part, "elm.text")) {
		if (CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST == type)
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_FIRSTLAST));
		else if (CONTACTS_NAME_DISPLAY_ORDER_LASTFIRST == type)
			return strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_LASTFIRST));
	}
	return NULL;
}

static Evas_Object *setting_gl_radio_icon_get(void *data, Evas_Object *obj, const char *part)
{
	int type = (int)data;
	ct_setting_data *csetting_d;

	if (0 == strcmp(part, "elm.icon")) {
		Evas_Object *radio;

		csetting_d = evas_object_data_get(obj, "csetting_d");
		p_retvm_if(NULL == csetting_d, NULL, "evas_object_data_get() return NULL");

		radio = elm_radio_add(obj);
		p_retvm_if(NULL == radio, NULL, "elm_radio_add() return NULL");

		elm_radio_group_add(radio, csetting_d->radio_order);
		elm_radio_state_value_set(radio, type);

		return radio;
	}
	return NULL;
}

static Evas_Object *setting_gl_mf_radio_icon_get(void *data, Evas_Object *obj, const char *part)
{
	int type = (int)data;
	ct_setting_data *csetting_d;

	if (0 == strcmp(part, "elm.icon")) {
		Evas_Object *radio;

		csetting_d = evas_object_data_get(obj, "csetting_d");
		p_retvm_if(NULL == csetting_d, NULL, "evas_object_data_get() return NULL");

		radio = elm_radio_add(obj);
		p_retvm_if(NULL == radio, NULL, "elm_radio_add() return NULL");

		elm_radio_group_add(radio, csetting_d->radio_mfc);
		elm_radio_state_value_set(radio, type);

		return radio;
	}
	return NULL;
}

static char* setting_gl_sdn_label_get(void *data, Evas_Object *obj, const char *part)
{
	ct_sdn_list *info = data;
	p_retvm_if(NULL == data, NULL, "The parameter(data) is NULL");

	if (0 == strcmp(part, "elm.text.2"))
		return SAFE_STRDUP(info->name);
	else if (0 == strcmp(part, "elm.text.1"))
		return SAFE_STRDUP(info->number);
	return NULL;
}

static void setting_gl_sdn_del(void *data, Evas_Object *obj)
{
	ct_sdn_list *info = data;
	p_retm_if(NULL == info, "parameter(info) is NULL");

	free(info->name);
	free(info->number);
	free(info);
}


static inline void setting_create_itcs()
{
	setting_itcs[SETTING_ITC_TEXT].func.text_get = setting_gl_text_label_get;

	setting_itcs[SETTING_ITC_ORDER].func.text_get = setting_gl_order_label_get;

	setting_itcs[SETTING_ITC_MFC].func.text_get = setting_gl_mf_label_get;

	setting_itcs[SETTING_ITC_RADIO].func.text_get = setting_gl_radio_label_get;
	setting_itcs[SETTING_ITC_MFC_RIDIO].func.text_get = setting_gl_mf_radio_label_get;
	setting_itcs[SETTING_ITC_RADIO].func.content_get = setting_gl_radio_icon_get;
	setting_itcs[SETTING_ITC_MFC_RIDIO].func.content_get = setting_gl_mf_radio_icon_get;

	setting_itcs[SETTING_ITC_SDN].func.text_get = setting_gl_sdn_label_get;
	setting_itcs[SETTING_ITC_SDN].func.del = setting_gl_sdn_del;
}

static void setting_view_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Eina_List *l;
	contacts_record_h record = NULL;
	ct_setting_data *csetting_d = data;
	p_retm_if(NULL == csetting_d, "parameter(csetting_d) is NULL");

	if (csetting_d->popup)
		evas_object_del(csetting_d->popup);

	EINA_LIST_FOREACH(csetting_d->list_import_records, l, record) {
		contacts_record_destroy(record, true);
	}
	eina_list_free(csetting_d->list_import_records);

	if (csetting_d->p_info) {
		csetting_d->p_info->alive = false;
		pthread_join(csetting_d->p_info->thread, NULL);

		ecore_timer_del(csetting_d->p_info->timer);
		free(csetting_d->p_info);
		csetting_d->p_info = NULL;
	}

	if (csetting_d->child_ug)
		ug_destroy(csetting_d->child_ug);

	free(csetting_d->path);
	free(csetting_d);
}

static void setting_gl_sdn_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = event_info;
	ct_sdn_list *info = elm_object_item_data_get(item);

	elm_genlist_item_selected_set(item, EINA_FALSE);
	phone_launch_voice_call(0, info->number);
}


static Evas_Object* setting_fill_sdn_view(Evas_Object *parent, ct_setting_data *csetting_d)
{
	Evas_Object *genlist;

	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list = NULL;

	genlist = elm_genlist_add(parent);
	elm_object_style_set(genlist,"dialogue");
	p_retvm_if(NULL == genlist, NULL, "elm_genlist_add() return NULL");

	ctui_genlist_append_separator_20px_with_line(genlist);

	err = contacts_db_get_all_records(_contacts_sdn._uri, 0, 0, &list);
	p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_db_get_all_records...() Failed(%d)", err);

	csetting_d->cnt_sdn_list = 0;
	while (CONTACTS_ERROR_NONE == err) {
		ct_sdn_list *data;

		data = calloc(1, sizeof(ct_sdn_list));
		if (NULL == data) {
			ERR("calloc() return NULL");
			break;
		}
		err = phone_cts_get_str_from_list(list, _contacts_sdn.name, &(data->name));
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);

		err = phone_cts_get_str_from_list(list, _contacts_sdn.number, &(data->number));
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);

		csetting_d->cnt_sdn_list++;
		elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_SDN], data, NULL, ELM_GENLIST_ITEM_NONE, setting_gl_sdn_sel, NULL);

		err = contacts_list_next(list);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
	}
	err = contacts_list_destroy(list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

	return genlist;
}

static void setting_create_genlist_sdn(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *back_btn;
	Evas_Object *genlist;
	ct_setting_data *csetting_d = data;
	char buf[PH_TEXT_MAX_LEN+1]= {0};;

	elm_genlist_item_selected_set(event_info, EINA_FALSE);

	genlist = setting_fill_sdn_view(csetting_d->navi, csetting_d);

	back_btn = elm_button_add(csetting_d->navi);
	evas_object_smart_callback_add(back_btn, "clicked", phone_view_back_btn_cb, csetting_d->navi);
	elm_object_style_set(back_btn, "naviframe/back_btn/default");

	snprintf(buf, sizeof(buf), "%s (%d)", T_(CT_GET_TEXT_BASIC, CTTEXT_SDN), csetting_d->cnt_sdn_list);

	elm_naviframe_item_push(csetting_d->navi, buf, back_btn, NULL, genlist, NULL);
}

Evas_Object* ct_setting_create_layout(ct_setting_data *csetting_d)
{
	unsigned int sdn_count;
	contacts_list_h list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Evas_Object *genlist;
	Elm_Object_Item *item;

	setting_create_itcs();

	genlist = elm_genlist_add(csetting_d->navi);
	elm_object_style_set(genlist,"dialogue");
	evas_object_data_set(genlist, "csetting_d", csetting_d);
	evas_object_event_callback_add(genlist, EVAS_CALLBACK_DEL, setting_view_delete_cb, csetting_d);

	ctui_genlist_append_separator_20px_with_line(genlist);

	elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_IMPORT],
			T_(CT_GET_TEXT_BASE_OP, CTTEXT_IMPORT_CONTACTS), NULL, ELM_GENLIST_ITEM_NONE,
			setting_import_contacts_cb, csetting_d);
	item = elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_ORDER],
			NULL, NULL, ELM_GENLIST_ITEM_TREE, setting_order_item_sel, NULL);
	evas_object_data_set(genlist, "order", item);

	item = elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_MFC],
			NULL, NULL, ELM_GENLIST_ITEM_TREE, setting_mfc_item_sel, NULL);
	evas_object_data_set(genlist, "mfc", item);

	err = contacts_db_get_all_records(_contacts_sdn._uri, 0, 0, &list);
	if (CONTACTS_ERROR_NONE == err) {
		err = contacts_list_get_count(list, &sdn_count);
		if (CONTACTS_ERROR_NONE == err && sdn_count) {
			elm_genlist_item_append(genlist, &setting_itcs[SETTING_ITC_TEXT],
					T_(CT_GET_TEXT_BASIC, CTTEXT_SDN), NULL, ELM_GENLIST_ITEM_NONE, setting_create_genlist_sdn, csetting_d);
		}
		err = contacts_list_destroy(list, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);
	}

	evas_object_smart_callback_add(genlist, "expanded", setting_gl_exp, csetting_d);
	evas_object_smart_callback_add(genlist, "contracted", setting_gl_con, csetting_d);

	return genlist;
}


